<?php

defined( 'ABSPATH' ) or die;

$GLOBALS['processed_terms'] = array();
$GLOBALS['processed_posts'] = array();

require_once ABSPATH . 'wp-admin/includes/post.php';
require_once ABSPATH . 'wp-admin/includes/taxonomy.php';
require_once ABSPATH . 'wp-admin/includes/image.php';

/**
 * Add an Import Action, this data is stored for processing after import is done.
 *
 * Each action is sent as an Ajax request and is handled by themify-ajax.php file
 */ 
function themify_add_import_action( $action = '', $data = array() ) {
	global $import_actions;

	if ( ! isset( $import_actions[ $action ] ) ) {
		$import_actions[ $action ] = array();
	}

	$import_actions[ $action ][] = $data;
}

function themify_import_post( $post ) {
	global $processed_posts, $processed_terms;

	if ( ! post_type_exists( $post['post_type'] ) ) {
		return;
	}

	/* Menu items don't have reliable post_title, skip the post_exists check */
	if( $post['post_type'] !== 'nav_menu_item' ) {
		$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
		if ( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
			$processed_posts[ intval( $post['ID'] ) ] = intval( $post_exists );
			return;
		}
	}

	if( $post['post_type'] == 'nav_menu_item' ) {
		if( ! isset( $post['tax_input']['nav_menu'] ) || ! term_exists( $post['tax_input']['nav_menu'], 'nav_menu' ) ) {
			return;
		}
		$_menu_item_type = $post['meta_input']['_menu_item_type'];
		$_menu_item_object_id = $post['meta_input']['_menu_item_object_id'];

		if ( 'taxonomy' == $_menu_item_type && isset( $processed_terms[ intval( $_menu_item_object_id ) ] ) ) {
			$post['meta_input']['_menu_item_object_id'] = $processed_terms[ intval( $_menu_item_object_id ) ];
		} else if ( 'post_type' == $_menu_item_type && isset( $processed_posts[ intval( $_menu_item_object_id ) ] ) ) {
			$post['meta_input']['_menu_item_object_id'] = $processed_posts[ intval( $_menu_item_object_id ) ];
		} else if ( 'custom' != $_menu_item_type ) {
			// associated object is missing or not imported yet, we'll retry later
			// $missing_menu_items[] = $item;
			return;
		}
	}

	$post_parent = ( $post['post_type'] == 'nav_menu_item' ) ? $post['meta_input']['_menu_item_menu_item_parent'] : (int) $post['post_parent'];
	$post['post_parent'] = 0;
	if ( $post_parent ) {
		// if we already know the parent, map it to the new local ID
		if ( isset( $processed_posts[ $post_parent ] ) ) {
			if( $post['post_type'] == 'nav_menu_item' ) {
				$post['meta_input']['_menu_item_menu_item_parent'] = $processed_posts[ $post_parent ];
			} else {
				$post['post_parent'] = $processed_posts[ $post_parent ];
			}
		}
	}

	/**
	 * for hierarchical taxonomies, IDs must be used so wp_set_post_terms can function properly
	 * convert term slugs to IDs for hierarchical taxonomies
	 */
	if( ! empty( $post['tax_input'] ) ) {
		foreach( $post['tax_input'] as $tax => $terms ) {
			if( is_taxonomy_hierarchical( $tax ) ) {
				$terms = explode( ', ', $terms );
				$post['tax_input'][ $tax ] = array_map( 'themify_get_term_id_by_slug', $terms, array_fill( 0, count( $terms ), $tax ) );
			}
		}
	}

	$post['post_author'] = (int) get_current_user_id();
	$post['post_status'] = 'publish';

	$old_id = $post['ID'];

	unset( $post['ID'] );
	$post_id = wp_insert_post( $post, true );
	if( is_wp_error( $post_id ) ) {
		return false;
	} else {
		$processed_posts[ $old_id ] = $post_id;

		return $post_id;
	}
}

function themify_get_placeholder_image() {
	static $placeholder_image = null;

	if( $placeholder_image == null ) {
		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
		}
		WP_Filesystem();
		global $wp_filesystem;
		$upload = wp_upload_bits( $post['post_name'] . '.jpg', null, $wp_filesystem->get_contents( THEMIFY_DIR . '/img/image-placeholder.jpg' ) );

		if ( $info = wp_check_filetype( $upload['file'] ) )
			$post['post_mime_type'] = $info['type'];
		else
			return new WP_Error( 'attachment_processing_error', __( 'Invalid file type', 'themify' ) );

		$post['guid'] = $upload['url'];
		$post_id = wp_insert_attachment( $post, $upload['file'] );
		wp_update_attachment_metadata( $post_id, wp_generate_attachment_metadata( $post_id, $upload['file'] ) );

		$placeholder_image = $post_id;
	}

	return $placeholder_image;
}

function themify_import_term( $term ) {
	global $processed_terms;

	if( $term_id = term_exists( $term['slug'], $term['taxonomy'] ) ) {
		if ( is_array( $term_id ) ) $term_id = $term_id['term_id'];
		if ( isset( $term['term_id'] ) )
			$processed_terms[ intval( $term['term_id'] ) ] = (int) $term_id;
		return (int) $term_id;
	}

	if ( empty( $term['parent'] ) ) {
		$parent = 0;
	} else {
		$parent = term_exists( $processed_terms[ intval( $term['parent'] ) ], $term['taxonomy'] );
		if ( is_array( $parent ) ) $parent = $parent['term_id'];
	}

	$id = wp_insert_term( $term['name'], $term['taxonomy'], array(
		'parent' => $parent,
		'slug' => $term['slug'],
		'description' => $term['description'],
	) );
	if ( ! is_wp_error( $id ) ) {
		if ( isset( $term['term_id'] ) ) {
			// success!
			$processed_terms[ intval($term['term_id']) ] = $id['term_id'];
			if ( isset( $term['thumbnail'] ) ) {
				themify_add_import_action( 'term_thumb', array(
					'id' => $id['term_id'],
					'thumb' => $term['thumbnail'],
				) );
			}
			return $term['term_id'];
		}
	}

	return false;
}

function themify_get_term_id_by_slug( $slug, $tax ) {
	$term = get_term_by( 'slug', $slug, $tax );
	if( $term ) {
		return $term->term_id;
	}

	return false;
}

function themify_undo_import_term( $term ) {
	$term_id = term_exists( $term['slug'], $term['taxonomy'] );
	if ( $term_id ) {
		if ( is_array( $term_id ) ) $term_id = $term_id['term_id'];

		if ( $term_thumbnail = get_term_meta( $term['term_id'], 'thumbnail_id', true ) ) {
			wp_delete_attachment( $term_thumbnail, true );
		}

		if ( isset( $term_id ) ) {
			wp_delete_term( $term_id, $term['taxonomy'] );
		}
	}
}

/**
 * Determine if a post exists based on title, content, and date
 *
 * @global wpdb $wpdb WordPress database abstraction object.
 *
 * @param array $args array of database parameters to check
 * @return int Post ID if post exists, 0 otherwise.
 */
function themify_post_exists( $args = array() ) {
	global $wpdb;

	$query = "SELECT ID FROM $wpdb->posts WHERE 1=1";
	$db_args = array();

	foreach ( $args as $key => $value ) {
		$value = wp_unslash( sanitize_post_field( $key, $value, 0, 'db' ) );
		if( ! empty( $value ) ) {
			$query .= ' AND ' . $key . ' = %s';
			$db_args[] = $value;
		}
	}

	if ( !empty ( $args ) )
		return (int) $wpdb->get_var( $wpdb->prepare($query, $args) );

	return 0;
}

function themify_undo_import_post( $post ) {
	if( $post['post_type'] == 'nav_menu_item' ) {
		$post_exists = themify_post_exists( array(
			'post_name' => $post['post_name'],
			'post_modified' => $post['post_date'],
			'post_type' => 'nav_menu_item',
		) );
	} else {
		$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
	}
	if( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
		/**
		 * check if the post has been modified, if so leave it be
		 *
		 * NOTE: posts are imported using wp_insert_post() which modifies post_modified field
		 * to be the same as post_date, hence to check if the post has been modified,
		 * the post_modified field is compared against post_date in the original post.
		 */
		if( $post['post_date'] == get_post_field( 'post_modified', $post_exists ) ) {
			// find and remove all post attachments
			$attachments = get_posts( array(
				'post_type' => 'attachment',
				'posts_per_page' => -1,
				'post_parent' => $post_exists,
			) );
			if ( $attachments ) {
				foreach ( $attachments as $attachment ) {
					wp_delete_attachment( $attachment->ID, true );
				}
			}
			wp_delete_post( $post_exists, true ); // true: bypass trash
		}
	}
}

function themify_process_post_import( $post ) {
	if( ERASEDEMO ) {
		themify_undo_import_post( $post );
	} else {
		if ( $id = themify_import_post( $post ) ) {
			if ( defined( 'IMPORT_IMAGES' ) && ! IMPORT_IMAGES ) {
				/* if importing images is disabled and post is supposed to have a thumbnail, create a placeholder image instead */
				if ( isset( $post['thumb'] ) ) { // the post is supposed to have featured image
					$placeholder = themify_get_placeholder_image();
					if( ! is_wp_error( $placeholder ) ) {
						set_post_thumbnail( $id, $placeholder );
					}
				}
			} else {
				if ( isset( $post["thumb"] ) ) {
					themify_add_import_action( 'post_thumb', array(
						'id' => $id,
						'thumb' => $post["thumb"],
					) );
				}
				if ( isset( $post["gallery_shortcode"] ) ) {
					themify_add_import_action( 'gallery_field', array(
						'id' => $id,
						'fields' => $post["gallery_shortcode"],
					) );
				}
				if ( isset( $post["_product_image_gallery"] ) ) {
					themify_add_import_action( 'product_gallery', array(
						'id' => $id,
						'images' => $post["_product_image_gallery"],
					) );
				}
			}
		}
	}
}
$thumbs = array();
function themify_do_demo_import() {
global $import_actions;

	if ( isset( $GLOBALS["ThemifyBuilder_Data_Manager"] ) ) {
		remove_action( "save_post", array( $GLOBALS["ThemifyBuilder_Data_Manager"], "save_builder_text_only"), 10, 3 );
	}
$term = array (
  'term_id' => 16,
  'name' => 'Photography',
  'slug' => 'photography',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 17,
  'name' => 'Artists',
  'slug' => 'artists',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 18,
  'name' => 'Fashion',
  'slug' => 'fashion',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 19,
  'name' => 'Designers',
  'slug' => 'designers',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 20,
  'name' => 'Writers',
  'slug' => 'writers',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 21,
  'name' => 'Tourists',
  'slug' => 'tourists',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 36,
  'name' => 'Videos',
  'slug' => 'videos',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 13,
  'name' => 'Women',
  'slug' => 'women',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 14,
  'name' => 'Dresses',
  'slug' => 'dresses',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 13,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 22,
  'name' => 'Jumpers',
  'slug' => 'jumpers',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 13,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 26,
  'name' => 'Shirts &amp; Tops',
  'slug' => 'shirts-tops',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 13,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 27,
  'name' => 'Accessories',
  'slug' => 'accessories',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 28,
  'name' => 'Men',
  'slug' => 'men',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 29,
  'name' => 'Long Sleeve T-Shirts',
  'slug' => 'long-sleeve-t-shirts',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 28,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 30,
  'name' => 'Shirts',
  'slug' => 'shirts',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 28,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 31,
  'name' => 'Suits &amp; Blazers',
  'slug' => 'suits-blazers',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 28,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 32,
  'name' => 'T-Shirts',
  'slug' => 't-shirts',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 28,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 33,
  'name' => 'Accessories',
  'slug' => 'accessories-men',
  'term_group' => 0,
  'taxonomy' => 'product_cat',
  'description' => '',
  'parent' => 28,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 15,
  'name' => 'Red',
  'slug' => 'red',
  'term_group' => 0,
  'taxonomy' => 'product_tag',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 34,
  'name' => 'Team',
  'slug' => 'team',
  'term_group' => 0,
  'taxonomy' => 'team-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 35,
  'name' => 'Executive',
  'slug' => 'executive',
  'term_group' => 0,
  'taxonomy' => 'team-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 23,
  'name' => 'Photo',
  'slug' => 'photo',
  'term_group' => 0,
  'taxonomy' => 'portfolio-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 25,
  'name' => 'Featured',
  'slug' => 'featured',
  'term_group' => 0,
  'taxonomy' => 'portfolio-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 40,
  'name' => 'Fashion',
  'slug' => 'fashion',
  'term_group' => 0,
  'taxonomy' => 'portfolio-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 41,
  'name' => 'Designers',
  'slug' => 'designers',
  'term_group' => 0,
  'taxonomy' => 'portfolio-category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 37,
  'name' => 'Test',
  'slug' => 'test',
  'term_group' => 0,
  'taxonomy' => 'nav_menu',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 52,
  'name' => 'Main Navigation',
  'slug' => 'main-navigation',
  'term_group' => 0,
  'taxonomy' => 'nav_menu',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$post = array (
  'ID' => 2451,
  'post_date' => '2014-01-16 17:06:26',
  'post_date_gmt' => '2014-01-16 17:06:26',
  'post_content' => 'Aliquam finibus ante leo. Vestibulum et odio erat. Sed elementum sapien quis porta lobortis. Proin sed imperdiet arcu, eget tincidunt dui. Aenean tristique mollis neque a pretium. Nulla mattis, metus non interdum semper, tortor diam pharetra dolor, non auctor tellus risus non enim. Donec pulvinar, purus ut laoreet sollicitudin, mi sapien eleifend turpis, eu ultricies orci ipsum ornare augue. Donec aliquet nisl vel ex blandit luctus. Curabitur consectetur velit eget est ullamcorper commodo. Proin leo velit, hendrerit sit amet tellus a, tempus bibendum diam. Integer ut ligula eros.

Lorem ipsum dolor sit amet, consectetur adipiscing elit. Fusce interdum sem quis nunc sollicitudin, at finibus nunc gravida. Pellentesque euismod urna eu lacus vehicula, faucibus vehicula turpis pharetra. Pellentesque nisi erat, elementum bibendum eleifend non, malesuada sed nulla. In vitae velit non purus efficitur dictum. Proin at mi luctus, tristique diam vel, porttitor metus. Sed eget ligula sed nisl sollicitudin imperdiet id nec nisl. Proin mollis orci odio, ut faucibus dolor semper vestibulum. Quisque vehicula, enim sed vulputate tristique, risus leo posuere urna, id sagittis nunc massa at mauris.

Integer est nisi, commodo ac quam congue, finibus viverra magna. Mauris sit amet nulla ut nulla consequat tristique dictum sit amet purus. Quisque nec dolor a tortor fringilla viverra. Quisque elementum facilisis mauris fringilla luctus. Fusce convallis sed elit vitae accumsan. Vivamus placerat nisi nec elementum tristique. Nulla facilisi. Vivamus nibh urna, imperdiet in enim eget, maximus vulputate tortor. Donec vitae lobortis tortor. Cras in ante ac tortor pharetra semper. Pellentesque posuere eget risus nec pharetra.

<!--more-->
<h3>Dreaming Big...</h3>
Donec vitae lobortis tortor. Cras in ante ac tortor pharetra semper. Pellentesque posuere eget risus nec pharetra. Wuisque elementum facilisis mauris fringilla luctus. Fusce convallis sed elit vitae accumsan. Vivamus placerat nisi nec elementum tristique. Nulla facilisi. Vivamus nibh urna, imperdiet in enim eget, maximus vulputate tortor. Donec vitae lobortis tortor. Cras in ante ac tortor pharetra semper. Pellentesque posuere eget risus nec pharetra.

Phasellus fringilla in enim vel rutrum. Etiam a neque accumsan, scelerisque turpis vitae, venenatis lorem. Nullam eget tortor imperdiet, lacinia libero id, malesuada nibh. Donec at turpis sit amet dui vestibulum elementum. Vivamus ac eleifend felis. Maecenas eu ligula tincidunt, sagittis velit ac, consectetur metus. Etiam consectetur pretium lectus, ut malesuada elit blandit et. Aenean sit amet imperdiet neque, ac sodales elit. Morbi a metus bibendum quam imperdiet blandit.',
  'post_title' => 'Bring it to Hollywood',
  'post_excerpt' => '',
  'post_name' => 'bring-hollywood',
  'post_modified' => '2017-08-21 05:48:11',
  'post_modified_gmt' => '2017-08-21 05:48:11',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2451',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'fashion, tourists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/01/53654263.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2455,
  'post_date' => '2014-10-16 17:44:19',
  'post_date_gmt' => '2014-10-16 17:44:19',
  'post_content' => 'Tuspendisse aliquet ex vel purus tristique, vel accumsan massa faucibus. Mauris at leo in mi pretium eleifend. In sagittis in enim eu fermentum. Etiam ornare tempor volutpat. Pellentesque dictum id ex non convallis. Pellentesque varius tellus ac lacus lacinia, quis sagittis nisl commodo. Nunc pulvinar felis a odio condimentum convallis. Curabitur auctor tortor in placerat dapibus. Vivamus consectetur facilisis est at euismod. Cras quis justo vel sapien porta semper.

In pellentesque placerat nunc a vulputate. Sed feugiat vitae leo et ullamcorper. Maecenas vitae risus eget quam feugiat feugiat. Donec orci urna, varius vehicula eleifend id, feugiat at quam. Pellentesque pulvinar neque eu eros ornare, sit amet egestas tellus mattis. Curabitur id euismod lorem, non efficitur nibh. Phasellus eleifend malesuada nisl, vitae lobortis tellus hendrerit nec. Pellentesque accumsan nulla vel faucibus euismod. Donec facilisis est id sapien maximus, et porta dui pulvinar. Sed volutpat pulvinar risus, quis egestas orci luctus et. Maecenas turpis arcu, iaculis vel lacinia in, pharetra eu augue. Donec id scelerisque velit. Curabitur sit amet tincidunt ipsum. Nunc in ligula purus.

<!--more-->
<h2>Beauty that brings me to my knees.</h2>
Nullam ac turpis ac neque blandit dignissim vel quis lectus. Fusce aliquet ipsum sed lacus mollis aliquet. Sed in dolor ullamcorper, malesuada odio sit amet, posuere lacus. In hac habitasse platea dictumst. Pellentesque blandit ullamcorper massa, nec sodales dolor viverra eget. Maecenas elementum ante quis magna semper feugiat. Suspendisse pellentesque pretium lorem, id cursus elit vehicula tincidunt. Quisque gravida, nunc a sagittis eleifend, magna turpis mattis lorem, hendrerit pharetra felis nunc rutrum mi. Fusce venenatis dictum semper. Aliquam fermentum facilisis lectus, vitae condimentum diam eleifend at. Phasellus ut dolor elementum, consequat neque a, molestie diam.

Integer vel tortor ac tellus feugiat posuere. Praesent sed urna sed mauris interdum consequat pulvinar quis felis. Vestibulum in urna sagittis, mollis sapien id, rhoncus neque. Etiam ex augue, euismod quis leo nec, dignissim laoreet metus. Sed sit amet libero ac dolor pretium egestas. Mauris urna turpis, ultricies eget convallis eu, porttitor lacinia lorem. Donec efficitur purus dui, a pulvinar nulla egestas eu. Praesent vel mauris eget nibh suscipit consequat. Nulla non lorem tortor. Sed pulvinar tristique justo, sed malesuada nisl rhoncus a. Proin dapibus ullamcorper est, ac porttitor risus tincidunt id. Quisque porttitor faucibus nunc in posuere.',
  'post_title' => 'A Slave to Beauty',
  'post_excerpt' => '',
  'post_name' => 'slave-beauty',
  'post_modified' => '2017-08-21 05:47:54',
  'post_modified_gmt' => '2017-08-21 05:47:54',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2455',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'designers, photography',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/176244302.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2458,
  'post_date' => '2014-10-16 17:15:31',
  'post_date_gmt' => '2014-10-16 17:15:31',
  'post_content' => 'Vivamus ipsum lacus, malesuada ac euismod a, convallis sed risus. Phasellus a ligula sed turpis tincidunt gravida. Duis nec leo dolor. Mauris dignissim aliquam urna. Curabitur tempor posuere porta. Sed ultrices enim non lectus pulvinar dapibus at sit amet lacus. Aliquam erat volutpat. Curabitur eros sem, bibendum at leo ac, laoreet congue erat. Nullam condimentum erat id nulla lobortis facilisis. Aliquam erat volutpat. Etiam pulvinar suscipit risus vel auctor. Proin dignissim dui purus, et auctor metus facilisis vitae. Ut sagittis nunc vitae sapien imperdiet sagittis. Integer vel pellentesque felis, id malesuada dolor. Morbi risus ex, hendrerit a mauris ut, tristique malesuada turpis.

In faucibus magna purus, id dictum turpis cursus cursus. Proin eget orci faucibus diam rhoncus sollicitudin eu sit amet urna. Proin facilisis sollicitudin rutrum. Nunc consectetur sagittis nulla et condimentum. Donec sollicitudin gravida nisl at mollis. Integer ut maximus augue. Vivamus aliquam imperdiet tortor, vitae vehicula turpis. Proin faucibus congue quam, at congue nibh scelerisque at. In molestie leo id mattis gravida.

Mauris et luctus purus. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Ut pulvinar, orci et lacinia blandit, arcu libero mollis massa, ut elementum dolor ex at massa. Duis enim dolor, auctor a odio vitae, cursus semper mauris. Praesent faucibus, magna non auctor vehicula, tellus felis accumsan felis, iaculis consequat urna libero eget lectus. Nam sed mattis ipsum. Nam et lorem vitae lorem eleifend euismod. Nunc non sem turpis. In euismod lectus vitae tellus laoreet, tempor dapibus turpis pellentesque. Etiam sed sem scelerisque, accumsan elit at, ultricies nulla. Curabitur mollis euismod bibendum. Quisque sit amet risus eget leo ultrices accumsan. Aliquam erat volutpat.

<!--more-->
<h3>Looking into the future I see a new beginning...</h3>
Aenean dui dolor, accumsan at commodo eget, rutrum et nisi. Duis sollicitudin, tellus sed sodales ultrices, lorem metus placerat risus, ut luctus sem velit sit amet lorem. Etiam nec justo et dui varius hendrerit. Quisque ipsum quam, dictum in congue ut, vehicula ac leo. Integer hendrerit tellus velit, vel eleifend orci pulvinar vel. Ut ultricies, enim at maximus consequat, elit purus interdum neque, pretium feugiat dui augue non mi. Sed vel arcu nisl. Etiam vitae sapien non nisi dapibus viverra. Sed volutpat imperdiet eros, ut lobortis quam consectetur a. Pellentesque suscipit diam ut enim ultricies maximus. Etiam blandit dignissim elementum. Aenean nec urna non urna tincidunt egestas vel sit amet purus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Curabitur massa augue, fermentum at tellus nec, euismod vehicula est. Nullam venenatis consectetur ultricies. Lorem ipsum dolor sit amet, consectetur adipiscing elit.

Quisque vel tellus volutpat, accumsan metus vel, varius augue. Sed viverra eleifend tortor. Donec dictum leo id dui interdum, ac convallis risus accumsan. Morbi ac nisi dui. Fusce eu sapien sit amet velit mollis iaculis. Aliquam suscipit urna rutrum nunc accumsan, vel vehicula nisi lobortis. Nulla molestie justo sit amet efficitur venenatis. Sed hendrerit efficitur tellus et dictum. Etiam auctor orci eget dui dapibus consequat. Morbi porttitor ante nec augue commodo, vel vestibulum nunc blandit. Praesent consectetur vel purus vitae mattis.',
  'post_title' => 'Gaze to the Horizon',
  'post_excerpt' => '',
  'post_name' => 'gaze-horizon',
  'post_modified' => '2017-08-21 05:47:56',
  'post_modified_gmt' => '2017-08-21 05:47:56',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2458',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'photography',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/172002317.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2463,
  'post_date' => '2014-10-16 19:59:09',
  'post_date_gmt' => '2014-10-16 19:59:09',
  'post_content' => 'Left alignment has beautiful drop cap. Lorem ispum mi velit, efficitur a mollis vulputate, rhoncus egestas nisl. Fusce mauris quam, convallis eu lacus id, rutrum bibendum nisi. Quisque sit amet dictum ex, sed efficitur ex. Nullam augue ligula, condimentum in blandit sed, imperdiet sed sem. Pellentesque orci tortor, fermentum a risus id, aliquam bibendum lectus. Maecenas non elit fermentum, congue mauris non, porttitor libero. Aenean sit amet mauris quam. Suspendisse cursus pulvinar lectus, at sagittis diam imperdiet a. Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Quisque quis magna risus. Donec a placerat lacus. Integer sed ipsum faucibus, varius velit sed, pulvinar elit. Nullam eu turpis libero. Aenean accumsan et purus nec fringilla. Praesent consectetur ut neque a sagittis. Integer sed mi lorem.

Pellentesque eu eleifend ex. Sed vestibulum iaculis elit quis condimentum. Vestibulum ac porttitor arcu. Sed sollicitudin congue purus in mattis. In hac habitasse platea dictumst. Maecenas porttitor orci non porta posuere. Aliquam viverra dui sit amet faucibus lacinia. Suspendisse potenti. Proin sed porttitor velit.

<!--more-->
<h2>Sub Heading</h2>
Phasellus tempus tincidunt varius. Nullam luctus velit lorem, rhoncus interdum sapien sollicitudin quis. Proin sit amet diam euismod, consequat massa eget, ullamcorper turpis. Donec eu risus tellus. Proin in tempus sapien, sit amet fermentum justo. Maecenas ac enim et ligula feugiat pharetra ac bibendum dolor. Mauris laoreet malesuada nisl ut efficitur. Aliquam ut sem ex. Duis tempus, mi id rutrum vehicula, eros orci ultricies turpis, et scelerisque felis est id felis. Aenean sed risus pharetra, eleifend diam vitae, laoreet enim. Cras libero erat, finibus non imperdiet ut, lacinia ac dolor. Vestibulum ligula lacus, condimentum nec cursus lacinia, posuere vitae est. Aenean ipsum risus, lacinia id maximus nec, congue sit amet mi.

Aenean semper laoreet placerat. Proin eleifend lacus est, ut laoreet arcu sollicitudin sit amet. Cras sed est non est luctus sollicitudin. Proin ex elit, feugiat vel nibh eget, sagittis iaculis arcu. In in ante et purus dictum vestibulum tincidunt id diam. Nam dolor arcu, semper sed venenatis vel, mollis id ipsum. Suspendisse vel porttitor risus. Integer sagittis felis at sollicitudin fringilla. Maecenas eget pellentesque risus. Integer placerat sodales vehicula. Quisque non neque id felis feugiat egestas sit amet ut nisl. Aliquam scelerisque fermentum enim euismod viverra. Aenean facilisis, augue ultrices blandit euismod, elit lacus euismod ex, eget tincidunt dui enim a orci. Donec nec imperdiet tortor, quis feugiat erat. Donec euismod mi et varius posuere.',
  'post_title' => 'Left Aligned',
  'post_excerpt' => '',
  'post_name' => 'left-aligned',
  'post_modified' => '2017-08-21 05:47:21',
  'post_modified_gmt' => '2017-08-21 05:47:21',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2463',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'photography',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/192995783.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2489,
  'post_date' => '2014-10-16 19:14:24',
  'post_date_gmt' => '2014-10-16 19:14:24',
  'post_content' => '<div id="lipsum">

Vestibulum posuere ex eget pulvinar pellentesque. Suspendisse at nisl posuere, tempus dolor non, mattis nulla. Nam ut justo eget nunc auctor posuere. Aliquam porttitor lacus id ipsum mattis, eu fermentum tellus dapibus. Donec a purus ac augue pellentesque ultrices ac sed justo. Integer convallis lacus in est elementum, ac varius nibh tempor. Aliquam faucibus iaculis posuere. Nullam malesuada luctus quam et ullamcorper. Praesent in augue posuere, ultricies metus in, placerat velit.

Integer non viverra orci, eget placerat sapien. Duis sed lacus sit amet lacus elementum efficitur. Suspendisse hendrerit ligula magna, eu auctor nulla placerat at. Cras condimentum dictum pellentesque. Aenean luctus ipsum tellus. In in tortor consequat, cursus justo vel, molestie nibh. Morbi vel lobortis leo. Phasellus mi ex, laoreet sed posuere vitae, finibus non tortor. In hac habitasse platea dictumst. Cras eget odio at nunc vehicula auctor at vitae sem. Sed vulputate ipsum ac leo rhoncus molestie. Cras scelerisque mollis velit in lacinia. Donec maximus lacus non gravida lobortis. Fusce et nunc mattis risus rutrum convallis sed sed lacus. Nullam quis pretium velit.

</div>
<!--more-->

Pellentesque pulvinar turpis vel sapien ornare, eu tempus erat vestibulum. Mauris sit amet nibh vel odio tempor viverra. Maecenas euismod metus a velit facilisis bibendum. Aliquam molestie venenatis ipsum ut venenatis. Nam eleifend velit lectus, quis tempor justo lobortis sed. Curabitur imperdiet orci sit amet felis facilisis, lacinia pulvinar massa pulvinar. Maecenas facilisis enim tortor, ut aliquet velit pellentesque a.

Proin eu euismod lectus. Nam vulputate lorem a orci consectetur, non rutrum erat vehicula. Nullam non orci sed erat consectetur molestie id sit amet velit. Fusce tempor imperdiet nisl, et imperdiet nibh rhoncus sed. Duis mattis tempus nulla sit amet convallis. Nulla imperdiet turpis justo, at egestas orci lacinia eget. Sed ullamcorper felis ullamcorper commodo posuere. Cras imperdiet ipsum mauris, vitae varius ante posuere vitae. Aenean efficitur elit ligula, eu molestie nulla venenatis sit amet. Nam efficitur magna odio, iaculis fringilla elit mollis in.

&nbsp;',
  'post_title' => 'Off to the Next Adverture',
  'post_excerpt' => '',
  'post_name' => 'next-adverture',
  'post_modified' => '2017-08-21 05:47:52',
  'post_modified_gmt' => '2017-08-21 05:47:52',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2489',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'tourists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/174280970.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2492,
  'post_date' => '2014-10-16 19:16:19',
  'post_date_gmt' => '2014-10-16 19:16:19',
  'post_content' => '<div id="lipsum">

Curabitur auctor elit sem, eu semper sapien rhoncus non. Suspendisse ut suscipit lorem. Nulla molestie pharetra risus, nec iaculis turpis semper quis. Duis iaculis hendrerit dolor. Ut tincidunt ipsum aliquam purus fringilla tempus. Nullam blandit varius enim at placerat. Quisque congue ornare porttitor. In dignissim molestie justo at tempus. Phasellus dapibus congue dictum.

Donec hendrerit, augue non vestibulum tincidunt, nisi tortor venenatis sapien, nec auctor quam purus vitae nisl. Ut velit turpis, bibendum sed convallis eget, facilisis id tortor. Aliquam aliquam auctor sem, nec porttitor nisi feugiat ac. Vivamus eleifend justo a tellus commodo, auctor auctor quam dignissim. Nullam quis sollicitudin justo, quis varius nunc. Sed tempor mattis metus in finibus. Curabitur eget viverra mauris. Morbi porttitor, mauris vel tempus sagittis, dolor eros tincidunt massa, et iaculis enim ante nec purus. Nam sed enim at odio fermentum tempus vitae nec urna. Curabitur varius elit a arcu cursus, non scelerisque orci accumsan.

Aliquam ac nunc mi. Quisque mollis nibh in aliquet scelerisque. Curabitur ornare cursus magna. Maecenas venenatis eros non felis fermentum sagittis. Fusce vestibulum vestibulum lacus, a viverra lorem laoreet eu. Ut posuere ullamcorper urna, vel vehicula felis tincidunt euismod. In tristique malesuada dui vitae vestibulum. Aenean bibendum, nibh ut ultricies ultricies, ex metus fermentum purus, ut tincidunt turpis metus id augue. Morbi commodo erat sit amet felis elementum pharetra. Nulla quis condimentum purus, vitae scelerisque elit. Morbi suscipit, elit a pulvinar placerat, tellus leo ultricies enim, id malesuada augue nibh a elit. Nullam et efficitur magna. Ut in tortor faucibus, aliquet dolor varius, imperdiet metus. Integer vulputate felis sit amet dapibus pretium. Mauris et eros orci. Donec molestie vehicula turpis finibus lacinia.

</div>
<!--more-->

Sed porta laoreet nulla, sed fringilla dui euismod aliquam. Quisque nisi tellus, sollicitudin imperdiet diam at, posuere porta eros. Duis consequat varius quam. Suspendisse a porta enim. Morbi euismod tincidunt pretium. Nam molestie scelerisque justo, non scelerisque nulla ullamcorper et. Ut non nulla placerat, venenatis magna nec, gravida libero. Maecenas id erat nisi. Vivamus enim enim, cursus a quam id, facilisis luctus orci. Nullam maximus sapien sed mi convallis pretium. Proin eget dolor porttitor, gravida lacus id, vestibulum ante. Phasellus tincidunt ex nec accumsan pellentesque. Aliquam erat volutpat. Donec aliquam, odio eu lobortis dictum, libero orci mollis leo, quis interdum nisi ligula at magna. Donec pulvinar aliquam nunc, sed luctus mauris commodo id.

Nam condimentum interdum mi eget auctor. Pellentesque et nulla elementum, molestie ligula eu, vestibulum mi. Integer consectetur tellus sit amet diam elementum fringilla. Duis pretium urna pellentesque libero congue luctus. Vivamus suscipit nisi justo. Vivamus rutrum porta dolor, eget gravida sem gravida non. Nulla interdum commodo nibh in dignissim. Nullam feugiat, ex ut auctor convallis, lorem mauris hendrerit risus, eu convallis lectus velit nec massa.

&nbsp;',
  'post_title' => 'Dance to the Beat',
  'post_excerpt' => '',
  'post_name' => 'dance-beat',
  'post_modified' => '2017-08-21 05:47:45',
  'post_modified_gmt' => '2017-08-21 05:47:45',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2492',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'artists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/181161062.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2495,
  'post_date' => '2014-10-16 22:19:19',
  'post_date_gmt' => '2014-10-16 22:19:19',
  'post_content' => '<!--themify_builder_static--><h1 style="text-align: left;">Hello Elegant</h1><h3 style="text-align: left;">Story post built using the Themify Builder</h3> 
 <h3 style="text-align: left;">With Themify Builder, you can literally design any type of layouts using the drag &amp; drop interface.</h3><p>Etiam volutpat odio sem, vel suscipit nibh posuere sed. Curabitur gravida leo id est gravida, sed egestas purus elementum. Etiam et fringilla metus. Sed eget orci nunc. Suspendisse potenti. Maecenas luctus imperdiet ante sit amet gravida. Vivamus auctor felis et dui blandit, a commodo est aliquam. Ut dignissim eu quam id ultrices. Morbi ac neque non quam auctor fringilla. Sed volutpat justo efficitur pharetra sagittis.</p><p style="text-align: left;">Fusce sapien eros, tempor vel scelerisque rutrum, luctus a sem. Etiam condimentum condimentum sodales. Nunc sit amet diam erat. Nam a lectus in nibh rutrum euismod. Curabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.</p><p style="text-align: left;"> </p> 
 <h3>You can insert text, parallax scrolling background, video background, videos, galleries. All within your control. The Styling panel allows you take full control of the appearance.</h3> <p>Fusce sapien eros, tempor vel scelerisque rutrum, luctus a sem. Etiam condimentum condimentum sodales. Nunc sit amet diam erat. Nam a lectus in nibh rutrum euismod. Curabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.</p> <p>Furabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.</p> 
 <h4 style="text-align: left;">Sub Heading</h4><p style="text-align: left;">Mauris ut ornare enim. In porta libero arcu, ut aliquam ante posuere ultrices. Vestibulum non posuere metus. Morbi velit sem, gravida nec quam eget, accumsan posuere urna. Vestibulum in velit ac quam suscipit blandit sit amet tempor lectus.</p> 
 <h4>Credits</h4><p>This photo is a stock image</p> 
 <p>Are you trying to read me? Integer in arcu tortor. Praesent volutpat pellentesque mi, a semper quam posuere ut. Fusce urna felis, condimentum ac lectus vel, lacinia euismod erat. Vivamus vehicula euismod felis, sed rhoncus quam. Sed risus ipsum, ultrices ac hendrerit ut, finibus a nisl. Nunc scelerisque, nibh et porttitor viverra, ante dui pellentesque sapien, sit amet sollicitudin risus mauris in diam. In hac habitasse platea dictumst. Nam vehicula libero neque, at vestibulum eros tincidunt ut. Pellentesque bibendum elementum posuere.</p><p>Aliquam sed est vitae ex viverra venenatis. Duis quis aliquet justo, vitae dapibus libero. Vestibulum varius purus sed vehicula condimentum. Sed at maximus lorem. Morbi in lacinia nibh. Donec quis ante vitae odio varius viverra eget ac arcu. Proin pretium rhoncus vehicula. Etiam vel nunc nulla. Vivamus pellentesque viverra iaculis. Ut elementum urna sed metus mattis ultricies. Vivamus eu nisl pretium, maximus lacus eu, dapibus enim. Quisque mattis porttitor vulputate. Quisque congue vel est nec lacinia. Cras ornare ipsum in dui dignissim, feugiat luctus nisi sagittis.</p> 
 <img src="https://themify.me/demo/themes/elegant/files/2014/10/1988883051-900x500.jpg" width="900" height="500" alt="198888305" /> 
[gallery _orderbyfield="menu_order ID" link="file" _orderByField="menu_order ID" ids="3130,3141,3030,3092,3075,3079"]
 <h2>Blooming</h2><p>Still try to read lorum ispum? Quisque vitae facilisis nunc. Maecenas sed aliquet turpis. Integer tincidunt at tellus id accumsan. Nunc consectetur, velit eu consequat elementum.</p> 
 <h1>Finally</h1><p>Nulla ullamcorper vel magna non gravida. Quisque vitae facilisis nunc. Maecenas sed aliquet turpis. Integer tincidunt at tellus id accumsan. Nunc consectetur, velit eu consequat elementum.</p> 
 <iframe src="https://player.vimeo.com/video/26017275?dnt=1&app_id=122963" width="576" height="320" title="Mulberry FW 2011 Ad Campaign Video" allow="autoplay; fullscreen" allowfullscreen></iframe><!--/themify_builder_static-->',
  'post_title' => 'Hello Elegant',
  'post_excerpt' => '',
  'post_name' => 'hello-elegant',
  'post_modified' => '2019-04-08 20:16:57',
  'post_modified_gmt' => '2019-04-08 20:16:57',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2495',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'full_width',
    'layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'hide_post_title' => 'yes',
    'hide_meta_all' => 'yes',
    'hide_post_date' => 'yes',
    'hide_post_image' => 'yes',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"element_id\\":\\"3978763\\",\\"row_order\\":\\"0\\",\\"cols\\":[{\\"element_id\\":\\"19c8820\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"3f1e031\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Hello Elegant<\\\\/h1><h3 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Story post built using the Themify Builder<\\\\/h3>\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"1.4\\",\\"font_size_unit\\":\\"em\\",\\"padding_left\\":\\"2\\",\\"padding_left_unit\\":\\"%\\"}}]},{\\"element_id\\":\\"b78e7d2\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\"}],\\"styling\\":{\\"row_height\\":\\"fullheight\\",\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1341195711.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\"}},{\\"element_id\\":\\"cd464b2\\",\\"row_order\\":\\"1\\",\\"cols\\":[{\\"element_id\\":\\"18d2832\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1\\"},{\\"element_id\\":\\"1734c57\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"8887650\\",\\"mod_settings\\":{\\"content_text\\":\\"<h3 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">With Themify Builder, you can literally design any type of layouts using the drag &amp; drop interface.<\\\\/h3><p>Etiam volutpat odio sem, vel suscipit nibh posuere sed. Curabitur gravida leo id est gravida, sed egestas purus elementum. Etiam et fringilla metus. Sed eget orci nunc. Suspendisse potenti. Maecenas luctus imperdiet ante sit amet gravida. Vivamus auctor felis et dui blandit, a commodo est aliquam. Ut dignissim eu quam id ultrices. Morbi ac neque non quam auctor fringilla. Sed volutpat justo efficitur pharetra sagittis.<\\\\/p><p style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Fusce sapien eros, tempor vel scelerisque rutrum, luctus a sem. Etiam condimentum condimentum sodales. Nunc sit amet diam erat. Nam a lectus in nibh rutrum euismod. Curabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.<\\\\/p><p style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\"> <\\\\/p>\\"}}]},{\\"element_id\\":\\"8c53ea2\\",\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col4-1\\"}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"element_id\\":\\"2c017bb\\",\\"row_order\\":\\"2\\",\\"cols\\":[{\\"element_id\\":\\"99bec93\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-3\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"210d426\\",\\"mod_settings\\":{\\"background_image-type\\":\\"image\\",\\"background_repeat\\":\\"repeat\\",\\"background_position\\":\\"left-top\\",\\"font_color_type\\":\\"font_color_solid\\",\\"padding_right\\":\\"10\\",\\"padding_right_unit\\":\\"%\\",\\"checkbox_margin_apply_all\\":\\"1\\",\\"border-type\\":\\"top\\",\\"font_color_type_h1\\":\\"font_color_h1_solid\\",\\"font_color_type_h2\\":\\"font_color_h2_solid\\",\\"font_color_type_h3\\":\\"font_color_h3_solid\\",\\"font_color_type_h4\\":\\"font_color_h4_solid\\",\\"font_color_type_h5\\":\\"font_color_h5_solid\\",\\"font_color_type_h6\\":\\"font_color_h6_solid\\",\\"dropcap_background_color\\":\\"#ffffff_0.00\\",\\"dropcap_font_color\\":\\"#000000\\",\\"dropcap_font_size\\":\\"5.0\\",\\"dropcap_font_size_unit\\":\\"em\\",\\"dropcap_padding_top\\":\\"0\\",\\"checkbox_dropcap_padding_apply_all\\":\\"1\\",\\"dropcap_margin_top\\":\\"34\\",\\"dropcap_margin_right\\":\\"27\\",\\"dropcap_border-type\\":\\"top\\",\\"breakpoint_mobile\\":{\\"background_image-type\\":\\"image\\",\\"background_repeat\\":\\"repeat\\",\\"background_position\\":\\"left-top\\",\\"font_color_type\\":\\"font_color_solid\\",\\"padding_right_unit\\":\\"%\\",\\"checkbox_margin_apply_all\\":\\"1\\",\\"border-type\\":\\"top\\",\\"font_color_type_h1\\":\\"font_color_h1_solid\\",\\"font_color_type_h2\\":\\"font_color_h2_solid\\",\\"font_color_type_h3\\":\\"font_color_h3_solid\\",\\"font_color_type_h4\\":\\"font_color_h4_solid\\",\\"font_color_type_h5\\":\\"font_color_h5_solid\\",\\"font_color_type_h6\\":\\"font_color_h6_solid\\",\\"dropcap_background_color\\":\\"#ffffff_0\\",\\"dropcap_font_size_unit\\":\\"em\\",\\"checkbox_dropcap_padding_apply_all\\":\\"1\\",\\"dropcap_border-type\\":\\"top\\"},\\"content_text\\":\\"<h3>You can insert text, parallax scrolling background, video background, videos, galleries. All within your control. The Styling panel allows you take full control of the appearance.<\\\\/h3>\\\\n<p>Fusce sapien eros, tempor vel scelerisque rutrum, luctus a sem. Etiam condimentum condimentum sodales. Nunc sit amet diam erat. Nam a lectus in nibh rutrum euismod. Curabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.<\\\\/p>\\\\n<p>Furabitur sed odio est. Nam ut nibh pretium metus bibendum consequat. Nulla sit amet viverra enim, at pretium magna. Aenean eu volutpat orci. Donec sed laoreet quam, porta vulputate nunc. Sed nibh mauris, vestibulum in pellentesque non, efficitur vel ligula. Proin tincidunt finibus justo, ac sodales nisi finibus vel. Morbi arcu metus, dictum eu sapien vitae, ultrices commodo nulla.<\\\\/p>\\",\\"text_drop_cap\\":\\"dropcap\\",\\"stick_at_position\\":\\"top\\",\\"unstick_when_element\\":\\"builder_end\\",\\"unstick_when_el_row_id\\":\\"3978763\\",\\"unstick_when_el_mod_id\\":\\"3f1e031\\",\\"unstick_when_condition\\":\\"hits\\",\\"unstick_when_pos\\":\\"this\\"}}]},{\\"element_id\\":\\"b7b5901\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"8596c62\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Sub Heading<\\\\/h4><p style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Mauris ut ornare enim. In porta libero arcu, ut aliquam ante posuere ultrices. Vestibulum non posuere metus. Morbi velit sem, gravida nec quam eget, accumsan posuere urna. Vestibulum in velit ac quam suscipit blandit sit amet tempor lectus.<\\\\/p>\\",\\"padding_top\\":\\"10\\",\\"border_top_color\\":\\"000000_1.00\\",\\"border_top_width\\":\\"5\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"element_id\\":\\"43d4a94\\",\\"row_order\\":\\"3\\",\\"cols\\":[{\\"element_id\\":\\"d80f8dd\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"ec700b6\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Credits<\\\\/h4><p>This photo is a stock image<\\\\/p>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/171688262.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\",\\"font_color\\":\\"ffffff_1.00\\",\\"padding_top\\":\\"40\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"15\\"}},{\\"element_id\\":\\"6deca60\\",\\"row_order\\":\\"4\\",\\"cols\\":[{\\"element_id\\":\\"eb18ddb\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1\\"},{\\"element_id\\":\\"b03cf56\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"efd5a16\\",\\"mod_settings\\":{\\"content_text\\":\\"<p>Are you trying to read me? Integer in arcu tortor. Praesent volutpat pellentesque mi, a semper quam posuere ut. Fusce urna felis, condimentum ac lectus vel, lacinia euismod erat. Vivamus vehicula euismod felis, sed rhoncus quam. Sed risus ipsum, ultrices ac hendrerit ut, finibus a nisl. Nunc scelerisque, nibh et porttitor viverra, ante dui pellentesque sapien, sit amet sollicitudin risus mauris in diam. In hac habitasse platea dictumst. Nam vehicula libero neque, at vestibulum eros tincidunt ut. Pellentesque bibendum elementum posuere.<\\\\/p><p>Aliquam sed est vitae ex viverra venenatis. Duis quis aliquet justo, vitae dapibus libero. Vestibulum varius purus sed vehicula condimentum. Sed at maximus lorem. Morbi in lacinia nibh. Donec quis ante vitae odio varius viverra eget ac arcu. Proin pretium rhoncus vehicula. Etiam vel nunc nulla. Vivamus pellentesque viverra iaculis. Ut elementum urna sed metus mattis ultricies. Vivamus eu nisl pretium, maximus lacus eu, dapibus enim. Quisque mattis porttitor vulputate. Quisque congue vel est nec lacinia. Cras ornare ipsum in dui dignissim, feugiat luctus nisi sagittis.<\\\\/p>\\",\\"text_align\\":\\"left\\",\\"add_css_text\\":\\"drop-cap\\"}}]},{\\"element_id\\":\\"b04e79d\\",\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col4-1\\"}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"3\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"element_id\\":\\"943c377\\",\\"row_order\\":\\"5\\",\\"cols\\":[{\\"element_id\\":\\"89004ba\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"element_id\\":\\"6b5724a\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1988883051.jpg\\",\\"width_image\\":\\"900\\",\\"height_image\\":\\"500\\"}}]},{\\"element_id\\":\\"eb7bc21\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"gallery\\",\\"element_id\\":\\"8c352e1\\",\\"mod_settings\\":{\\"layout_gallery\\":\\"grid\\",\\"shortcode_gallery\\":\\"[gallery _orderbyfield=\\\\\\\\\\\\\\"menu_order ID\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"file\\\\\\\\\\\\\\" _orderByField=\\\\\\\\\\\\\\"menu_order ID\\\\\\\\\\\\\\" ids=\\\\\\\\\\\\\\"3130,3141,3030,3092,3075,3079\\\\\\\\\\\\\\"]\\",\\"thumb_w_gallery\\":\\"200\\",\\"thumb_h_gallery\\":\\"175\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"element_id\\":\\"ecf3e82\\",\\"row_order\\":\\"6\\",\\"cols\\":[{\\"element_id\\":\\"d4a4cd6\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-3\\"},{\\"element_id\\":\\"7f4c0c4\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"a217741\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2>Blooming<\\\\/h2><p>Still try to read lorum ispum? Quisque vitae facilisis nunc. Maecenas sed aliquet turpis. Integer tincidunt at tellus id accumsan. Nunc consectetur, velit eu consequat elementum.<\\\\/p>\\"}}]}],\\"styling\\":{\\"row_width\\":\\"fullwidth\\",\\"row_height\\":\\"fullheight\\",\\"background_type\\":\\"video\\",\\"background_video\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/timelapse_flower.mp4\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/timelapse_flower-1024x576.jpg\\",\\"background_color\\":\\"29c2ff_1.00\\",\\"font_color\\":\\"ffffff_1.00\\"}},{\\"element_id\\":\\"79f8740\\",\\"row_order\\":\\"7\\",\\"cols\\":[{\\"element_id\\":\\"1752ed9\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"element_id\\":\\"29e3cd3\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Finally<\\\\/h1><p>Nulla ullamcorper vel magna non gravida. Quisque vitae facilisis nunc. Maecenas sed aliquet turpis. Integer tincidunt at tellus id accumsan. Nunc consectetur, velit eu consequat elementum.<\\\\/p>\\",\\"margin_top\\":\\"15\\",\\"border_top_color\\":\\"000000_1.00\\",\\"border_top_width\\":\\"5\\"}}]},{\\"element_id\\":\\"4f834ce\\",\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\"}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\"}},{\\"element_id\\":\\"b0aba64\\",\\"row_order\\":\\"8\\",\\"cols\\":[{\\"element_id\\":\\"4400584\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"video\\",\\"element_id\\":\\"21ac5e5\\",\\"mod_settings\\":{\\"style_video\\":\\"video-top\\",\\"url_video\\":\\"http:\\\\/\\\\/vimeo.com\\\\/26017275\\"}}]}]},{\\"element_id\\":\\"ac48d6c\\",\\"row_order\\":\\"9\\",\\"cols\\":[{\\"element_id\\":\\"f548692\\",\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\"}]}]',
  ),
  'tax_input' => 
  array (
    'category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1341195711.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2500,
  'post_date' => '2014-10-16 19:57:45',
  'post_date_gmt' => '2014-10-16 19:57:45',
  'post_content' => 'Phasellus ac venenatis lectus, non scelerisque nunc. Cras nisi massa, accumsan sed magna at, rhoncus euismod risus. Pellentesque viverra, orci auctor ultricies finibus, mauris massa auctor ante, vel rhoncus magna tortor eget tortor. Pellentesque mollis nibh non elit semper scelerisque. Pellentesque purus mauris, luctus interdum dictum ut, bibendum fringilla enim. Sed pretium turpis ac arcu venenatis, vel sagittis quam congue. Proin aliquam felis vitae odio aliquam eleifend. Praesent quis nulla nibh. Fusce bibendum sem at odio commodo dignissim. Sed est metus, iaculis id mattis a, iaculis eget mauris. Vivamus vestibulum commodo elit, eget venenatis nisl mollis a. Nunc non condimentum urna. Curabitur lacinia dapibus arcu. Aenean turpis nulla, molestie nec viverra id, lacinia eget nulla. Morbi sed volutpat sapien, vel aliquam eros. Cras ut felis non massa placerat auctor ut ac nisl. Duis tempor eleifend libero, non tincidunt sapien hendrerit non. Donec eu aliquam lorem, commodo faucibus leo. Fusce tempus in erat id iaculis. Praesent eget risus laoreet, interdum leo non, gravida neque.

<!--more-->

Nunc ac justo bibendum, suscipit orci non, ultrices orci. Sed varius ipsum et neque pellentesque aliquet. Donec egestas purus neque, eget aliquam mi porttitor vel. Ut in pretium ante, quis bibendum arcu. Nulla vestibulum maximus lorem quis ornare. Morbi eget vehicula dui. In sit amet tortor id lorem iaculis dapibus nec eget magna. Duis et elit non eros hendrerit vestibulum. Donec quis tincidunt enim. Nam quis dictum magna.

Quisque a ante a ex semper tincidunt sit amet eu felis. Proin facilisis quam vitae est egestas lacinia. Cras faucibus orci ut augue tempus facilisis. Sed auctor ultrices purus eu rutrum. In sodales justo sit amet lectus imperdiet varius. Quisque nec sem nisl. Nullam nec egestas magna.

Integer vitae dictum nunc. Proin egestas, elit sed molestie posuere, purus mi rhoncus arcu, sed gravida nibh est id arcu. Maecenas eget suscipit leo, in pulvinar leo. Nulla id leo feugiat, hendrerit orci vel, consectetur ex. Duis rutrum lectus ante, vel consectetur urna pharetra sit amet. Suspendisse gravida ante arcu, sed cursus dolor auctor a. Vestibulum nulla mi, rhoncus a venenatis non, malesuada at arcu. Pellentesque fringilla eleifend urna. Aliquam tincidunt vitae dui tristique convallis. Fusce ac eros volutpat lorem congue condimentum. Phasellus placerat augue vel nisi pellentesque consectetur. Mauris volutpat metus sed nibh pellentesque iaculis. Vivamus neque velit, imperdiet et diam nec, commodo lacinia sem. In ornare justo a est volutpat, suscipit venenatis erat tempor. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Praesent suscipit mollis mi nec fringilla.

&nbsp;',
  'post_title' => 'Writers Inspiration',
  'post_excerpt' => '',
  'post_name' => 'writers-inspiration',
  'post_modified' => '2017-08-21 05:47:29',
  'post_modified_gmt' => '2017-08-21 05:47:29',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2500',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'writers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/207036016.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2505,
  'post_date' => '2014-04-16 19:23:23',
  'post_date_gmt' => '2014-04-16 19:23:23',
  'post_content' => 'Quisque at turpis nec lacus egestas sollicitudin eget a est. Cras accumsan molestie lectus, ac venenatis magna facilisis vitae. Suspendisse potenti. Donec sit amet consequat augue. Integer sollicitudin, lectus sed luctus varius, dolor elit pretium nunc, sed efficitur arcu ipsum ac nibh. Aenean vitae volutpat urna, et vestibulum urna. Curabitur ac neque sit amet lorem imperdiet dapibus finibus a augue. Mauris semper libero et nibh auctor, ut condimentum felis pharetra. Etiam vitae fermentum magna. Praesent egestas, turpis ut consectetur auctor, urna est efficitur justo, at dapibus neque massa a arcu. Donec condimentum erat eget justo egestas scelerisque. Fusce hendrerit efficitur leo, sed consectetur justo elementum ac.

Suspendisse neque massa, congue et semper sit amet, vehicula quis libero. Phasellus hendrerit elit massa, pretium tempor magna semper nec. Pellentesque vitae convallis dui. Donec at dictum lorem, vitae maximus dolor. Pellentesque vel vehicula nunc. Donec in consequat nisl. Pellentesque metus felis, laoreet eget magna sed, scelerisque pellentesque ipsum.

<!--more-->Aliquam interdum eleifend viverra. Mauris viverra, lorem at lobortis elementum, nunc felis malesuada diam, vel pretium felis odio at diam. Nulla at risus faucibus, congue ligula a, semper nibh. Quisque dictum malesuada porttitor. Vestibulum massa neque, ornare vel erat tincidunt, interdum varius purus. Suspendisse potenti. Curabitur odio justo, pellentesque placerat felis nec, gravida semper odio. Aenean tempus magna sit amet semper egestas. Aliquam tristique orci eget lorem egestas eleifend.

Ut eros enim, hendrerit in tristique et, mattis et magna. In molestie orci non pulvinar rutrum. Donec scelerisque sapien vitae massa fringilla, non semper lorem consequat. Aenean ornare leo eget quam iaculis semper. Vivamus semper pellentesque semper. Quisque dignissim nisi ipsum, et mattis diam gravida sed. Phasellus accumsan, arcu eu fringilla fringilla, quam leo varius sapien, ut dignissim augue dui et tortor. Aliquam erat volutpat. Maecenas efficitur ultricies pretium. Aenean urna sem, posuere et venenatis et, mollis a sem. Duis semper sem odio, vitae pellentesque lacus dignissim vitae.',
  'post_title' => 'Photography Magic',
  'post_excerpt' => '',
  'post_name' => 'photography-magic',
  'post_modified' => '2017-08-21 05:48:09',
  'post_modified_gmt' => '2017-08-21 05:48:09',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2505',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'designers, photography',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2096474412.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2508,
  'post_date' => '2014-05-16 19:25:11',
  'post_date_gmt' => '2014-05-16 19:25:11',
  'post_content' => 'Aenean sagittis lorem purus, in fringilla quam bibendum quis. Aliquam nisl ipsum, eleifend ac sapien vel, dictum feugiat metus. Nullam at leo id turpis finibus accumsan vel ac justo. Phasellus laoreet ex feugiat ligula facilisis porttitor. Nulla a ante ipsum. Morbi blandit, est eu sodales sagittis, diam est pretium purus, ut eleifend justo risus at risus. Duis at felis ut risus egestas mattis vel non elit. Maecenas dapibus urna eu eros malesuada auctor. Mauris ligula ante, consequat sit amet pretium et, dapibus non nunc.

Fusce maximus est eu nisl commodo, quis luctus neque condimentum. Proin non orci rhoncus, eleifend turpis id, auctor orci. Praesent ut augue in augue efficitur cursus vel eu diam. Sed ornare diam in luctus pretium. Donec volutpat tellus ac odio posuere dignissim. Sed et libero malesuada, varius lectus nec, sodales felis. Proin fermentum massa nec lacinia sollicitudin. Etiam varius nec metus vel mollis. Praesent at urna purus. Sed tempor ligula a enim feugiat, id lobortis massa volutpat. Integer fringilla at ante eget vehicula. Etiam elit velit, facilisis ac enim a, ultrices faucibus mi.',
  'post_title' => '70\'s Retro Look',
  'post_excerpt' => '',
  'post_name' => '70s-retro-look',
  'post_modified' => '2017-08-21 05:48:07',
  'post_modified_gmt' => '2017-08-21 05:48:07',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2508',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'fashion',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/185751812.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2511,
  'post_date' => '2014-10-16 19:27:24',
  'post_date_gmt' => '2014-10-16 19:27:24',
  'post_content' => 'Proin arcu ipsum, congue id eleifend quis, efficitur sit amet nibh. Suspendisse at laoreet tellus, eu luctus diam. Donec tempor est vitae nibh facilisis, nec aliquet mauris lobortis. Integer convallis non ipsum quis vestibulum. Etiam consectetur ante ac ligula suscipit mattis. Aliquam erat volutpat. Proin fringilla non magna et egestas. Aliquam sit amet ante eu sem euismod accumsan sit amet ac ipsum. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Fusce volutpat egestas leo, non aliquam orci cursus at.

In hac habitasse platea dictumst. Mauris ac consequat turpis, ac lacinia felis. Vivamus sem leo, faucibus et felis in, iaculis eleifend ipsum. Nam dolor dolor, semper ut risus at, faucibus iaculis justo. Sed lectus lectus, semper et urna in, molestie eleifend ex. Sed nunc urna, elementum non tempor a, commodo ac dui. Aenean condimentum vel nulla eget consectetur. Ut est neque, pretium eget cursus et, posuere non metus. Proin molestie rutrum fringilla. Duis dignissim nunc est, eu pulvinar nisi facilisis ac. Quisque mi felis, viverra vel ipsum at, volutpat molestie sem. Quisque aliquam neque viverra, suscipit tellus tincidunt, consequat urna. Etiam gravida molestie justo, vel maximus mauris iaculis in.',
  'post_title' => 'Extend your Reach',
  'post_excerpt' => '',
  'post_name' => 'extend-reach',
  'post_modified' => '2017-08-21 05:47:42',
  'post_modified_gmt' => '2017-08-21 05:47:42',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2511',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'writers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380661.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2520,
  'post_date' => '2014-10-16 19:31:51',
  'post_date_gmt' => '2014-10-16 19:31:51',
  'post_content' => 'Proin sodales faucibus tempor. Phasellus imperdiet turpis neque, at rhoncus ligula elementum at. Maecenas feugiat efficitur dui interdum laoreet. Nullam hendrerit eget purus nec finibus. Fusce vitae nibh a mauris ultricies finibus non iaculis mauris. Integer ex nulla, porta et suscipit et, viverra eget dui. Sed gravida, elit ut pharetra lacinia, nulla dui egestas ligula, vel tincidunt velit justo id mauris. In tincidunt mi vitae eros posuere cursus. In vitae mi arcu. Nulla scelerisque sed tortor vitae rhoncus. Aenean pellentesque rhoncus ligula sit amet lacinia. Integer libero ipsum, accumsan eget dolor in, venenatis pretium urna. Duis vitae tempor dolor, nec pretium augue. Vestibulum dignissim non magna eget pellentesque. Fusce in urna euismod ligula ultrices iaculis non gravida nulla.',
  'post_title' => 'Dress for the Weather',
  'post_excerpt' => '',
  'post_name' => 'dress-weather',
  'post_modified' => '2017-08-21 05:47:40',
  'post_modified_gmt' => '2017-08-21 05:47:40',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2520',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'fashion',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2523,
  'post_date' => '2014-08-16 19:34:04',
  'post_date_gmt' => '2014-08-16 19:34:04',
  'post_content' => 'Vestibulum non risus aliquet, venenatis metus quis, volutpat magna. Duis suscipit tortor tellus. Ut consectetur massa viverra, molestie nunc a, euismod eros. Aliquam ornare nisi risus, non tempus turpis tincidunt sed. Cras pretium ipsum sit amet augue bibendum volutpat. Donec ultricies dui quis mi blandit, ut sollicitudin risus maximus. Donec ut commodo urna. Nulla sodales interdum posuere. Vivamus posuere molestie enim, vel commodo est. Nunc iaculis commodo auctor. Etiam at nisl eros. Mauris rutrum mi eget faucibus iaculis. Nullam sit amet neque sed ex dictum molestie. Sed diam neque, tempor et sapien quis, consequat sollicitudin leo.

<!--more-->
<h3>Sometimes it\'s important to get away from it all...</h3>
Curabitur et orci id ipsum accumsan lobortis. Vestibulum id lorem felis. Phasellus sed nisi commodo, dapibus libero vitae, commodo urna. In consequat iaculis dui, pretium vehicula metus malesuada a. Sed luctus pharetra scelerisque. Aliquam finibus convallis dolor, a blandit neque finibus at. Mauris ornare congue bibendum. Maecenas blandit turpis eget aliquet pretium. Maecenas nec congue risus. Vivamus a pulvinar orci. Mauris eget fermentum velit, vitae pharetra risus. Donec nibh ante, dignissim vel mi vitae, porttitor imperdiet est. Curabitur id sapien velit. Quisque iaculis viverra bibendum. Duis condimentum augue eget metus pretium feugiat.

Donec sit amet magna enim. Lorem ipsum dolor sit amet, consectetur adipiscing elit. In vulputate luctus posuere. Etiam at nisl eros. Mauris rutrum mi eget faucibus iaculis. Nullam sit amet neque sed ex dictum molestie. Sed diam neque, tempor et sapien quis, consequat sollicitudin leo.',
  'post_title' => 'Take a Moment to Think',
  'post_excerpt' => '',
  'post_name' => 'take-moment-think',
  'post_modified' => '2017-08-21 05:47:59',
  'post_modified_gmt' => '2017-08-21 05:47:59',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2523',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'writers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/199648799.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2531,
  'post_date' => '2014-10-16 19:38:56',
  'post_date_gmt' => '2014-10-16 19:38:56',
  'post_content' => 'Aliquam eget augue tempor, blandit lectus eget, laoreet urna. Duis non mauris vel justo aliquet egestas. Vivamus vitae hendrerit purus. Sed et nulla ac turpis eleifend aliquam. In aliquet venenatis molestie. Duis laoreet aliquam quam, quis maximus nunc faucibus volutpat. Proin eget tortor dui.

Ut et cursus dolor. Nunc lacinia tortor justo, tempus vulputate sapien porttitor eu. Etiam at hendrerit velit. Phasellus placerat nisi massa, at sodales sem sollicitudin vitae. Nulla ultricies mi mattis purus accumsan, eu efficitur dui egestas. Donec sit amet sapien lacus. Nunc mollis molestie arcu, nec malesuada sem finibus nec. Phasellus ac congue leo, sed dictum massa. Morbi vel ultricies metus, vitae posuere nunc. Aenean id mauris non nunc molestie pharetra in sit amet elit.',
  'post_title' => 'Elegance and Beauty',
  'post_excerpt' => '',
  'post_name' => 'elegance-beauty',
  'post_modified' => '2017-08-21 05:47:38',
  'post_modified_gmt' => '2017-08-21 05:47:38',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2531',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'artists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/183592982.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2536,
  'post_date' => '2014-06-16 19:42:56',
  'post_date_gmt' => '2014-06-16 19:42:56',
  'post_content' => 'Aliquam sodales lobortis blandit. Integer interdum, felis eget pharetra rhoncus, tortor odio pellentesque ipsum, ac suscipit nunc tortor in risus. Suspendisse porta urna laoreet ultricies varius. Aliquam id luctus lectus. Suspendisse erat urna, ullamcorper eget mi at, facilisis vestibulum justo. Sed tincidunt aliquam velit, vitae porttitor nisl sollicitudin vel. Fusce ac gravida ipsum. Donec a varius risus.

Ut vel ex nec turpis interdum elementum. Fusce dolor diam, consequat a nisi vitae, bibendum convallis sapien. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Praesent at orci eget libero tempus faucibus. Ut at leo at est varius tincidunt. Fusce non orci arcu. Nam ut viverra metus, a posuere nisl.',
  'post_title' => 'Designers Basic Essential',
  'post_excerpt' => '',
  'post_name' => 'designers-basic-essential',
  'post_modified' => '2017-08-21 05:48:05',
  'post_modified_gmt' => '2017-08-21 05:48:05',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2536',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/198699503.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2539,
  'post_date' => '2014-10-16 19:45:48',
  'post_date_gmt' => '2014-10-16 19:45:48',
  'post_content' => 'Ut nisl nisl, faucibus vitae eros vitae, bibendum sollicitudin eros. Duis sed metus varius, ullamcorper metus sed, dapibus leo. Donec dictum tempus magna, quis fringilla nunc porta id. Duis lacinia tincidunt arcu sit amet varius. Sed consectetur eu massa ac rhoncus. Aenean a dui nec tortor condimentum porta. Mauris finibus hendrerit tempus. Suspendisse dictum eu felis vel aliquet. Nulla ac ornare purus, in iaculis mi. Curabitur porttitor tincidunt dolor, sed aliquam massa. Nulla sodales quam ipsum, non porta dui vulputate et. Aenean non suscipit risus. Nulla nec turpis venenatis, viverra justo quis, tincidunt tellus. Duis facilisis lobortis aliquet. Morbi ac dignissim felis.

Sed est massa, egestas eget accumsan in, accumsan ullamcorper quam. Ut tincidunt, sapien sed lobortis laoreet, velit metus vestibulum sem, placerat aliquet ipsum ante nec libero. Sed lacus odio, vestibulum in enim vel, vestibulum volutpat magna. Curabitur id eros vel ipsum finibus aliquet vel sed diam. Proin fringilla enim sed quam suscipit fermentum. Nunc ut lectus a nisl mollis ornare. Cras vitae felis feugiat, convallis massa et, mollis elit. Morbi vel erat aliquam, interdum ex nec, interdum dui.

Curabitur blandit ultricies pharetra. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nam quis ullamcorper velit, id consequat magna. Curabitur ex tortor, condimentum a est ac, laoreet fringilla ligula. Nunc eget magna rhoncus, euismod orci nec, mollis dolor. Donec ac pulvinar sem. Cras eu volutpat diam, non porta odio. Quisque eget erat massa. Aenean non aliquet augue. Curabitur ut ligula a nunc auctor pulvinar. Etiam volutpat ipsum ac magna venenatis, eget molestie dolor lobortis. Donec a elit porttitor, semper lorem a, pharetra libero. Mauris eget consequat nisi.

<!--more-->
<h3>Food was phenomenal wish I could stay longer...</h3>
Sed a nunc libero. Vestibulum sollicitudin massa pharetra nisi convallis elementum. Nullam congue egestas volutpat. Mauris sed felis vitae eros iaculis tempus id at nibh. Integer ut augue porta, sagittis dui id, tristique lorem. Mauris hendrerit nisi id ligula eleifend, nec scelerisque ipsum dictum. Donec faucibus at lorem commodo ultrices. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Maecenas viverra ipsum vitae metus convallis, tempor molestie diam feugiat.

Vestibulum molestie rhoncus dapibus. Nullam odio risus, cursus sed purus nec, fermentum tempus urna. Donec libero metus, mattis ac iaculis vitae, viverra at enim. Phasellus accumsan lectus consequat laoreet imperdiet. Suspendisse aliquet ex eget nulla cursus, id fermentum dolor lacinia. Fusce gravida ante nec malesuada varius. Vestibulum purus massa, molestie ac commodo a, consectetur in nunc. Cras mollis ante et neque rhoncus, non tempor nibh mollis. Pellentesque convallis, massa sit amet posuere semper, quam purus consectetur arcu, a imperdiet magna nibh ac risus. Suspendisse ac pulvinar magna. Nam et tellus a turpis maximus pellentesque sed sed quam. Proin ante quam, euismod id est at, dapibus pulvinar turpis. Nullam laoreet pulvinar eleifend. Aliquam posuere molestie massa, sollicitudin vestibulum quam posuere sit amet. Praesent et ipsum placerat dolor efficitur lacinia.',
  'post_title' => 'New York Visit',
  'post_excerpt' => '',
  'post_name' => 'new-york-visit',
  'post_modified' => '2017-08-21 05:47:37',
  'post_modified_gmt' => '2017-08-21 05:47:37',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2539',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'tourists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2120163251.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2543,
  'post_date' => '2014-10-16 19:48:36',
  'post_date_gmt' => '2014-10-16 19:48:36',
  'post_content' => 'Aenean vulputate sapien eu massa porttitor, rhoncus pellentesque mi tristique. Donec faucibus ipsum non erat fringilla, vitae volutpat nisl lobortis. Donec ante justo, feugiat sed felis at, semper gravida nulla. Nulla facilisi. Vivamus quam neque, egestas nec eros a, convallis porta dolor. Duis hendrerit augue in mattis lobortis. Pellentesque accumsan mollis quam, at scelerisque erat aliquam eu. Integer sit amet mollis metus. Fusce aliquet ipsum vitae nisi eleifend, ut rutrum enim interdum.',
  'post_title' => 'Bringing Life to the Art',
  'post_excerpt' => '',
  'post_name' => 'bringing-life-art',
  'post_modified' => '2017-08-21 05:47:35',
  'post_modified_gmt' => '2017-08-21 05:47:35',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2543',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'artists, designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2050088201.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2550,
  'post_date' => '2014-09-28 19:54:01',
  'post_date_gmt' => '2014-09-28 19:54:01',
  'post_content' => 'Proin non dui at tortor efficitur vestibulum ut eu metus. Vivamus eu lobortis eros. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Quisque fringilla viverra urna quis gravida. Mauris lobortis libero luctus orci ultrices interdum. Donec eleifend neque et arcu tristique faucibus. Nulla in eros vel justo venenatis condimentum. Cras sit amet augue sit amet quam pulvinar tincidunt ut eu felis. Duis congue varius gravida. Duis mattis fringilla felis, quis blandit dui malesuada ut. Duis volutpat tortor est. Phasellus ornare sem erat, malesuada cursus sem blandit ut. Integer quis lectus nulla. Morbi pellentesque convallis dui, laoreet viverra est finibus fringilla.

<!--more-->Morbi aliquet vel mauris quis consectetur. Sed eu nunc a nisl dictum aliquam nec vel odio. Ut facilisis nec arcu ac varius. Curabitur eu lacinia mi, eget pretium leo. Mauris aliquam quis ipsum quis sagittis. Etiam in odio varius, ullamcorper tortor vel, interdum odio. Curabitur id ipsum magna. Cras mattis ex mi, ac finibus nunc porttitor eu. Phasellus cursus lacus a felis ultrices, non interdum nibh fermentum. In odio leo, condimentum eget arcu non, pharetra rhoncus felis. Pellentesque faucibus nunc nec hendrerit finibus. Vivamus nec ornare tortor.

Mauris eget lectus sem. Quisque venenatis ex eget odio placerat, et facilisis orci accumsan. Quisque maximus est at metus pellentesque, non tristique justo facilisis. Duis dignissim diam lobortis, mollis nulla vitae, condimentum augue. Donec mi libero, dignissim eget accumsan at, lacinia ac nunc. Cras cursus velit convallis, finibus arcu ut, ultricies metus. Quisque quis lorem luctus, aliquet lorem ac, varius dolor.',
  'post_title' => 'The Next Big Thing',
  'post_excerpt' => '',
  'post_name' => 'next-big-thing',
  'post_modified' => '2017-08-21 05:47:58',
  'post_modified_gmt' => '2017-08-21 05:47:58',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2550',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/201732071.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2553,
  'post_date' => '2014-10-16 19:55:33',
  'post_date_gmt' => '2014-10-16 19:55:33',
  'post_content' => 'Quisque dui tortor, congue sed condimentum ut, vestibulum ut purus. Proin ut lorem non sem laoreet efficitur. In augue ante, pretium eu tempus non, tincidunt maximus augue. Integer pellentesque semper pharetra. Praesent dolor ipsum, elementum eget rutrum et, aliquam vitae justo. Fusce feugiat tincidunt dictum. Vivamus sit amet feugiat odio. Sed porttitor et massa eu facilisis. Maecenas sagittis vestibulum ultrices. Phasellus eget turpis massa. Cras sit amet nulla odio. Nulla porta odio risus, vitae imperdiet urna congue eget. Nulla finibus sodales magna, a tempor ante lobortis sit amet. Integer rhoncus faucibus magna porttitor iaculis. Curabitur fermentum magna at arcu faucibus, in euismod magna sodales.

Duis condimentum gravida arcu, nec vehicula ligula ultricies et. Phasellus vulputate nisl id augue vestibulum accumsan. Interdum et malesuada fames ac ante ipsum primis in faucibus. In a augue mi. Sed ultrices volutpat congue. Mauris quis tristique mi. Nunc condimentum magna nec placerat vehicula.',
  'post_title' => 'Selfie Photography',
  'post_excerpt' => '',
  'post_name' => 'selfie-photography',
  'post_modified' => '2017-08-21 05:47:30',
  'post_modified_gmt' => '2017-08-21 05:47:30',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2553',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'designers, photography',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1988883051.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2557,
  'post_date' => '2014-08-16 19:01:01',
  'post_date_gmt' => '2014-08-16 19:01:01',
  'post_content' => 'Etiam eu pellentesque enim, et vulputate magna. Fusce malesuada ultricies magna, et venenatis nisi sagittis pulvinar. Mauris malesuada, lacus ut maximus rutrum, quam risus volutpat eros, vel viverra tellus neque ac leo. Morbi vel pharetra dolor. Donec convallis egestas facilisis. Vivamus et tortor cursus, luctus mauris at, placerat dui. Curabitur congue libero sed metus tempus blandit. Duis erat lorem, dapibus vitae fringilla blandit, pretium non sapien. Etiam feugiat justo nibh, tincidunt tempu itur in pretium lacus. Quisque rhoncus lectus felis, sit amet ullamcorper arcu tincidunt vel. Cras molestie massa sit amet ante scelerisque, sit amet porta mi malesuada. In nec diam lectus. Proin aliquet tellus nisi, eget sagittis lorem tempuset mi eget bibend elementum urna.

<!--more-->
<h3>Writer\'s Block.</h3>
In augue enim, volutpat a feugiat eget, venenatis ut lectus. Nullam non lobortis sapien. Suspendisse efficitur luctus dolor eu condimentum. Sed ac tortor urna. Suspendisse mollis arcu vel accumsan suscipit. Pellentesque vitae urna maximus, luctus dui eu, tristique sem. Mauris vulputate dapibus ligula ut ornare. Quisque maximus, nisl sed efficitur tincidunt, orci massa imperdiet dolor, id posuere orci ligula nec quam. Morbi maximus lorem eu sapien bibendum mollis. Aenean at maximus diam. Nunc tempor ligula a tellus rutrum, eget maximus velit porttitor. Ut efficitur gravida enim, ac vestibulum tellus efficitur vitae. Aenean efficitur est ut tincidunt consequat. Sed interdum maximus mollis. Phasellus sodales ex ac molestie dictum. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae;

Pellentesque id tincidunt ex, vel mollis arcu. Quisque convallis ex et enim lacinia, eget rhoncus mi porta. Proin eu nisl vulputate purus fringilla tempus nec vel est. Praesent condimentum arcu in gravida tempor. Nullam sit amet laoreet odio, nec molestie est. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed interdum ornare ipsum, vel dapibus libero bibendum sit amet. Mauris maximus rhoncus pretium.',
  'post_title' => 'Finding the Words',
  'post_excerpt' => '',
  'post_name' => 'finding-words',
  'post_modified' => '2017-08-21 05:48:01',
  'post_modified_gmt' => '2017-08-21 05:48:01',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2557',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'writers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2561,
  'post_date' => '2014-10-16 19:58:03',
  'post_date_gmt' => '2014-10-16 19:58:03',
  'post_content' => 'This post has sidebar. Praesent pretium tincidunt pellentesque. Donec pellentesque maximus ipsum, eget venenatis libero. Donec augue velit, pellentesque non libero ac, pretium consequat diam. Nunc nec nisl et eros fermentum interdum. Pellentesque dapibus magna auctor metus dictum rhoncus. Praesent rhoncus viverra ex, in rhoncus neque congue in. Suspendisse at velit eget risus sagittis accumsan. Ut in finibus metus. In viverra tellus eget diam consectetur tempor. Sed consequat dui sed purus molestie luctus.

<!--more-->Donec viverra id nulla dictum hendrerit. Sed nec justo id turpis mollis ornare. In scelerisque non arcu et ullamcorper. Morbi pellentesque vel ligula at cursus. Praesent in felis ultricies, hendrerit erat non, placerat odio. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Praesent congue diam ante, sed bibendum nibh finibus nec. Donec imperdiet pharetra sapien, ac posuere dui pharetra non. Integer condimentum ligula at viverra volutpat.

Pellentesque eu enim malesuada, interdum lacus nec, laoreet velit. Etiam id molestie nibh, sed pellentesque neque. Duis at lacus ut justo lobortis lacinia et vel magna. Pellentesque felis erat, porttitor id tincidunt ut, suscipit vitae lacus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed et nisl non magna pulvinar dignissim. Cras imperdiet ut arcu eu accumsan. Sed sed sodales orci. Mauris pharetra dictum turpis. Nunc convallis, odio a elementum lacinia, felis sem placerat neque, pharetra imperdiet ipsum urna a nisi. Pellentesque commodo malesuada accumsan. Maecenas varius venenatis semper. Morbi ut purus vestibulum, porta nisi eu, rhoncus tortor. Nunc elit ante, mollis non facilisis vehicula, volutpat id lacus. Sed vitae diam eleifend, rhoncus orci nec, posuere quam. Cras ullamcorper mauris massa, ut faucibus erat facilisis in.',
  'post_title' => 'Sidebar Option',
  'post_excerpt' => '',
  'post_name' => 'floral-artistry',
  'post_modified' => '2017-08-21 05:47:26',
  'post_modified_gmt' => '2017-08-21 05:47:26',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=2561',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'layout' => 'sidebar1',
    'content_text_align' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'artists',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/214781395.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3244,
  'post_date' => '2014-10-16 19:58:14',
  'post_date_gmt' => '2014-10-16 19:58:14',
  'post_content' => 'Tasellus ac venenatis lectus, non scelerisque nunc. Cras nisi massa, accumsan sed magna at, rhoncus euismod risus. Pellentesque viverra, orci auctor ultricies finibus, mauris massa auctor ante, vel rhoncus magna tortor eget tortor. Pellentesque mollis nibh non elit semper scelerisque. Pellentesque purus mauris, luctus interdum dictum ut, bibendum fringilla enim. Sed pretium turpis ac arcu venenatis, vel sagittis quam congue. Proin aliquam felis vitae odio aliquam eleifend. Praesent quis nulla nibh. Fusce bibendum sem at odio commodo dignissim. Sed est metus, iaculis id mattis a, iaculis eget mauris. Vivamus vestibulum commodo elit, eget venenatis nisl mollisa. Cras ut felis non massa placerat auctor ut ac nisl. Duis tempor eleifend libero, non tincidunt sapien hendrerit non. Donec eu aliquam lorem, commodo faucibus leo. Fusce tempus in erat id iaculis. Praesent eget risus laoreet, interdum leo non, gravida neque.',
  'post_title' => 'Adidas Snowboarding',
  'post_excerpt' => '',
  'post_name' => 'adidas-snowboarding',
  'post_modified' => '2018-02-02 17:54:49',
  'post_modified_gmt' => '2018-02-02 17:54:49',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=3244',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'video_url' => 'https://vimeo.com/122435905',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\"}]}]',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3284,
  'post_date' => '2014-06-19 17:29:24',
  'post_date_gmt' => '2014-06-19 17:29:24',
  'post_content' => 'Cras ut felis non massa placerat auctor ut ac nisl. Duis tempor eleifend libero, non tincidunt sapien hendrerit non. Donec eu aliquam lorem, commodo faucibus leo. Fusce tempus in erat id iaculis. Praesent eget risus laoreet, interdum leo non, gravida neque. Tasellus ac venenatis lectus, non scelerisque nunc. Cras nisi massa, accumsan sed magna at, rhoncus euismod risus. Pellentesque viverra, orci auctor ultricies finibus, mauris massa auctor ante, vel rhoncus magna tortor eget tortor. Pellentesque mollis nibh non elit semper scelerisque. Pellentesque purus mauris, luctus interdum dictum ut, bibendum fringilla enim.',
  'post_title' => 'Edges of Sanity',
  'post_excerpt' => '',
  'post_name' => 'edges-sanity',
  'post_modified' => '2017-08-21 05:48:03',
  'post_modified_gmt' => '2017-08-21 05:48:03',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=3284',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'video_url' => 'http://vimeo.com/108798045',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/DSC_4738.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3249,
  'post_date' => '2014-10-16 19:50:04',
  'post_date_gmt' => '2014-10-16 19:50:04',
  'post_content' => 'Cras ut felis non massa placerat auctor ut ac nisl. Duis tempor eleifend libero, non tincidunt sapien hendrerit non. Donec eu aliquam lorem, commodo faucibus leo. Fusce tempus in erat id iaculis. Praesent eget risus laoreet, interdum leo non, gravida neque. Tasellus ac venenatis lectus, non scelerisque nunc. Cras nisi massa, accumsan sed magna at, rhoncus euismod risus. Pellentesque viverra, orci auctor ultricies finibus, mauris massa auctor ante, vel rhoncus magna tortor eget tortor. Pellentesque mollis nibh non elit semper scelerisque. Pellentesque purus mauris, luctus interdum dictum ut, bibendum fringilla enim.

Sed pretium turpis ac arcu venenatis, vel sagittis quam congue. Proin aliquam felis vitae odio aliquam eleifend. Praesent quis nulla nibh. Fusce bibendum sem at odio commodo dignissim. Sed est metus, iaculis id mattis a, iaculis eget mauris. Vivamus vestibulum commodo elit, eget venenatis nisl mollisa.',
  'post_title' => 'Chanel Ad',
  'post_excerpt' => '',
  'post_name' => 'chanel-ad',
  'post_modified' => '2017-08-21 05:47:32',
  'post_modified_gmt' => '2017-08-21 05:47:32',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=3249',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'video_url' => 'http://www.youtube.com/watch?v=v4cWa_DV1wE',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/chanel-ad.png',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3252,
  'post_date' => '2014-10-16 19:16:19',
  'post_date_gmt' => '2014-10-16 19:16:19',
  'post_content' => 'Sed pretium turpis ac arcu venenatis, vel sagittis quam congue. Proin aliquam felis vitae odio aliquam eleifend. Praesent quis nulla nibh. Fusce bibendum sem at odio commodo dignissim. Sed est metus, iaculis id mattis a, iaculis eget mauris. Vivamus vestibulum commodo elit, eget venenatis nisl mollisa.

Cras ut felis non massa placerat auctor ut ac nisl. Duis tempor eleifend libero, non tincidunt sapien hendrerit non. Donec eu aliquam lorem, commodo faucibus leo. Fusce tempus in erat id iaculis. Praesent eget risus laoreet, interdum leo non, gravida neque. Tasellus ac venenatis lectus, non scelerisque nunc. Cras nisi massa, accumsan sed magna at, rhoncus euismod risus. Pellentesque viverra, orci auctor ultricies finibus, mauris massa auctor ante, vel rhoncus magna tortor eget tortor. Pellentesque mollis nibh non elit semper scelerisque. Pellentesque purus mauris, luctus interdum dictum ut, bibendum fringilla enim.',
  'post_title' => 'iPhone 6',
  'post_excerpt' => '',
  'post_name' => 'iphone-6',
  'post_modified' => '2017-08-21 05:47:48',
  'post_modified_gmt' => '2017-08-21 05:47:48',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=3252',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'video_url' => 'https://www.youtube.com/watch?v=h-ErA1w8Iy0',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/DSC_4738.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2354,
  'post_date' => '2014-10-13 18:10:05',
  'post_date_gmt' => '2014-10-13 18:10:05',
  'post_content' => '[woocommerce_cart]',
  'post_title' => 'Cart',
  'post_excerpt' => '',
  'post_name' => 'cart',
  'post_modified' => '2017-08-21 05:50:33',
  'post_modified_gmt' => '2017-08-21 05:50:33',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/cart/',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2355,
  'post_date' => '2014-10-13 18:10:05',
  'post_date_gmt' => '2014-10-13 18:10:05',
  'post_content' => '[woocommerce_checkout]',
  'post_title' => 'Checkout',
  'post_excerpt' => '',
  'post_name' => 'checkout',
  'post_modified' => '2017-08-21 05:50:35',
  'post_modified_gmt' => '2017-08-21 05:50:35',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/checkout/',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2356,
  'post_date' => '2014-10-13 18:10:05',
  'post_date_gmt' => '2014-10-13 18:10:05',
  'post_content' => '[woocommerce_my_account]',
  'post_title' => 'My Account',
  'post_excerpt' => '',
  'post_name' => 'my-account',
  'post_modified' => '2017-08-21 05:50:36',
  'post_modified_gmt' => '2017-08-21 05:50:36',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/my-account/',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2406,
  'post_date' => '2014-10-14 18:55:00',
  'post_date_gmt' => '2014-10-14 18:55:00',
  'post_content' => '<!--themify_builder_static--><h1 style="text-align: center;">About</h1><h3 style="text-align: center;">Heading 3: A creative way to show what your page is about is with the Progress Bar Builder Addon. This Addon is professional, simple, and comes with multiple styling features. Check it out below.</h3>
 
 <i>Music</i> <i>Performance</i> <i>Cooking</i> 
 <p style="text-align: center;"><a href="#">CONTACT ME</a>
 
 
 Mill Visitors
 
 
 
 Billion
 
 
 
 Fans
 
 
 
 Raised Funds
 
 
 <img src="https://themify.me/demo/themes/elegant/files/2014/10/202812511-600x600.jpg" width="600" height="600" alt="202812511" srcset="https://themify.me/demo/themes/elegant/files/2014/10/202812511-600x600.jpg 600w, https://themify.me/demo/themes/elegant/files/2014/10/202812511-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/202812511-300x300.jpg 300w, https://themify.me/demo/themes/elegant/files/2014/10/202812511-1024x1024.jpg 1024w, https://themify.me/demo/themes/elegant/files/2014/10/202812511-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/202812511-800x800.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg 1200w" sizes="(max-width: 600px) 100vw, 600px" /> 
 <h2>New Album</h2><p>Our Audio Addon is great way for you to showcase your audio files on your site. </p>
 
 <ol> <li itemprop="track" itemscope="" itemtype="http://schema.org/MusicRecording"> <a href="#" itemprop="url">Helpless</a> <audio id="audio-0-1" preload="none" style="width: 100%;" controls="controls"><source type="audio/mpeg" src="https://themify.me/demo/themes/music/files/2014/07/Helpless.mp3?_=1" /><a href="https://themify.me/demo/themes/music/files/2014/07/Helpless.mp3">https://themify.me/demo/themes/music/files/2014/07/Helpless.mp3</a></audio> </li> <li itemprop="track" itemscope="" itemtype="http://schema.org/MusicRecording"> <a href="#" itemprop="url">It\'s Not Over Yet</a> <audio id="audio-0-2" preload="none" style="width: 100%;" controls="controls"><source type="audio/mpeg" src="https://themify.me/demo/themes/music/files/2014/07/myuu-ItsNotOverYet.mp3?_=2" /><a href="https://themify.me/demo/themes/music/files/2014/07/myuu-ItsNotOverYet.mp3">https://themify.me/demo/themes/music/files/2014/07/myuu-ItsNotOverYet.mp3</a></audio> </li> <li itemprop="track" itemscope="" itemtype="http://schema.org/MusicRecording"> <a href="#" itemprop="url">O Song</a> <audio id="audio-0-3" preload="none" style="width: 100%;" controls="controls"><source type="audio/mpeg" src="https://themify.me/demo/themes/music/files/2014/07/o.mp3?_=3" /><a href="https://themify.me/demo/themes/music/files/2014/07/o.mp3">https://themify.me/demo/themes/music/files/2014/07/o.mp3</a></audio> </li> <li itemprop="track" itemscope="" itemtype="http://schema.org/MusicRecording"> <a href="#" itemprop="url">Contemplatio Remix</a> <audio id="audio-0-4" preload="none" style="width: 100%;" controls="controls"><source type="audio/mpeg" src="https://themify.me/demo/themes/music/files/2014/06/lounge-Contemplatio_Remix.mp3?_=4" /><a href="https://themify.me/demo/themes/music/files/2014/06/lounge-Contemplatio_Remix.mp3">https://themify.me/demo/themes/music/files/2014/06/lounge-Contemplatio_Remix.mp3</a></audio> </li> <li itemprop="track" itemscope="" itemtype="http://schema.org/MusicRecording"> <a href="#" itemprop="url">Blisco Natural</a> <audio id="audio-0-5" preload="none" style="width: 100%;" controls="controls"><source type="audio/mpeg" src="https://themify.me/demo/themes/music/files/2014/06/chillout-Blisco-Natural.mp3?_=5" /><a href="https://themify.me/demo/themes/music/files/2014/06/chillout-Blisco-Natural.mp3">https://themify.me/demo/themes/music/files/2014/06/chillout-Blisco-Natural.mp3</a></audio> </li> </ol> 
 <h2>See <em>my</em> Photos</h2>
 
 <img src="https://themify.me/demo/themes/elegant/files/2014/10/211462315-650x517.jpg" width="650" height="517" title="Photo Image Caption" alt="Photo Image Caption" /> <h3> Photo Image Caption </h3> 
 
 <img src="https://themify.me/demo/themes/elegant/files/2014/10/199095755-630x240.jpg" width="630" height="240" alt="199095755" /> 
 
 <img src="https://themify.me/demo/themes/elegant/files/2014/10/165819980-630x235.jpg" width="630" height="235" alt="165819980" /> 
 <ul data-id="slider-0-" data-visible="1" data-scroll="1" data-auto-scroll="4" data-speed="1" data-wrap="yes" data-arrow="yes" data-pagination="yes" data-effect="scroll" data-height="variable" data-pause-on-hover="resume" > 
 <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/1289128971-1300x600.jpg" width="1300" height="600" alt="Photo Image Caption" /> 
 <h3> Photo Image Caption </h3> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/1485127491-1300x600.jpg" width="1300" height="600" alt="Photo Image Caption" /> 
 <h3> Photo Image Caption </h3> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/2084238971-1300x600.jpg" width="1300" height="600" alt="Photo Image Caption" /> 
 <h3> Photo Image Caption </h3> </li> </ul><!--/themify_builder_static-->',
  'post_title' => 'Artist',
  'post_excerpt' => '',
  'post_name' => 'artist',
  'post_modified' => '2018-02-02 17:58:11',
  'post_modified_gmt' => '2018-02-02 17:58:11',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=2406',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'full_width',
    'hide_page_title' => 'yes',
    'background_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/artist-header-img.jpg',
    'headerwrap_text_color' => '#7a4b1c',
    'headerwrap_link_color' => '#7a4b1c',
    'display_content' => 'content',
    'portfolio_layout' => 'list-post',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">About<\\\\/h1><h3 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Heading 3: A creative way to show what your page is about is with the <span style=\\\\\\\\\\\\\\"text-decoration: underline;\\\\\\\\\\\\\\">Progress Bar Builder Addon<\\\\/span>. This Addon is professional, simple, and comes with multiple styling features. Check it out below.<\\\\/h3>\\",\\"cid\\":\\"c17\\"}},{\\"mod_name\\":\\"progressbar\\",\\"mod_settings\\":{\\"padding_top\\":\\"1\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"1\\",\\"padding_right_unit\\":\\"%\\",\\"padding_bottom\\":\\"1\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"1\\",\\"padding_left_unit\\":\\"%\\",\\"checkbox_margin_apply_all\\":\\"1\\",\\"checkbox_border_apply_all\\":\\"1\\",\\"progress_bars\\":[{\\"bar_label\\":\\"Music\\",\\"bar_percentage\\":\\"80\\",\\"bar_color\\":\\"e32d8b\\"},{\\"bar_label\\":\\"Performance\\",\\"bar_percentage\\":\\"90\\",\\"bar_color\\":\\"11b875\\"},{\\"bar_label\\":\\"Cooking\\",\\"bar_percentage\\":\\"40\\",\\"bar_color\\":\\"c47316\\"}],\\"hide_percentage_text\\":\\"yes\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">[themify_button style=\\\\\\\\\\\\\\"large flat rect outline\\\\\\\\\\\\\\" ]CONTACT ME[\\\\/themify_button]<\\\\/p>\\",\\"cid\\":\\"c25\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"7.5\\",\\"label_counter\\":\\"Mill Visitors\\",\\"circle_percentage_counter\\":\\"80\\",\\"circle_stroke_counter\\":\\"4\\",\\"circle_color_counter\\":\\"fafa17_1.00\\",\\"size_counter\\":\\"large\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"25\\",\\"cid\\":\\"c36\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"$41\\",\\"label_counter\\":\\"Billion\\",\\"circle_percentage_counter\\":\\"65\\",\\"circle_stroke_counter\\":\\"4\\",\\"circle_color_counter\\":\\"08ceff_1.00\\",\\"size_counter\\":\\"large\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"25\\",\\"cid\\":\\"c44\\"}}]},{\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"3.2k\\",\\"label_counter\\":\\"Fans\\",\\"circle_percentage_counter\\":\\"90\\",\\"circle_stroke_counter\\":\\"4\\",\\"circle_color_counter\\":\\"f55fee_1.00\\",\\"size_counter\\":\\"large\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"25\\",\\"cid\\":\\"c52\\"}}]},{\\"column_order\\":\\"3\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"70%\\",\\"label_counter\\":\\"Raised Funds\\",\\"circle_percentage_counter\\":\\"70\\",\\"circle_stroke_counter\\":\\"4\\",\\"circle_color_counter\\":\\"ff7424_1.00\\",\\"size_counter\\":\\"large\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"25\\",\\"cid\\":\\"c60\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1921473141.jpg\\",\\"background_repeat\\":\\"fullcover\\",\\"cover_color\\":\\"ffc800_0.23\\",\\"padding_top\\":\\"15\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"15\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/202812511.jpg\\",\\"width_image\\":\\"600\\",\\"height_image\\":\\"600\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"cid\\":\\"c71\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2>New Album<\\\\/h2><p>Our Audio Addon is great way for you to showcase your audio files on your site. <\\\\/p>\\",\\"line_height\\":\\"20\\",\\"padding_top\\":\\"0\\",\\"padding_bottom\\":\\"0\\",\\"cid\\":\\"c79\\"}},{\\"mod_name\\":\\"audio\\",\\"mod_settings\\":{\\"music_playlist\\":[{\\"audio_name\\":\\"Helpless\\",\\"audio_url\\":\\"https:\\\\/\\\\/themify.me\\\\/demo\\\\/themes\\\\/music\\\\/files\\\\/2014\\\\/07\\\\/Helpless.mp3\\"},{\\"audio_name\\":\\"It\\\\\\\\\\\'s Not Over Yet\\",\\"audio_url\\":\\"https:\\\\/\\\\/themify.me\\\\/demo\\\\/themes\\\\/music\\\\/files\\\\/2014\\\\/07\\\\/myuu-ItsNotOverYet.mp3\\"},{\\"audio_name\\":\\"O Song\\",\\"audio_url\\":\\"https:\\\\/\\\\/themify.me\\\\/demo\\\\/themes\\\\/music\\\\/files\\\\/2014\\\\/07\\\\/o.mp3\\"},{\\"audio_name\\":\\"Contemplatio Remix\\",\\"audio_url\\":\\"https:\\\\/\\\\/themify.me\\\\/demo\\\\/themes\\\\/music\\\\/files\\\\/2014\\\\/06\\\\/lounge-Contemplatio_Remix.mp3\\"},{\\"audio_name\\":\\"Blisco Natural\\",\\"audio_url\\":\\"https:\\\\/\\\\/themify.me\\\\/demo\\\\/themes\\\\/music\\\\/files\\\\/2014\\\\/06\\\\/chillout-Blisco-Natural.mp3\\"}],\\"cid\\":\\"c83\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"background_color\\":\\"f2f2f2_1.00\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2>See <em>my<\\\\/em> Photos<\\\\/h2>\\",\\"cid\\":\\"c94\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"6\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"15\\"}},{\\"row_order\\":\\"4\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-overlay\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/211462315.jpg\\",\\"width_image\\":\\"650\\",\\"height_image\\":\\"517\\",\\"title_image\\":\\"Photo Image Caption\\",\\"cid\\":\\"c105\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/199095755.jpg\\",\\"width_image\\":\\"630\\",\\"height_image\\":\\"240\\",\\"padding_bottom\\":\\"2\\",\\"padding_bottom_unit\\":\\"%\\",\\"margin_top_unit\\":\\"%\\",\\"margin_bottom_unit\\":\\"%\\",\\"cid\\":\\"c113\\"}},{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/165819980.jpg\\",\\"width_image\\":\\"630\\",\\"height_image\\":\\"235\\",\\"padding_top\\":\\"2\\",\\"padding_top_unit\\":\\"%\\",\\"margin_top_unit\\":\\"%\\",\\"cid\\":\\"c117\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top_unit\\":\\"%\\"}},{\\"row_order\\":\\"5\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"slider\\",\\"mod_settings\\":{\\"layout_display_slider\\":\\"image\\",\\"blog_category_slider\\":\\"|single\\",\\"slider_category_slider\\":\\"|single\\",\\"portfolio_category_slider\\":\\"|single\\",\\"testimonial_category_slider\\":\\"|single\\",\\"order_slider\\":\\"desc\\",\\"orderby_slider\\":\\"date\\",\\"display_slider\\":\\"content\\",\\"img_content_slider\\":[{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1289128971.jpg\\",\\"img_title_slider\\":\\"Photo Image Caption\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1485127491.jpg\\",\\"img_title_slider\\":\\"Photo Image Caption\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/2084238971.jpg\\",\\"img_title_slider\\":\\"Photo Image Caption\\"}],\\"layout_slider\\":\\"slider-overlay\\",\\"img_w_slider\\":\\"1300\\",\\"img_h_slider\\":\\"600\\",\\"visible_opt_slider\\":\\"1\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"scroll\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"text_align\\":\\"left\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"2\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"6\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\"}]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3170,
  'post_date' => '2014-10-17 01:51:03',
  'post_date_gmt' => '2014-10-17 01:51:03',
  'post_content' => '<h3>The blog posts are queried in the Themify Custom Panel. In this page, the masonry layout is enabled and grayscale image filter is selected.</h3>
&nbsp;',
  'post_title' => '4 Column Blog',
  'post_excerpt' => '',
  'post_name' => 'blog-4-columns',
  'post_modified' => '2017-08-21 05:50:07',
  'post_modified_gmt' => '2017-08-21 05:50:07',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3170',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'imagefilter_options' => 'grayscale',
    'imagefilter_options_hover' => 'none',
    'hide_page_title' => 'no',
    'query_category' => '0',
    'layout' => 'grid4',
    'enable_masonry' => 'yes',
    'posts_per_page' => '12',
    'image_width' => '294',
    'image_height' => '0',
    'portfolio_layout' => 'list-post',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3176,
  'post_date' => '2014-10-17 02:10:56',
  'post_date_gmt' => '2014-10-17 02:10:56',
  'post_content' => '<h3>In this page, the masonry layout is disabled and blur image filter is applied on hover.</h3>
&nbsp;',
  'post_title' => '3 Column Blog',
  'post_excerpt' => '',
  'post_name' => 'blog-3-columns',
  'post_modified' => '2017-08-21 05:50:11',
  'post_modified_gmt' => '2017-08-21 05:50:11',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3176',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'blur',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'no',
    'query_category' => '0',
    'layout' => 'grid3',
    'enable_masonry' => 'no',
    'posts_per_page' => '9',
    'image_width' => '406',
    'image_height' => '300',
    'portfolio_layout' => 'list-post',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3209,
  'post_date' => '2014-10-17 18:56:37',
  'post_date_gmt' => '2014-10-17 18:56:37',
  'post_content' => '<h3>Left aligned layout with sepia image hover filter.</h3>
&nbsp;',
  'post_title' => '2 Column Blog',
  'post_excerpt' => '',
  'post_name' => 'blog-2-columns',
  'post_modified' => '2017-08-21 05:50:01',
  'post_modified_gmt' => '2017-08-21 05:50:01',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3209',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'sepia',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'no',
    'query_category' => '0',
    'layout' => 'grid2',
    'enable_masonry' => 'no',
    'posts_per_page' => '8',
    'portfolio_layout' => 'list-post',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3213,
  'post_date' => '2014-10-17 19:08:50',
  'post_date_gmt' => '2014-10-17 19:08:50',
  'post_content' => '<h3>To show you the beauty of Elegant, all images are disabled.</h3>
&nbsp;',
  'post_title' => 'Text Only Blog',
  'post_excerpt' => '',
  'post_name' => 'text-blog',
  'post_modified' => '2017-08-21 05:50:19',
  'post_modified_gmt' => '2017-08-21 05:50:19',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3213',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'no',
    'query_category' => '0',
    'layout' => 'grid2',
    'enable_masonry' => 'no',
    'posts_per_page' => '8',
    'hide_date' => 'yes',
    'hide_meta_author' => 'yes',
    'hide_image' => 'yes',
    'portfolio_layout' => 'list-post',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3269,
  'post_date' => '2014-10-18 02:19:28',
  'post_date_gmt' => '2014-10-18 02:19:28',
  'post_content' => '',
  'post_title' => '3 Columns',
  'post_excerpt' => '',
  'post_name' => '3-columns',
  'post_modified' => '2017-08-21 05:50:24',
  'post_modified_gmt' => '2017-08-21 05:50:24',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/portfolio-copy/',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'sepia',
    'imagefilter_applyto' => 'all',
    'display_content' => 'content',
    'portfolio_query_category' => '0',
    'portfolio_layout' => 'grid3',
    'portfolio_posts_per_page' => '15',
    'portfolio_display_content' => 'none',
    'portfolio_image_width' => '406',
    'portfolio_image_height' => '0',
    'portfolio_hide_date' => 'no',
    'portfolio_hide_meta_all' => 'no',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3273,
  'post_date' => '2014-10-18 02:23:30',
  'post_date_gmt' => '2014-10-18 02:23:30',
  'post_content' => '',
  'post_title' => 'Left 2 Columns',
  'post_excerpt' => '',
  'post_name' => 'left-2-columns',
  'post_modified' => '2017-08-21 05:50:27',
  'post_modified_gmt' => '2017-08-21 05:50:27',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3273',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'sepia',
    'imagefilter_applyto' => 'all',
    'display_content' => 'content',
    'portfolio_query_category' => '0',
    'portfolio_layout' => 'grid2',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_posts_per_page' => '14',
    'portfolio_display_content' => 'none',
    'portfolio_image_width' => '630',
    'portfolio_image_height' => '0',
    'portfolio_hide_date' => 'no',
    'portfolio_hide_meta_all' => 'no',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3231,
  'post_date' => '2014-10-17 20:20:01',
  'post_date_gmt' => '2014-10-17 20:20:01',
  'post_content' => '',
  'post_title' => 'Agency',
  'post_excerpt' => '',
  'post_name' => 'agency',
  'post_modified' => '2017-08-21 05:49:50',
  'post_modified_gmt' => '2017-08-21 05:49:50',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3231',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'full_width',
    'hide_page_title' => 'yes',
    'display_content' => 'content',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Studio</h1><h3 style=\\\\\\\\\\\\\\"text-align: left;\\\\\\\\\\\\\\">Elegant theme is a beautiful theme that is made for artists, photographers, designers, bloggers, and companies who want create a stylish creative way to showcase their site. Check out all the pages and see how this theme is perfect for you! </h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"7\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Services</h4><p>A great way to show off your skills or company services is with the Themify Progress Bar Addon. Customize the colour of the bar, font face, style, and colour it\\\\\\\\\\\'s a great way to make your site look more interesting.</p><p>[themify_button style=\\\\\\\\\\\\\\"rect black outline\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"https://themify.me/addons/progress-bar\\\\\\\\\\\\\\"]Progress Bar[/themify_button]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h3>The Progress module allows you to show team/individuals skills or the progress of an ongoing project in an animated interesting way. It comes packed with all our signature styling panel paired with a custom animation that allows users to see your progress along the bar.</h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\",\\"add_css_text\\":\\"drop-cap\\"}},{\\"mod_name\\":\\"progressbar\\",\\"mod_settings\\":{\\"progress_bars\\":[{\\"bar_label\\":\\"Wedding\\",\\"bar_percentage\\":\\"80\\",\\"bar_color\\":\\"eb2aa4_1.00\\"},{\\"bar_label\\":\\"Web Development\\",\\"bar_percentage\\":\\"90\\",\\"bar_color\\":\\"2ca5eb_1.00\\"},{\\"bar_label\\":\\"Marketing\\",\\"bar_percentage\\":\\"70\\",\\"bar_color\\":\\"a818f5_1.00\\"},{\\"bar_label\\":\\"Bakery\\",\\"bar_percentage\\":\\"85\\",\\"bar_color\\":\\"fa3f00_1.00\\"}],\\"hide_percentage_text\\":\\"yes\\",\\"background_color\\":\\"ffffff_1.00\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"padding_top\\":\\"0\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"ffffff_1.00\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>CLIENTS</h4><ul><li>Themify</li><li>Microsoft</li><li>Apple</li><li>Google</li><li>Yahoo</li><li>Firefox</li></ul>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Executives</h4><p>[themify_team_posts category=\\\\\\\\\\\\\\"executive\\\\\\\\\\\\\\" display=\\\\\\\\\\\\\\"excerpt\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"grid3\\\\\\\\\\\\\\" limit=\\\\\\\\\\\\\\"3\\\\\\\\\\\\\\" image_w=\\\\\\\\\\\\\\"300\\\\\\\\\\\\\\" image_h=\\\\\\\\\\\\\\"300\\\\\\\\\\\\\\"]</p><h4>General</h4><p>[themify_team_posts category=\\\\\\\\\\\\\\"team\\\\\\\\\\\\\\" display=\\\\\\\\\\\\\\"none\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"grid4\\\\\\\\\\\\\\" limit=\\\\\\\\\\\\\\"4\\\\\\\\\\\\\\" image_w=\\\\\\\\\\\\\\"250\\\\\\\\\\\\\\" image_h=\\\\\\\\\\\\\\"250\\\\\\\\\\\\\\"]</p><p> </p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"3\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Our Office</h4><p>123 Street Name<br /> Toronto, Ontario<br /> Canada</p><p>[themify_button style=\\\\\\\\\\\\\\"large rect outline black\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"https://themify.me/\\\\\\\\\\\\\\"]Contact us[/themify_button]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"map\\",\\"mod_settings\\":{\\"map_display_type\\":\\"dynamic\\",\\"address_map\\":\\"30 Adelaide St E, Toronto, ON, Canada\\",\\"zoom_map\\":\\"15\\",\\"w_map\\":\\"100\\",\\"unit_w\\":\\"%\\",\\"h_map\\":\\"400\\",\\"unit_h\\":\\"px\\",\\"b_style_map\\":\\"solid\\",\\"type_map\\":\\"ROADMAP\\",\\"scrollwheel_map\\":\\"disable\\",\\"draggable_map\\":\\"enable\\",\\"draggable_disable_mobile_map\\":\\"yes\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"checkbox_padding_apply_all\\":\\"padding\\",\\"checkbox_padding_apply_all_padding\\":\\"padding\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\",\\"checkbox_margin_apply_all\\":\\"margin\\",\\"checkbox_margin_apply_all_margin\\":\\"margin\\",\\"checkbox_border_apply_all\\":\\"border\\",\\"checkbox_border_apply_all_border\\":\\"border\\",\\"visibility_desktop\\":\\"show\\",\\"visibility_desktop_show\\":\\"show\\",\\"visibility_desktop_hide\\":\\"hide\\",\\"visibility_tablet\\":\\"show\\",\\"visibility_tablet_show\\":\\"show\\",\\"visibility_tablet_hide\\":\\"hide\\",\\"visibility_mobile\\":\\"show\\",\\"visibility_mobile_show\\":\\"show\\",\\"visibility_mobile_hide\\":\\"hide\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":[]},{\\"row_order\\":\\"4\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first last\\",\\"modules\\":[],\\"styling\\":[]}],\\"styling\\":[]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3234,
  'post_date' => '2014-10-17 21:06:25',
  'post_date_gmt' => '2014-10-17 21:06:25',
  'post_content' => '',
  'post_title' => 'Magazine',
  'post_excerpt' => '',
  'post_name' => 'magazine',
  'post_modified' => '2017-08-21 05:49:55',
  'post_modified_gmt' => '2017-08-21 05:49:55',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3234',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'content_width' => 'full_width',
    'hide_page_title' => 'yes',
    'display_content' => 'content',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Magazine</h1><h3>Use the Builder to create unlimited magazine-like layouts. All are done with dragging and dropping.</h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[],\\"styling\\":[]},{\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col4-1 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Follow us</h4><p>[themify_icon icon=\\\\\\\\\\\\\\"fa-twitter\\\\\\\\\\\\\\" label=\\\\\\\\\\\\\\"Twitter\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"http://twitter.com/themify\\\\\\\\\\\\\\" icon_bg=\\\\\\\\\\\\\\"#000000\\\\\\\\\\\\\\" icon_color=\\\\\\\\\\\\\\"#ffffff\\\\\\\\\\\\\\" ]<br /><br />[themify_icon icon=\\\\\\\\\\\\\\"fa-facebook\\\\\\\\\\\\\\" label=\\\\\\\\\\\\\\"Facebook\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"http://facebook.com/themify\\\\\\\\\\\\\\" icon_bg=\\\\\\\\\\\\\\"#000000\\\\\\\\\\\\\\" icon_color=\\\\\\\\\\\\\\"#ffffff\\\\\\\\\\\\\\" ]<br /><br />[themify_icon icon=\\\\\\\\\\\\\\"fa-youtube\\\\\\\\\\\\\\" label=\\\\\\\\\\\\\\"YouTube\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"http://www.youtube.com/user/themifyme\\\\\\\\\\\\\\" icon_bg=\\\\\\\\\\\\\\"#000000\\\\\\\\\\\\\\" icon_color=\\\\\\\\\\\\\\"#ffffff\\\\\\\\\\\\\\" ]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"40\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"6\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"6\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Top Stories</h4>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":[]},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2 first\\",\\"modules\\":[{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-post\\",\\"category_post\\":\\"designers|multiple\\",\\"post_per_page_post\\":\\"1\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"excerpt\\",\\"hide_post_date_post\\":\\"no\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2 last\\",\\"modules\\":[{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-thumb-image\\",\\"category_post\\":\\"designers|multiple\\",\\"post_per_page_post\\":\\"4\\",\\"offset_post\\":\\"1\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"40\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Videos</h4>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"grid4\\",\\"category_post\\":\\"videos|multiple\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_date_post\\":\\"yes\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":[]},{\\"row_order\\":\\"4\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col3-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Artists</h4>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-thumb-image\\",\\"category_post\\":\\"artists|multiple\\",\\"post_per_page_post\\":\\"3\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col3-1\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Fashion</h4>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-thumb-image\\",\\"category_post\\":\\"fashion|multiple\\",\\"post_per_page_post\\":\\"3\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col3-1 last\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Writers</h4>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-thumb-image\\",\\"category_post\\":\\"writers|multiple\\",\\"post_per_page_post\\":\\"3\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"40\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"5\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[],\\"styling\\":[]}],\\"styling\\":[]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3276,
  'post_date' => '2014-10-18 02:28:15',
  'post_date_gmt' => '2014-10-18 02:28:15',
  'post_content' => '',
  'post_title' => 'Grid Portfolio',
  'post_excerpt' => '',
  'post_name' => 'grid-portfolio',
  'post_modified' => '2017-08-21 05:50:25',
  'post_modified_gmt' => '2017-08-21 05:50:25',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3276',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'sepia',
    'imagefilter_applyto' => 'all',
    'display_content' => 'content',
    'portfolio_query_category' => '0',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_posts_per_page' => '14',
    'portfolio_display_content' => 'none',
    'portfolio_image_width' => '300',
    'portfolio_image_height' => '200',
    'portfolio_hide_date' => 'no',
    'portfolio_hide_meta_all' => 'no',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3278,
  'post_date' => '2014-10-18 02:31:58',
  'post_date_gmt' => '2014-10-18 02:31:58',
  'post_content' => '<h3></h3>',
  'post_title' => 'Left Aligned Blog',
  'post_excerpt' => '',
  'post_name' => 'left-aligned-blog',
  'post_modified' => '2017-08-21 05:50:17',
  'post_modified_gmt' => '2017-08-21 05:50:17',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3278',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'default',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'no',
    'query_category' => '0',
    'enable_masonry' => 'no',
    'posts_per_page' => '5',
    'hide_date' => 'no',
    'portfolio_layout' => 'list-post',
    'portfolio_enable_masonry' => 'yes',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3238,
  'post_date' => '2014-10-17 22:15:42',
  'post_date_gmt' => '2014-10-17 22:15:42',
  'post_content' => '',
  'post_title' => 'Features',
  'post_excerpt' => '',
  'post_name' => 'features',
  'post_modified' => '2017-08-21 05:49:48',
  'post_modified_gmt' => '2017-08-21 05:49:48',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=3238',
  'menu_order' => 1,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'full_width',
    'content_text_align' => 'default',
    'imagefilter_options' => 'grayscale',
    'imagefilter_options_hover' => 'none',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'yes',
    'background_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/146971178.jpg',
    'display_content' => 'content',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Beautiful <br />Typography</h1>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"5\\",\\"padding_right_unit\\":\\"%\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p>Sometimes creating simplest design can be the most difficult part when making your site. That\\\\\\\\\\\'s why we made the Elegant theme, simple yet elegant it removes all the the fancy animations and graphic effects and instead focuses more on your content. Crafted with minimal design and great attentions to typography, Elegant theme provides a stunning visual that displays and functions beautifully across all devices. With the bonus Portfolio and Team post type on top, Elegant is the perfect theme for writers, artists, designers, agencies, edtiorial sites, and individual portfolios.</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"10\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"6\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h3 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Parallax scrolling made easy</h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"fullheight\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant/files/2014/10/134119568.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2 first\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant/files/2014/10/176244302.jpg\\",\\"appearance_image\\":\\"|\\",\\"width_image\\":\\"600\\",\\"height_image\\":\\"450\\",\\"param_image\\":\\"|\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2>Image Filters</h2><p>Auto apply filters to your images without editing the original image source (no Photoshop needed).</p><ul><li>Grayscale</li><li>Sephia</li><li>Blur</li></ul>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h3 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Background Video</h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"fullheight\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"video\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"https://themify.me/demo/themes/corporate/files/2014/09/messier_marathon.mp4\\",\\"background_image\\":\\"https://themify.me/demo/themes/corporate/files/2014/09/messier_marathon.jpg\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"4\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Portfolio</h1><p>Portfolio offers varous masonry and grid layouts along with post filtering.</p><p>[themify_portfolio_posts limit=\\\\\\\\\\\\\\"12\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"grid4\\\\\\\\\\\\\\" image_w=\\\\\\\\\\\\\\"294\\\\\\\\\\\\\\" image_h=\\\\\\\\\\\\\\"0\\\\\\\\\\\\\\"]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"f2f2f2_1.00\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"6\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"6\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"5\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2 first\\",\\"modules\\":[{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-post\\",\\"category_post\\":\\"photography|multiple\\",\\"post_per_page_post\\":\\"1\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"excerpt\\",\\"img_width_post\\":\\"600\\",\\"img_height_post\\":\\"300\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Elegant Blog</h1>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"5\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"5\\",\\"bottom_margin_divider\\":\\"45\\"}},{\\"mod_name\\":\\"post\\",\\"mod_settings\\":{\\"layout_post\\":\\"list-thumb-image\\",\\"category_post\\":\\"designers|multiple\\",\\"post_per_page_post\\":\\"4\\",\\"order_post\\":\\"desc\\",\\"orderby_post\\":\\"date\\",\\"display_post\\":\\"none\\",\\"hide_post_meta_post\\":\\"yes\\",\\"hide_page_nav_post\\":\\"yes\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"9\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"4\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"6\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Meet</h4><h2 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\"><em>Our</em> Team</h2>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p>[themify_team_posts limit=\\\\\\\\\\\\\\"3\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"grid3\\\\\\\\\\\\\\" display=\\\\\\\\\\\\\\"none\\\\\\\\\\\\\\" image_w=\\\\\\\\\\\\\\"300\\\\\\\\\\\\\\" image_h=\\\\\\\\\\\\\\"350\\\\\\\\\\\\\\"]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"f2f2f2\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"4\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"7\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Shop</h1><p style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Use <a href=\\\\\\\\\\\\\\"https://themify.me/addons/woocommerce\\\\\\\\\\\\\\">WooCommerce</a> Addon to build shop <a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/features/shop-landing/\\\\\\\\\\\\\\">landing</a> pages.</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"products\\",\\"mod_settings\\":{\\"query_products\\":\\"all\\",\\"category_products\\":\\"0|multiple\\",\\"hide_free_products\\":\\"no\\",\\"post_per_page_products\\":\\"8\\",\\"orderby_products\\":\\"date\\",\\"order_products\\":\\"desc\\",\\"template_products\\":\\"list\\",\\"layout_products\\":\\"grid4\\",\\"layout_slider\\":\\"slider-default\\",\\"visible_opt_slider\\":\\"1\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"scroll\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"description_products\\":\\"none\\",\\"hide_feat_img_products\\":\\"no\\",\\"unlink_feat_img_products\\":\\"no\\",\\"hide_post_title_products\\":\\"no\\",\\"unlink_post_title_products\\":\\"no\\",\\"hide_price_products\\":\\"no\\",\\"hide_add_to_cart_products\\":\\"no\\",\\"hide_rating_products\\":\\"no\\",\\"hide_sales_badge\\":\\"no\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"2\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"8\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Customize</h2><p style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Customize everything from color, background, fonts, spacing, and more!</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"image\\",\\"mod_settings\\":{\\"style_image\\":\\"image-top\\",\\"url_image\\":\\"https://themify.me/demo/themes/elegant/files/2014/10/elegant-customize-panel.jpg\\",\\"appearance_image\\":\\"|\\",\\"width_image\\":\\"1265\\",\\"height_image\\":\\"732\\",\\"param_image\\":\\"|\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"000000_1.00\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"ffffff_1.00\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"4\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"9\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[],\\"styling\\":[]}],\\"styling\\":[]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2376,
  'post_date' => '2014-10-14 18:40:04',
  'post_date_gmt' => '2014-10-14 18:40:04',
  'post_content' => '',
  'post_title' => 'Blog',
  'post_excerpt' => '',
  'post_name' => 'blog',
  'post_modified' => '2017-08-21 05:49:59',
  'post_modified_gmt' => '2017-08-21 05:49:59',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=2376',
  'menu_order' => 2,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'hide_page_title' => 'yes',
    'query_category' => '0',
    'posts_per_page' => '5',
    'hide_date' => 'no',
    'hide_meta_all' => 'no',
    'portfolio_layout' => 'list-post',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2380,
  'post_date' => '2014-10-14 18:40:25',
  'post_date_gmt' => '2014-10-14 18:40:25',
  'post_content' => '<h3>The portfolio posts in this page is queried in the Themify Custom Panel. Masonry layout is enabled. Grayscale image filter is selected. When you hover the image, it will show the original image without the filter.</h3>',
  'post_title' => 'Portfolio',
  'post_excerpt' => '',
  'post_name' => 'portfolio',
  'post_modified' => '2017-08-21 05:50:22',
  'post_modified_gmt' => '2017-08-21 05:50:22',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=2380',
  'menu_order' => 3,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_text_align' => 'center',
    'imagefilter_options' => 'grayscale',
    'imagefilter_options_hover' => 'none',
    'imagefilter_applyto' => 'all',
    'display_content' => 'content',
    'portfolio_query_category' => '0',
    'portfolio_posts_per_page' => '16',
    'portfolio_image_width' => '294',
    'portfolio_image_height' => '0',
    'portfolio_hide_date' => 'no',
    'portfolio_hide_meta_all' => 'no',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2382,
  'post_date' => '2014-10-14 18:40:33',
  'post_date_gmt' => '2014-10-14 18:40:33',
  'post_content' => '',
  'post_title' => 'Contact',
  'post_excerpt' => '',
  'post_name' => 'contact',
  'post_modified' => '2017-08-21 05:50:29',
  'post_modified_gmt' => '2017-08-21 05:50:29',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=2382',
  'menu_order' => 4,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'content_text_align' => 'default',
    'display_content' => 'content',
    'portfolio_layout' => 'list-post',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-3 first\\",\\"modules\\":[{\\"mod_name\\":\\"map\\",\\"mod_settings\\":{\\"address_map\\":\\"Toronto, Ontario, Canada\\",\\"zoom_map\\":\\"8\\",\\"w_map\\":\\"100\\",\\"unit_w\\":\\"%\\",\\"h_map\\":\\"500\\",\\"unit_h\\":\\"px\\",\\"b_style_map\\":\\"solid\\",\\"type_map\\":\\"ROADMAP\\",\\"scrollwheel_map\\":\\"disable\\",\\"draggable_map\\":\\"enable\\",\\"draggable_disable_mobile_map\\":\\"yes\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1 last\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Phone</h4><p>433-344-6788</p><h4>Address</h4><p>124 Street<br />Toronto, ON<br />Canada</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":[]},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[],\\"styling\\":[]}],\\"styling\\":[]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2353,
  'post_date' => '2014-10-13 18:10:05',
  'post_date_gmt' => '2014-10-13 18:10:05',
  'post_content' => '',
  'post_title' => 'Shop',
  'post_excerpt' => '',
  'post_name' => 'shop',
  'post_modified' => '2017-08-21 05:50:31',
  'post_modified_gmt' => '2017-08-21 05:50:31',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/shop/',
  'menu_order' => 7,
  'post_type' => 'page',
  'meta_input' => 
  array (
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2431,
  'post_date' => '2014-10-16 02:25:07',
  'post_date_gmt' => '2014-10-16 02:25:07',
  'post_content' => '',
  'post_title' => 'Shop Landing',
  'post_excerpt' => '',
  'post_name' => 'shop-landing',
  'post_modified' => '2017-08-21 05:49:57',
  'post_modified_gmt' => '2017-08-21 05:49:57',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?page_id=2431',
  'menu_order' => 8,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'full_width',
    'content_text_align' => 'default',
    'imagefilter_options' => 'none',
    'imagefilter_options_hover' => 'grayscale',
    'imagefilter_applyto' => 'all',
    'hide_page_title' => 'yes',
    'background_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/200171444.jpg',
    'display_content' => 'content',
    'portfolio_layout' => 'list-post',
    'portfolio_display_content' => 'content',
    'builder_switch_frontend' => '0',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full first\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Shop</h1><h3>Use <a href=\\\\\\\\\\\\\\"https://themify.me/addons/woocommerce\\\\\\\\\\\\\\">WooCommerce Addon</a> to create magazine-like shop layouts</h3>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}},{\\"mod_name\\":\\"products\\",\\"mod_settings\\":{\\"query_products\\":\\"all\\",\\"category_products\\":\\"|single\\",\\"hide_free_products\\":\\"no\\",\\"orderby_products\\":\\"date\\",\\"order_products\\":\\"desc\\",\\"template_products\\":\\"slider\\",\\"layout_products\\":\\"list-post\\",\\"layout_slider\\":\\"slider-default\\",\\"visible_opt_slider\\":\\"5\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"scroll\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"right_margin_slider\\":\\"10\\",\\"description_products\\":\\"none\\",\\"hide_feat_img_products\\":\\"no\\",\\"unlink_feat_img_products\\":\\"no\\",\\"hide_post_title_products\\":\\"yes\\",\\"unlink_post_title_products\\":\\"no\\",\\"hide_price_products\\":\\"yes\\",\\"hide_add_to_cart_products\\":\\"yes\\",\\"hide_rating_products\\":\\"yes\\",\\"hide_sales_badge\\":\\"no\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"10\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"3\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Men</h4><p><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/accessories-men/\\\\\\\\\\\\\\">Accessories<br /></a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/long-sleeve-t-shirts/\\\\\\\\\\\\\\">Long Sleeve T-Shirts<br /></a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/shirts/\\\\\\\\\\\\\\">Shirts<br /></a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/suits-blazers/\\\\\\\\\\\\\\">Suits & Blazers<br /></a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/t-shirts/\\\\\\\\\\\\\\">T-Shirts</a></p><p>[themify_button link=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/men/\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"black outline rect small\\\\\\\\\\\\\\"]More[/themify_button]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"products\\",\\"mod_settings\\":{\\"query_products\\":\\"all\\",\\"category_products\\":\\"men|multiple\\",\\"hide_free_products\\":\\"no\\",\\"post_per_page_products\\":\\"6\\",\\"orderby_products\\":\\"date\\",\\"order_products\\":\\"desc\\",\\"template_products\\":\\"list\\",\\"layout_products\\":\\"grid3\\",\\"layout_slider\\":\\"slider-default\\",\\"visible_opt_slider\\":\\"1\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"scroll\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"description_products\\":\\"none\\",\\"hide_feat_img_products\\":\\"no\\",\\"unlink_feat_img_products\\":\\"no\\",\\"hide_post_title_products\\":\\"no\\",\\"unlink_post_title_products\\":\\"no\\",\\"hide_price_products\\":\\"no\\",\\"hide_add_to_cart_products\\":\\"no\\",\\"hide_rating_products\\":\\"no\\",\\"hide_sales_badge\\":\\"no\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1 first\\",\\"modules\\":[{\\"mod_name\\":\\"divider\\",\\"mod_settings\\":{\\"style_divider\\":\\"solid\\",\\"stroke_w_divider\\":\\"8\\",\\"color_divider\\":\\"000000_1.00\\",\\"top_margin_divider\\":\\"0\\",\\"bottom_margin_divider\\":\\"30\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4>Women</h4><p><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/women/dresses/\\\\\\\\\\\\\\">Dresses<br /> </a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/women/jumpers/\\\\\\\\\\\\\\">Jumpers<br /> </a><a href=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/women/shirts-tops/\\\\\\\\\\\\\\">Shirts & Tops<br /> </a></p><p>[themify_button link=\\\\\\\\\\\\\\"https://themify.me/demo/themes/elegant/product-category/women/\\\\\\\\\\\\\\" style=\\\\\\\\\\\\\\"black outline rect small\\\\\\\\\\\\\\"]More[/themify_button]</p>\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3 last\\",\\"modules\\":[{\\"mod_name\\":\\"products\\",\\"mod_settings\\":{\\"query_products\\":\\"all\\",\\"category_products\\":\\"women|multiple\\",\\"hide_free_products\\":\\"no\\",\\"post_per_page_products\\":\\"9\\",\\"orderby_products\\":\\"date\\",\\"order_products\\":\\"desc\\",\\"template_products\\":\\"list\\",\\"layout_products\\":\\"grid3\\",\\"layout_slider\\":\\"slider-default\\",\\"visible_opt_slider\\":\\"1\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"scroll\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"description_products\\":\\"none\\",\\"hide_feat_img_products\\":\\"no\\",\\"unlink_feat_img_products\\":\\"no\\",\\"hide_post_title_products\\":\\"no\\",\\"unlink_post_title_products\\":\\"no\\",\\"hide_price_products\\":\\"no\\",\\"hide_add_to_cart_products\\":\\"no\\",\\"hide_rating_products\\":\\"no\\",\\"hide_sales_badge\\":\\"no\\",\\"font_family\\":\\"default\\",\\"text_align_left\\":\\"left\\",\\"text_align_center\\":\\"center\\",\\"text_align_right\\":\\"right\\",\\"text_align_justify\\":\\"justify\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom_unit\\":\\"px\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left_unit\\":\\"px\\"}}],\\"styling\\":[]}],\\"column_alignment\\":\\"\\",\\"styling\\":{\\"row_width\\":\\"\\",\\"row_height\\":\\"\\",\\"animation_effect\\":\\"\\",\\"background_type\\":\\"image\\",\\"background_slider\\":\\"\\",\\"background_slider_mode\\":\\"\\",\\"background_video\\":\\"\\",\\"background_image\\":\\"\\",\\"background_repeat\\":\\"\\",\\"background_color\\":\\"\\",\\"font_family\\":\\"default\\",\\"font_color\\":\\"\\",\\"font_size\\":\\"\\",\\"font_size_unit\\":\\"\\",\\"line_height\\":\\"\\",\\"line_height_unit\\":\\"\\",\\"text_align\\":\\"\\",\\"link_color\\":\\"\\",\\"text_decoration\\":\\"\\",\\"padding_top\\":\\"\\",\\"padding_top_unit\\":\\"px\\",\\"padding_right\\":\\"\\",\\"padding_right_unit\\":\\"px\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\",\\"padding_left\\":\\"\\",\\"padding_left_unit\\":\\"px\\",\\"margin_top\\":\\"\\",\\"margin_top_unit\\":\\"px\\",\\"margin_right\\":\\"\\",\\"margin_right_unit\\":\\"px\\",\\"margin_bottom\\":\\"\\",\\"margin_bottom_unit\\":\\"px\\",\\"margin_left\\":\\"\\",\\"margin_left_unit\\":\\"px\\",\\"border_top_color\\":\\"\\",\\"border_top_width\\":\\"\\",\\"border_top_style\\":\\"\\",\\"border_right_color\\":\\"\\",\\"border_right_width\\":\\"\\",\\"border_right_style\\":\\"\\",\\"border_bottom_color\\":\\"\\",\\"border_bottom_width\\":\\"\\",\\"border_bottom_style\\":\\"\\",\\"border_left_color\\":\\"\\",\\"border_left_width\\":\\"\\",\\"border_left_style\\":\\"\\",\\"custom_css_row\\":\\"\\",\\"row_anchor\\":\\"\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[],\\"styling\\":[]}],\\"styling\\":[]}]',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2467,
  'post_date' => '2014-10-16 18:54:22',
  'post_date_gmt' => '2014-10-16 18:54:22',
  'post_content' => 'Fusce lobortis mi non vestibulum convallis. Aliquam auctor vitae nulla eu dapibus. Curabitur luctus nisl lorem. Aenean non diam vitae nisi lacinia fermentum vitae at turpis. Nulla porttitor, sem quis auctor porttitor, urna nunc consequat diam, eget imperdiet dolor diam pretium dui.',
  'post_title' => 'Flower Texture Dress',
  'post_excerpt' => '57% Polyester, 37% Acrylic, 6% Nylon',
  'post_name' => 'flower-texture-dress',
  'post_modified' => '2017-08-21 05:54:11',
  'post_modified_gmt' => '2017-08-21 05:54:11',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2467',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294723:172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '100',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '100',
    '_max_variation_price' => '100',
    '_min_price_variation_id' => '2469',
    '_max_price_variation_id' => '2469',
    '_min_variation_regular_price' => '100',
    '_max_variation_regular_price' => '100',
    '_min_regular_price_variation_id' => '2469',
    '_max_regular_price_variation_id' => '2469',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '3086',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
    'product_tag' => 'red',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2496,
  'post_date' => '2014-10-16 19:22:11',
  'post_date_gmt' => '2014-10-16 19:22:11',
  'post_content' => 'Fusce eget semper magna. Maecenas leo augue, imperdiet nec dignissim vitae, laoreet laoreet mi. Nunc dapibus mi sed magna aliquet, eget fermentum nunc eleifend. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas.',
  'post_title' => 'Textured Knit Dress',
  'post_excerpt' => '67% Cotton, 33% Acrylic',
  'post_name' => 'textured-knit-dress',
  'post_modified' => '2017-08-21 05:54:09',
  'post_modified_gmt' => '2017-08-21 05:54:09',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2496',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294722:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '89',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '89',
    '_max_variation_price' => '89',
    '_min_price_variation_id' => '2502',
    '_max_price_variation_id' => '2502',
    '_min_variation_regular_price' => '89',
    '_max_variation_regular_price' => '89',
    '_min_regular_price_variation_id' => '2502',
    '_max_regular_price_variation_id' => '2502',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '3014',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/177600644.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
    'product_tag' => 'red',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/177600644.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2517,
  'post_date' => '2014-10-16 19:37:01',
  'post_date_gmt' => '2014-10-16 19:37:01',
  'post_content' => 'Duis in vestibulum elit. Quisque vitae arcu lobortis, cursus lorem et, tincidunt diam. Nunc tincidunt risus libero, sit amet auctor est ornare congue. Fusce bibendum, orci vitae varius ornare',
  'post_title' => 'Reverse Jumper',
  'post_excerpt' => 'Regular fit - true to size
Machine wash
58% Polyester, 42% Polyamide',
  'post_name' => 'reverse-jumper',
  'post_modified' => '2017-08-21 05:54:08',
  'post_modified_gmt' => '2017-08-21 05:54:08',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2517',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294721:172',
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '69',
    '_sale_price' => '39',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '39',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_thumbnail_id' => '3018',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/178954478.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'jumpers, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/178954478.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2558,
  'post_date' => '2014-10-16 20:17:11',
  'post_date_gmt' => '2014-10-16 20:17:11',
  'post_content' => 'Cum sociis natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Maecenas sodales lacinia pulvinar. Nam rhoncus elit sed ex hendrerit ornare.',
  'post_title' => 'Cut Out T-Shirt Dress',
  'post_excerpt' => 'Machine wash
93% Polyester, 7% Elastane',
  'post_name' => 'cut-t-shirt-dress',
  'post_modified' => '2017-08-21 05:54:05',
  'post_modified_gmt' => '2017-08-21 05:54:05',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2558',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294720:172',
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '79',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '79',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_thumbnail_id' => '3021',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/179803418.jpg',
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'dresses, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/179803418.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2622,
  'post_date' => '2014-10-16 20:56:47',
  'post_date_gmt' => '2014-10-16 20:56:47',
  'post_content' => 'Etiam massa neque, egestas vitae sollicitudin sit amet, feugiat vel augue. Nunc quis odio non lacus dapibus dictum. Proin hendrerit auctor venenatis. Cras in fringilla lorem.',
  'post_title' => 'Blue Check Shirt',
  'post_excerpt' => 'Regular fit - true to size
100% Cotton',
  'post_name' => 'blue-check-shirt',
  'post_modified' => '2017-08-21 05:54:03',
  'post_modified_gmt' => '2017-08-21 05:54:03',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2622',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294720:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '1',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '49',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '49',
    '_max_variation_price' => '49',
    '_min_price_variation_id' => '2634',
    '_max_price_variation_id' => '2634',
    '_min_variation_regular_price' => '89',
    '_max_variation_regular_price' => '89',
    '_min_regular_price_variation_id' => '2634',
    '_max_regular_price_variation_id' => '2634',
    '_min_variation_sale_price' => '49',
    '_max_variation_sale_price' => '49',
    '_min_sale_price_variation_id' => '2634',
    '_max_sale_price_variation_id' => '2634',
    '_default_attributes' => 
    array (
    ),
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_thumbnail_id' => '3029',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1833135441.jpg',
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'shirts-tops, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1833135441.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2678,
  'post_date' => '2014-10-16 21:46:41',
  'post_date_gmt' => '2014-10-16 21:46:41',
  'post_content' => 'Sed posuere nunc sed magna cursus consequat. Curabitur at tempor lacus. Fusce ac venenatis nisl. Nulla dictum, nisl placerat finibus porttitor, diam dolor maximus dolor, nec ornare ligula nisi vulputate tortor.',
  'post_title' => 'Floral Dress',
  'post_excerpt' => 'Regular fit
56% Acrylic, 35% Nylon, 9% Wool',
  'post_name' => 'floral-dress',
  'post_modified' => '2017-08-21 05:53:57',
  'post_modified_gmt' => '2017-08-21 05:53:57',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2678',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294718:172',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '149',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '149',
    '_max_variation_price' => '149',
    '_min_price_variation_id' => '2699',
    '_max_price_variation_id' => '2699',
    '_min_variation_regular_price' => '149',
    '_max_variation_regular_price' => '149',
    '_min_regular_price_variation_id' => '2699',
    '_max_regular_price_variation_id' => '2699',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '3086',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2649,
  'post_date' => '2014-10-16 21:23:00',
  'post_date_gmt' => '2014-10-16 21:23:00',
  'post_content' => 'Ut condimentum rutrum quam, vitae hendrerit risus aliquet sit amet. Vestibulum vel leo id dui suscipit pharetra. Aenean non nibh velit. Vestibulum libero augue, vulputate vitae nibh id, tincidunt rutrum sem.',
  'post_title' => 'Floral Print T-Shirt',
  'post_excerpt' => 'Regular fit - true to size
100% Cotton',
  'post_name' => 'floral-print-t-shirt',
  'post_modified' => '2017-08-21 05:54:01',
  'post_modified_gmt' => '2017-08-21 05:54:01',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2649',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294719:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '79',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '79',
    '_max_variation_price' => '79',
    '_min_price_variation_id' => '2667',
    '_max_price_variation_id' => '2667',
    '_min_variation_regular_price' => '99',
    '_max_variation_regular_price' => '99',
    '_min_regular_price_variation_id' => '2667',
    '_max_regular_price_variation_id' => '2667',
    '_min_variation_sale_price' => '79',
    '_max_variation_sale_price' => '79',
    '_min_sale_price_variation_id' => '2667',
    '_max_sale_price_variation_id' => '2667',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '3183',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'shirts-tops, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3115,
  'post_date' => '2014-10-16 23:34:55',
  'post_date_gmt' => '2014-10-16 23:34:55',
  'post_content' => '',
  'post_title' => 'Long Sleeve Top',
  'post_excerpt' => 'Regular fit - true to size
Machine wash
100% Cotton',
  'post_name' => 'long-sleeve-top',
  'post_modified' => '2017-08-21 05:53:46',
  'post_modified_gmt' => '2017-08-21 05:53:46',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3115',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294713:172',
    '_edit_last' => '172',
    '_thumbnail_id' => '3116',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1576464382.jpg',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '29',
    '_sale_price' => '19',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '19',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'men, t-shirts',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1576464382.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2715,
  'post_date' => '2014-10-16 21:57:54',
  'post_date_gmt' => '2014-10-16 21:57:54',
  'post_content' => 'Vivamus in dignissim tellus, et scelerisque urna. Quisque semper nisi nec lectus sodales, sit amet faucibus arcu luctus. Proin ante velit, feugiat vitae pharetra quis, placerat eget risus.',
  'post_title' => 'Stripe in Long Sleeves',
  'post_excerpt' => 'Regular fit - true to size
Machine wash
100% Cotton',
  'post_name' => 'stripe-long-sleeves',
  'post_modified' => '2017-08-21 05:53:55',
  'post_modified_gmt' => '2017-08-21 05:53:55',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2715',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294717:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '39',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '39',
    '_max_variation_price' => '39',
    '_min_price_variation_id' => '2718',
    '_max_price_variation_id' => '2718',
    '_min_variation_regular_price' => '39',
    '_max_variation_regular_price' => '39',
    '_min_regular_price_variation_id' => '2718',
    '_max_regular_price_variation_id' => '2718',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '3027',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/183178742.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'long-sleeve-t-shirts, men',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/183178742.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2669,
  'post_date' => '2014-10-16 21:35:19',
  'post_date_gmt' => '2014-10-16 21:35:19',
  'post_content' => 'Donec porttitor facilisis turpis nec laoreet. Nulla placerat quis diam in lacinia. Phasellus consectetur leo in faucibus condimentum. Curabitur tincidunt in lacus vel sodales.',
  'post_title' => 'Black Bow Tie',
  'post_excerpt' => 'Regular fit - true to size
Hand wash
100% Polyester',
  'post_name' => 'black-bow-tie',
  'post_modified' => '2017-08-21 05:53:59',
  'post_modified_gmt' => '2017-08-21 05:53:59',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2669',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294863:172',
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '10',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '29',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '29',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_thumbnail_id' => '3061',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/195028886.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'accessories, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/195028886.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3112,
  'post_date' => '2014-10-16 23:25:54',
  'post_date_gmt' => '2014-10-16 23:25:54',
  'post_content' => 'Etiam et iaculis purus. Sed a justo est. Proin sit amet blandit mauris. Sed ac rhoncus nisi. Phasellus sem sapien, malesuada in mi quis, ultricies tincidunt felis.',
  'post_title' => 'Stripe T-Shirt',
  'post_excerpt' => 'Relaxed fit
Machine wash
100% Cotton',
  'post_name' => 'stripe-t-shirt',
  'post_modified' => '2017-08-21 05:53:47',
  'post_modified_gmt' => '2017-08-21 05:53:47',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3112',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294714:172',
    '_edit_last' => '172',
    '_thumbnail_id' => '3113',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1562947311.jpg',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '19',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '19',
    '_max_variation_price' => '19',
    '_min_price_variation_id' => '3114',
    '_max_price_variation_id' => '3114',
    '_min_variation_regular_price' => '19',
    '_max_variation_regular_price' => '19',
    '_min_regular_price_variation_id' => '3114',
    '_max_regular_price_variation_id' => '3114',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'men, t-shirts',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1562947311.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2779,
  'post_date' => '2014-10-16 22:15:49',
  'post_date_gmt' => '2014-10-16 22:15:49',
  'post_content' => 'In aliquam enim eget neque blandit facilisis. Vivamus felis lectus, facilisis hendrerit tortor a, feugiat feugiat turpis. Nam a ante ac orci sagittis lobortis aliquet quis ante. Pellentesque vel luctus magna. Morbi id aliquam felis.',
  'post_title' => 'Denim Shirt',
  'post_excerpt' => 'Made from cotton denim
Slim fit
67% Cotton, 33% Linen',
  'post_name' => 'denim-shirt',
  'post_modified' => '2017-08-21 05:53:54',
  'post_modified_gmt' => '2017-08-21 05:53:54',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=2779',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294716:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '49',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '49',
    '_max_variation_price' => '49',
    '_min_price_variation_id' => '3060',
    '_max_price_variation_id' => '3060',
    '_min_variation_regular_price' => '49',
    '_max_variation_regular_price' => '49',
    '_min_regular_price_variation_id' => '3060',
    '_max_regular_price_variation_id' => '3060',
    '_default_attributes' => 
    array (
    ),
    '_thumbnail_id' => '2969',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'men, shirts',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3101,
  'post_date' => '2014-10-16 22:42:13',
  'post_date_gmt' => '2014-10-16 22:42:13',
  'post_content' => 'Etiam vestibulum ullamcorper finibus. Proin at diam vel elit dignissim cursus. Etiam augue justo, tristique id quam vitae, condimentum ultrices eros. Vestibulum eu mauris egestas, cursus ante eu, tristique risus.',
  'post_title' => 'Button Blazer',
  'post_excerpt' => 'Slim fit
Body: 100% Wool, Body Lining: 100% Polyester',
  'post_name' => 'button-blazer',
  'post_modified' => '2017-08-21 05:53:51',
  'post_modified_gmt' => '2017-08-21 05:53:51',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3101',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294859:172',
    '_thumbnail_id' => '3108',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1972473293.jpg',
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '199',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '199',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_wc_rating_count' => 
    array (
      5 => '1',
    ),
    '_wc_average_rating' => '5.00',
    '_wc_review_count' => '1',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'men, suits-blazers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1972473293.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3109,
  'post_date' => '2014-10-16 23:12:31',
  'post_date_gmt' => '2014-10-16 23:12:31',
  'post_content' => 'Phasellus ut faucibus ante. Donec pellentesque nunc diam, in porta risus aliquam quis. Donec placerat leo faucibus, commodo mi eget, pulvinar quam.',
  'post_title' => 'Smart Shirt In Long Sleeve',
  'post_excerpt' => 'Regular fit - true to size
Machine wash
55% Cotton, 45% Polyester',
  'post_name' => 'smart-shirt-long-sleeve',
  'post_modified' => '2017-08-21 05:53:49',
  'post_modified_gmt' => '2017-08-21 05:53:49',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3109',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294715:172',
    '_thumbnail_id' => '3110',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1776006532.jpg',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '79',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '79',
    '_max_variation_price' => '79',
    '_min_price_variation_id' => '3111',
    '_max_price_variation_id' => '3111',
    '_min_variation_regular_price' => '79',
    '_max_variation_regular_price' => '79',
    '_min_regular_price_variation_id' => '3111',
    '_max_regular_price_variation_id' => '3111',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'men, shirts',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1776006532.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3118,
  'post_date' => '2014-10-16 23:43:19',
  'post_date_gmt' => '2014-10-16 23:43:19',
  'post_content' => 'Suspendisse ut bibendum dolor. Maecenas tincidunt lectus vel nunc convallis, at maximus tellus imperdiet. Aliquam vel eros neque.',
  'post_title' => 'Checked Shirt',
  'post_excerpt' => '',
  'post_name' => 'checked-shirt',
  'post_modified' => '2017-08-21 05:53:44',
  'post_modified_gmt' => '2017-08-21 05:53:44',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3118',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294856:172',
    '_edit_last' => '172',
    '_thumbnail_id' => '3119',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380662.jpg',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
      'color' => 
      array (
        'name' => 'Color',
        'value' => 'Purple | Green | Red',
        'position' => 1,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '19',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '19',
    '_max_variation_price' => '19',
    '_min_price_variation_id' => '3120',
    '_max_price_variation_id' => '3120',
    '_min_variation_regular_price' => '29',
    '_max_variation_regular_price' => '29',
    '_min_regular_price_variation_id' => '3120',
    '_max_regular_price_variation_id' => '3120',
    '_min_variation_sale_price' => '19',
    '_max_variation_sale_price' => '19',
    '_min_sale_price_variation_id' => '3120',
    '_max_sale_price_variation_id' => '3120',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
      5 => '1',
    ),
    '_wc_average_rating' => '5.00',
    '_wc_review_count' => '1',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'men, t-shirts',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380662.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3121,
  'post_date' => '2014-10-16 23:51:01',
  'post_date_gmt' => '2014-10-16 23:51:01',
  'post_content' => 'Mauris ac cursus sapien. Ut efficitur vulputate fermentum. Cras ut ex sagittis, blandit tortor in, ullamcorper dolor. Quisque eget tempus mauris',
  'post_title' => 'Acid Scarf',
  'post_excerpt' => 'Light fringing to ends
Hand wash
100% Cotton',
  'post_name' => 'acid-scarf',
  'post_modified' => '2017-08-21 05:53:43',
  'post_modified_gmt' => '2017-08-21 05:53:43',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3121',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294711:172',
    '_thumbnail_id' => '3122',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/2059733201.jpg',
    '_edit_last' => '172',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_regular_price' => '19',
    '_featured' => 'no',
    '_product_attributes' => 
    array (
    ),
    '_price' => '19',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_default_attributes' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'simple',
    'product_cat' => 'accessories-men, men',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2059733201.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3129,
  'post_date' => '2014-10-17 00:21:06',
  'post_date_gmt' => '2014-10-17 00:21:06',
  'post_content' => 'Vestibulum euismod lobortis ex, a porttitor nisi lobortis eget. In aliquam enim eget neque blandit facilisis. Vivamus felis lectus, facilisis hendrerit tortor a, feugiat feugiat turpis.',
  'post_title' => 'Floral Top',
  'post_excerpt' => 'Machine wash
100% Polyester',
  'post_name' => 'floral-top',
  'post_modified' => '2017-08-21 05:53:40',
  'post_modified_gmt' => '2017-08-21 05:53:40',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3129',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294711:172',
    '_thumbnail_id' => '3130',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '1',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '29',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '29',
    '_max_variation_price' => '29',
    '_min_price_variation_id' => '3131',
    '_max_price_variation_id' => '3131',
    '_min_variation_regular_price' => '29',
    '_max_variation_regular_price' => '29',
    '_min_regular_price_variation_id' => '3131',
    '_max_regular_price_variation_id' => '3131',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'shirts-tops, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3132,
  'post_date' => '2014-10-17 00:33:36',
  'post_date_gmt' => '2014-10-17 00:33:36',
  'post_content' => 'Nam a ante ac orci sagittis lobortis aliquet quis ante. Pellentesque vel luctus magna. Morbi id aliquam felis.

Etiam rhoncus urna quis lacinia imperdiet. Integer elementum felis in mauris dictum, eu consequat nisi congue. Phasellus id erat hendrerit, tempor odio a, ullamcorper nisl.',
  'post_title' => 'Stripe Dress',
  'post_excerpt' => '',
  'post_name' => 'stripe-dress',
  'post_modified' => '2017-08-21 05:53:37',
  'post_modified_gmt' => '2017-08-21 05:53:37',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3132',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294710:172',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
      'color' => 
      array (
        'name' => 'Color',
        'value' => 'B/W | Gray | Blue | Yellow',
        'position' => 1,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_thumbnail_id' => '3134',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '79',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '79',
    '_max_variation_price' => '79',
    '_min_price_variation_id' => '3133',
    '_max_price_variation_id' => '3133',
    '_min_variation_regular_price' => '79',
    '_max_variation_regular_price' => '79',
    '_min_regular_price_variation_id' => '3133',
    '_max_regular_price_variation_id' => '3133',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3135,
  'post_date' => '2014-10-17 00:43:15',
  'post_date_gmt' => '2014-10-17 00:43:15',
  'post_content' => 'Fusce at lacus tincidunt, elementum velit vitae, congue erat. Nullam consectetur, mi et convallis ultrices, velit ex pharetra orci, nec dignissim diam ipsum eget mauris.',
  'post_title' => 'Floral Beach Dress',
  'post_excerpt' => 'Machine wash
94% Polyester, 6% Elastane',
  'post_name' => 'floral-beach-dress',
  'post_modified' => '2017-08-21 05:53:35',
  'post_modified_gmt' => '2017-08-21 05:53:35',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3135',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294709:172',
    '_edit_last' => '172',
    '_thumbnail_id' => '3138',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '1',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '49',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '49',
    '_max_variation_price' => '49',
    '_min_price_variation_id' => '3139',
    '_max_price_variation_id' => '3139',
    '_min_variation_regular_price' => '49',
    '_max_variation_regular_price' => '49',
    '_min_regular_price_variation_id' => '3139',
    '_max_regular_price_variation_id' => '3139',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3140,
  'post_date' => '2014-10-17 00:52:22',
  'post_date_gmt' => '2014-10-17 00:52:22',
  'post_content' => 'Aenean consequat mauris non ultrices commodo. Duis sollicitudin, sem et accumsan volutpat, nunc mi pretium odio, ac sodales ante lorem quis felis. In a nunc turpis.',
  'post_title' => 'Summer Maxi Dress',
  'post_excerpt' => '100% Polyester
Lightweight semi-sheer fabric',
  'post_name' => 'summer-maxi-dress',
  'post_modified' => '2017-08-21 05:53:33',
  'post_modified_gmt' => '2017-08-21 05:53:33',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=product&#038;p=3140',
  'menu_order' => 0,
  'post_type' => 'product',
  'meta_input' => 
  array (
    '_edit_lock' => '1503294708:172',
    '_thumbnail_id' => '3141',
    'post_image' => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
    '_edit_last' => '172',
    '_product_attributes' => 
    array (
      'size' => 
      array (
        'name' => 'Size',
        'value' => 'Small | Medium | Large',
        'position' => 0,
        'is_visible' => 0,
        'is_variation' => 1,
        'is_taxonomy' => 0,
      ),
    ),
    '_visibility' => 'visible',
    '_stock_status' => 'instock',
    'total_sales' => '0',
    '_downloadable' => 'no',
    '_virtual' => 'no',
    '_featured' => 'no',
    '_price' => '89',
    '_manage_stock' => 'no',
    '_backorders' => 'no',
    '_stock' => NULL,
    '_min_variation_price' => '89',
    '_max_variation_price' => '89',
    '_min_price_variation_id' => '3142',
    '_max_price_variation_id' => '3142',
    '_min_variation_regular_price' => '89',
    '_max_variation_regular_price' => '89',
    '_min_regular_price_variation_id' => '3142',
    '_max_regular_price_variation_id' => '3142',
    '_default_attributes' => 
    array (
    ),
    '_wc_rating_count' => 
    array (
    ),
    '_wc_average_rating' => '0',
    '_wc_review_count' => '0',
    '_tax_status' => 'taxable',
    '_upsell_ids' => 
    array (
    ),
    '_crosssell_ids' => 
    array (
    ),
    '_download_limit' => '-1',
    '_download_expiry' => '-1',
    '_product_version' => '3.1.1',
    '_yoast_wpseo_content_score' => '30',
  ),
  'tax_input' => 
  array (
    'product_type' => 'variable',
    'product_cat' => 'dresses, women',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3211,
  'post_date' => '2014-10-17 19:45:25',
  'post_date_gmt' => '2014-10-17 19:45:25',
  'post_content' => 'Mauris eros magna, placerat sit amet accumsan sed, vehicula eget urna. Cras viverra libero eget risus consectetur, nec imperdiet dolor porttitor. Fusce tristique diam id nibh venenatis mollis. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas.',
  'post_title' => 'Peter Wilkinson',
  'post_excerpt' => '',
  'post_name' => 'peter-wilkinson',
  'post_modified' => '2017-08-21 05:51:21',
  'post_modified_gmt' => '2017-08-21 05:51:21',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3211',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'Designer',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'team',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/194890865.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3224,
  'post_date' => '2014-10-17 19:48:18',
  'post_date_gmt' => '2014-10-17 19:48:18',
  'post_content' => 'Test varius, ullamcorper neque sit amet, mattis dolor. Praesent malesuada, ligula sit amet lobortis molestie, erat lectus ornare ex, vel condimentum lectus libero eget magna. Aenean ultricies quam a turpis imperdiet molestie. Nulla id orci posuere, rhoncus dolor vitae, euismod ligula.',
  'post_title' => 'Mara Reid',
  'post_excerpt' => '',
  'post_name' => 'mara-reid',
  'post_modified' => '2017-08-21 05:51:19',
  'post_modified_gmt' => '2017-08-21 05:51:19',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3224',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'HR',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'team',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/192357971.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3226,
  'post_date' => '2014-10-17 19:52:21',
  'post_date_gmt' => '2014-10-17 19:52:21',
  'post_content' => 'Suspendisse dignissim, tellus nec placerat consectetur, lorem nulla euismod lacus, non congue risus lorem eu massa. Nulla maximus viverra pharetra. Vivamus feugiat, lorem sed varius efficitur, diam lorem condimentum ante, vel faucibus justo nisi non arcu. Aenean vestibulum metus vitae hendrerit placerat. Ut rutrum aliquet urna, ut vehicula risus malesuada quis. Donec a nisi nec nibh convallis vehicula.',
  'post_title' => 'Megan Croft',
  'post_excerpt' => '',
  'post_name' => 'megan-croft',
  'post_modified' => '2017-08-21 05:51:18',
  'post_modified_gmt' => '2017-08-21 05:51:18',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3226',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'Manager',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'team',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/164990474.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3227,
  'post_date' => '2014-10-17 18:00:34',
  'post_date_gmt' => '2014-10-17 18:00:34',
  'post_content' => 'Aenean convallis, tellus nec dignissim tempor, justo est aliquet sem, et bibendum mi urna et sem. Integer rutrum convallis urna, at pellentesque mi vestibulum et. Nam mattis semper quam, molestie porta tellus egestas ut. Proin sit amet lectus lorem.',
  'post_title' => 'Mike Patterson',
  'post_excerpt' => '',
  'post_name' => 'mike-patterson',
  'post_modified' => '2017-08-21 05:51:23',
  'post_modified_gmt' => '2017-08-21 05:51:23',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3227',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'CEO',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'executive',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2059733201.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3228,
  'post_date' => '2014-10-17 19:56:09',
  'post_date_gmt' => '2014-10-17 19:56:09',
  'post_content' => 'Fusce malesuada odio ac leo porta rhoncus. In condimentum fermentum libero nec tempor. Etiam neque sapien, finibus sodales dui at, vehicula gravida magna. Praesent commodo nunc a interdum euismod. Vivamus commodo bibendum sodales.',
  'post_title' => 'Cathy Monet',
  'post_excerpt' => '',
  'post_name' => 'catherine-monet',
  'post_modified' => '2017-08-21 05:51:16',
  'post_modified_gmt' => '2017-08-21 05:51:16',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3228',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'Vice President',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'executive',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/193052534.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3229,
  'post_date' => '2014-10-17 19:57:42',
  'post_date_gmt' => '2014-10-17 19:57:42',
  'post_content' => 'Nam in ante sit amet libero dictum pharetra vel ac ante. Mauris vel mauris dui. Ut ut tellus malesuada, maximus nisi eu, bibendum risus. Vestibulum ultrices ipsum sit amet ex euismod, eget rhoncus diam varius. Maecenas augue elit, faucibus eget nunc ac, luctus mollis sem.',
  'post_title' => 'Scott Young',
  'post_excerpt' => '',
  'post_name' => 'scott-young',
  'post_modified' => '2017-08-21 05:51:14',
  'post_modified_gmt' => '2017-08-21 05:51:14',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3229',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'CFO',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'executive',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1576464382.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3230,
  'post_date' => '2014-10-17 20:04:17',
  'post_date_gmt' => '2014-10-17 20:04:17',
  'post_content' => 'Tauris eros magna, placerat sit amet accumsan sed, vehicula eget urna. Cras viverra libero eget risus consectetur, nec imperdiet dolor porttitor. Fusce tristique diam id nibh venenatis mollis. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas.',
  'post_title' => 'Brian Heuman',
  'post_excerpt' => '',
  'post_name' => 'brian-heuman',
  'post_modified' => '2017-08-21 05:51:13',
  'post_modified_gmt' => '2017-08-21 05:51:13',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=team&#038;p=3230',
  'menu_order' => 0,
  'post_type' => 'team',
  'meta_input' => 
  array (
    'team_title' => 'Developer',
    'social' => '[themify_icon link="http://twitter.com/themify"  icon="fa-twitter" style="large"]
[themify_icon link="http://facebook.com/themify" icon="fa-facebook" style="large"]
',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'team-category' => 'team',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2564,
  'post_date' => '2014-10-12 17:19:49',
  'post_date_gmt' => '2014-10-12 17:19:49',
  'post_content' => '<div id="lipsum">

Vivamus ornare orci ac nisi varius gravida. Etiam facilisis ullamcorper varius. Phasellus vestibulum a odio ac facilisis. Maecenas velit purus, laoreet egestas volutpat non, varius sed sapien. Duis sagittis a dolor quis auctor. Quisque libero tellus, ullamcorper ut justo ut, pharetra accumsan nulla. Curabitur molestie placerat purus in gravida. Sed non commodo ipsum. Nunc sed est et sem iaculis faucibus ut et neque. Suspendisse mattis pharetra euismod. Proin in vestibulum velit. Maecenas a justo eu tellus ultrices sodales.

</div>
<!--more-->
<h3>Stay fashionable while keeping that professional look...</h3>
Etiam ut sem vitae nisl posuere rhoncus quis sed magna. Phasellus vehicula dolor vel nisi egestas, eu feugiat neque vehicula. Vestibulum molestie pulvinar est aliquet bibendum. Aliquam suscipit mauris sed volutpat pretium. In molestie, ex non fringilla faucibus, elit lorem semper lacus, vel dictum mi dui sit amet sem. Fusce ullamcorper dolor a augue luctus efficitur. Nam condimentum nisi et sapien posuere rutrum. Sed bibendum vulputate elit vel ullamcorper. Maecenas orci elit, scelerisque id justo eu, tristique sollicitudin nulla. Ut eu turpis risus. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Integer nec nibh hendrerit ligula vehicula volutpat id vel metus. Aenean porttitor augue ac lorem bibendum gravida. Sed non convallis nibh, a gravida velit.

Proin pulvinar semper vestibulum. Sed dignissim in justo non imperdiet. Duis consequat, elit quis pulvinar scelerisque, urna nulla varius nisi, in vulputate quam nisl convallis neque. Pellentesque mattis nulla et massa consectetur, quis tincidunt turpis porttitor. In consectetur enim arcu, ac maximus orci ultrices a. Suspendisse potenti. Quisque velit purus, aliquet sed dui non, placerat luctus mi. Duis gravida quis massa sit amet congue. Mauris ultrices odio enim, eget ultricies arcu maximus sit amet.

&nbsp;',
  'post_title' => '5 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 5 columns',
  'post_name' => '5-column-gallery',
  'post_modified' => '2017-08-21 05:52:20',
  'post_modified_gmt' => '2017-08-21 05:52:20',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=2564',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="5" size="medium" ids="3344,3130,3134,3138,3095,3096,3089,3091,3085,3078,3079,3057,3053,3039,3032,3035,3011,3000,2992,2990,2983"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'fashion',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/178349915.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/01/53654263.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/211376833.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/211462315.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/207036016.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/199648799.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/200171444.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/192449156.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/186014612.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/184335731.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/184876307.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/176083367.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/171935951.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/167123411.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/164990474.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2598,
  'post_date' => '2014-10-12 11:53:50',
  'post_date_gmt' => '2014-10-12 11:53:50',
  'post_content' => '<div id="lipsum">

Nunc pulvinar eros nec augue faucibus viverra. Aliquam gravida velit et ante mattis posuere. Vivamus vel arcu malesuada, lobortis ex ac, iaculis massa. In rhoncus dignissim facilisis. Cras et facilisis libero. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam facilisis ante a congue semper. Aenean consequat enim eu tellus accumsan, non laoreet leo venenatis. Maecenas ut erat dui. Proin sit amet enim eu lorem congue commodo eu ut metus. Vestibulum tincidunt sem ut est iaculis, ut semper mi ornare.

</div>
<!--more-->

Sed consequat accumsan porttitor. Nulla egestas id diam sit amet sodales. Pellentesque quis venenatis orci. Mauris sit amet maximus enim, nec ullamcorper velit. Nam sed pellentesque enim. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed in odio at est egestas accumsan. Integer vel consequat massa. Donec libero nisi, cursus molestie nisl sed, viverra sagittis arcu. Suspendisse maximus tortor ut felis sodales condimentum. Suspendisse vulputate lorem eleifend lobortis suscipit.

&nbsp;

&nbsp;',
  'post_title' => '8 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 8 columns',
  'post_name' => '8-column-gallery',
  'post_modified' => '2017-08-21 05:52:26',
  'post_modified_gmt' => '2017-08-21 05:52:26',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=2598',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="8" size="medium" ids="3344,3138,3130,3134,3119,3116,3141,3183,3265,3264,3092,3086,3080,3074,3049,3043,3037,3032,3026,3019,3014,3007,3001,3002,2990,2983,2986,2979,2971,2965,2414,2880,2363,2357,2368,2963,2964,2970,2976"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'photo',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/171475754.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/01/53654263.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380662.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/1576464382.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/1742809701.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/612778273.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/2084238971.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/2002890712.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/1988883051.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/192302303.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/189340448.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/185751812.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/184335731.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/183068741.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/179211281.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/177600644.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/173874377.jpg',
      22 => 'https://themify.me/demo/themes/elegant/files/2014/10/172002317.jpg',
      23 => 'https://themify.me/demo/themes/elegant/files/2014/10/172027325.jpg',
      24 => 'https://themify.me/demo/themes/elegant/files/2014/10/167123411.jpg',
      25 => 'https://themify.me/demo/themes/elegant/files/2014/10/164990474.jpg',
      26 => 'https://themify.me/demo/themes/elegant/files/2014/10/165420677.jpg',
      27 => 'https://themify.me/demo/themes/elegant/files/2014/10/164096684.jpg',
      28 => 'https://themify.me/demo/themes/elegant/files/2014/10/161184653.jpg',
      29 => 'https://themify.me/demo/themes/elegant/files/2014/10/159672662.jpg',
      30 => 'https://themify.me/demo/themes/elegant/files/2014/10/114041515.jpg',
      31 => 'https://themify.me/demo/themes/elegant/files/2014/10/1341195711.jpg',
      32 => 'https://themify.me/demo/themes/elegant/files/2014/10/user_jennifer.jpg',
      33 => 'https://themify.me/demo/themes/elegant/files/2014/10/user_jolie.jpg',
      34 => 'https://themify.me/demo/themes/elegant/files/2014/10/user_miles.jpg',
      35 => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
      36 => 'https://themify.me/demo/themes/elegant/files/2014/10/159543098.jpg',
      37 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      38 => 'https://themify.me/demo/themes/elegant/files/2014/10/162619364.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2637,
  'post_date' => '2014-10-16 21:07:22',
  'post_date_gmt' => '2014-10-16 21:07:22',
  'post_content' => 'Quisque dui ipsum, imperdiet non urna at, gravida sodales velit. Donec finibus sit amet libero id facilisis. Morbi leo neque, varius nec tempor a, dignissim a felis. Mauris feugiat mauris a purus interdum rutrum. Ut eu quam sit amet tortor molestie ornare. Aliquam tincidunt interdum libero, euismod fermentum ante commodo. Suspendisse vulputate leo vel lorem maximus sodales a vitae erat. Vestibulum aliquet placerat augue ac malesuada. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut dignissim ipsum vitae consectetur vehicula. Nunc suscipit condimentum nisl, nec dictum augue. Integer sapien neque, eleifend sed ante sed, sollicitudin iaculis risus. Suspendisse venenatis dolor massa, ut tincidunt tortor pharetra et. Suspendisse potenti. In hac habitasse platea dictumst. Maecenas maximus et felis eget auctor. Proin sit amet nisi tincidunt, ultricies erat in, cursus nunc.',
  'post_title' => 'Grayscale Filter',
  'post_excerpt' => 'Grayscale image filter with hover to show original',
  'post_name' => 'grayscale-filter',
  'post_modified' => '2017-08-21 05:52:06',
  'post_modified_gmt' => '2017-08-21 05:52:06',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=2637',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="4" size="large" ids="3138,3134,2965,2971,2970,2973,2979,2976,2991,2990,2988,3005,3008,3019,3020,3009,3024"]',
    'imagefilter_options' => 'grayscale',
    'imagefilter_options_hover' => 'none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'photo',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/159672662.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/161184653.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/161370467.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/164096684.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/162619364.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/167256881.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/167123411.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/165819980.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/173247353.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/174280970.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/179211281.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/179331083.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/174301409.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/181551398.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2717,
  'post_date' => '2014-10-15 21:57:17',
  'post_date_gmt' => '2014-10-15 21:57:17',
  'post_content' => 'Curabitur nisl mauris, aliquet et lectus at, rutrum accumsan urna. Sed gravida metus ut ex convallis dictum. Ut in tincidunt nulla. In ornare, risus id rhoncus aliquam, nulla lacus maximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum.

<!--more-->Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida. Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus.',
  'post_title' => 'Blur Image Filter',
  'post_excerpt' => 'This post has blur image filter with hover none',
  'post_name' => 'blur-image-filter',
  'post_modified' => '2017-08-21 05:52:08',
  'post_modified_gmt' => '2017-08-21 05:52:08',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=2717',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="5" size="large" ids="3092,3090,3085,3089,3084,3078,3072,3086,3067,3057,3041,3036,3037,3038,3025,3019,3016,2970,2967"]',
    'imagefilter_options' => 'blur',
    'imagefilter_options_hover' => 'none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/168015233.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/2084238971.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/205973320.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/2026087631.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/199648799.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/198781790.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/196663985.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/187777541.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/185056664.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/185751812.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/185809034.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/182895254.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/179211281.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/178349915.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/160533962.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 2666,
  'post_date' => '2014-10-17 02:48:42',
  'post_date_gmt' => '2014-10-17 02:48:42',
  'post_content' => '<!--themify_builder_static--><h1>Builder Project</h1><h3>Porfolio Page Completely Customized and Constructed Using the Builder</h3>
 <h1 style="text-align: right;">01</h1><h4 style="text-align: right;">Background</h4>
 <p>Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.</p><a href="https://themify.me">Launch Site</a>
 <h1 style="text-align: center;">02</h1><h4 style="text-align: center;">This is Where It All Begins</h4>
 <p style="text-align: center;">Faelis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.</p>
 <ul data-id="slider-0-" data-visible="4" data-scroll="1" data-auto-scroll="4" data-speed="1" data-wrap="yes" data-arrow="yes" data-pagination="yes" data-effect="continuously" data-height="variable" data-pause-on-hover="resume" > 
 <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/2024380661-400x400.jpg" width="400" height="400" alt="202438066" srcset="https://themify.me/demo/themes/elegant/files/2014/10/2024380661-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/2024380661-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/2024380661-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/2024380661-800x800.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/2024380661-1024x683-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/1988883051-400x400.jpg" width="400" height="400" alt="198888305" srcset="https://themify.me/demo/themes/elegant/files/2014/10/1988883051-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/1988883051-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/1988883051-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/1988883051-800x800.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/1988883051-1024x682-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/2002890712-400x400.jpg" width="400" height="400" alt="200289071" srcset="https://themify.me/demo/themes/elegant/files/2014/10/2002890712-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/2002890712-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/2002890712-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/2002890712-800x800.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/2002890712-1024x682-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/195028886-400x400.jpg" width="400" height="400" alt="195028886" srcset="https://themify.me/demo/themes/elegant/files/2014/10/195028886-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/195028886-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/195028886-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/195028886-800x798.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/195028886-1024x680-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/194739149-400x400.jpg" width="400" height="400" alt="194739149" srcset="https://themify.me/demo/themes/elegant/files/2014/10/194739149-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/194739149-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/194739149-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/194739149-800x800.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/194739149-1024x682-1024x1024.jpg 1024w, https://themify.me/demo/themes/elegant/files/2014/10/194739149-1024x682-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> <li> <img src="https://themify.me/demo/themes/elegant/files/2014/10/180333233-400x400.jpg" width="400" height="400" alt="180333233" srcset="https://themify.me/demo/themes/elegant/files/2014/10/180333233-400x400.jpg 400w, https://themify.me/demo/themes/elegant/files/2014/10/180333233-150x150.jpg 150w, https://themify.me/demo/themes/elegant/files/2014/10/180333233-500x500.jpg 500w, https://themify.me/demo/themes/elegant/files/2014/10/180333233-800x797.jpg 800w, https://themify.me/demo/themes/elegant/files/2014/10/180333233-1024x680-1024x1024.jpg 1024w, https://themify.me/demo/themes/elegant/files/2014/10/180333233-1024x680-300x300.jpg 300w" sizes="(max-width: 400px) 100vw, 400px" /> </li> </ul> 
 <p style="text-align: center;">Faelis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.</p>
 <h4 style="text-align: center;">Moving forward&#8230;</h4>
[gallery _orderbyfield="menu_order ID" _orderByField="menu_order ID" ids="3138,3089,3079,3072,3016,3019,2988,2943"]
 <h2>03 <em>Strategy</em></h2><p>Donec convallis luctus convallis. Proin gravida porttitor sem eget interdum. Donec fermentum orci nisi. Nunc id imperdiet metus. Nulla quis facilisis mauris, non consequat quam. Quisque ut dictum magna. Suspendisse aliquam at arcu sit amet hendrerit. Nam vel risus ut felis rutrum interdum. Nam sapien arcu, mattis eu diam eleifend, dictum tincidunt ante. Proin venenatis quis purus non tempor. Ut nibh nunc, mollis commodo pulvinar in, auctor sit amet orci. Suspendisse ligula metus, viverra ac rhoncus eu, consectetur quis odio.</p><p>Interdum et malesuada fames ac ante ipsum primis in faucibus. Aliquam eget placerat odio, vitae imperdiet urna. Mauris eu ligula eros. Curabitur semper non velit vitae consequat. Aliquam ut ipsum dui. Suspendisse scelerisque nulla non vehicula ullamcorper. Morbi cursus erat sed quam malesuada blandit.</p><p> </p>
 <h4 style="text-align: center;">Finally&#8230;</h4>
 <h1 style="text-align: center;"><em>Final</em></h1><p style="text-align: center;"><a href="https://themify.me/addons/counter">Buy Counter Addon</a> </p><p style="text-align: center;">Show off your numbers in an elegant and unique way by purchasing our Counter Addon. Whether it&#8217;s statistics about your company, followers, profit, or concerts performed the counting animation will make your numbers stand out. Check it below:</p>
 
 
 Traffic Increase
 
 
 
 Followers
 
 
 
 Performances
 
 
 
 Bill Income<!--/themify_builder_static-->',
  'post_title' => 'Builder Project',
  'post_excerpt' => 'The entire portfolio layout is designed with the Builder',
  'post_name' => 'builder-project',
  'post_modified' => '2018-05-10 21:30:11',
  'post_modified_gmt' => '2018-05-10 21:30:11',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=2666',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'content_width' => 'full_width',
    'content_text_align' => 'default',
    'gallery_shortcode' => '[gallery columns="6" size="medium" ids="3134,3054,3057,3141,3130,3096,3100,2987,3089,3085,3080,2962,2963,2974,2970,2981,3048"]',
    'hide_post_title' => 'yes',
    'hide_post_date' => 'yes',
    'hide_post_meta' => 'yes',
    '_themify_builder_settings_json' => '[{\\"row_order\\":\\"0\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1>Builder Project<\\\\/h1><h3>Porfolio Page Completely Customized and Constructed Using the Builder<\\\\/h3>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"7\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"2\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"1\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">01<\\\\/h1><h4 style=\\\\\\\\\\\\\\"text-align: right;\\\\\\\\\\\\\\">Background<\\\\/h4>\\",\\"padding_right\\":\\"5\\",\\"padding_right_unit\\":\\"%\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-3\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p>Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.<\\\\/p><p>[themify_button style=\\\\\\\\\\\\\\"rect outline black\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"https:\\\\/\\\\/themify.me\\\\\\\\\\\\\\"]Launch Site[\\\\/themify_button]<\\\\/p>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"10\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"2\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\"}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/161176295.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\",\\"padding_top\\":\\"20\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"20\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"3\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">02<\\\\/h1><h4 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">This is Where It All Begins<\\\\/h4>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"4\\",\\"padding_top_unit\\":\\"%\\"}},{\\"row_order\\":\\"4\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Faelis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.<\\\\/p>\\"}}]}]},{\\"row_order\\":\\"5\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"slider\\",\\"mod_settings\\":{\\"layout_display_slider\\":\\"image\\",\\"blog_category_slider\\":\\"|single\\",\\"slider_category_slider\\":\\"|single\\",\\"portfolio_category_slider\\":\\"|single\\",\\"testimonial_category_slider\\":\\"|single\\",\\"order_slider\\":\\"desc\\",\\"orderby_slider\\":\\"date\\",\\"display_slider\\":\\"content\\",\\"img_content_slider\\":[{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/2024380661.jpg\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/1988883051.jpg\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/2002890712.jpg\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/195028886.jpg\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/194739149.jpg\\"},{\\"img_url_slider\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/180333233.jpg\\"}],\\"layout_slider\\":\\"slider-default\\",\\"img_w_slider\\":\\"400\\",\\"img_h_slider\\":\\"400\\",\\"visible_opt_slider\\":\\"4\\",\\"auto_scroll_opt_slider\\":\\"4\\",\\"scroll_opt_slider\\":\\"1\\",\\"speed_opt_slider\\":\\"normal\\",\\"effect_slider\\":\\"continuously\\",\\"pause_on_hover_slider\\":\\"resume\\",\\"wrap_slider\\":\\"yes\\",\\"show_nav_slider\\":\\"yes\\",\\"show_arrow_slider\\":\\"yes\\",\\"left_margin_slider\\":\\"5\\",\\"right_margin_slider\\":\\"5\\"}},{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<p style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Faelis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.<\\\\/p>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"3\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"6\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Moving forward...<\\\\/h4>\\"}}]}],\\"styling\\":{\\"row_height\\":\\"fullheight\\",\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/214781395.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\",\\"background_color\\":\\"afcacc_1.00\\",\\"font_color\\":\\"ffffff_1.00\\",\\"padding_top_unit\\":\\"%\\",\\"margin_top_unit\\":\\"%\\",\\"margin_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"7\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"gallery\\",\\"mod_settings\\":{\\"layout_gallery\\":\\"showcase\\",\\"shortcode_gallery\\":\\"[gallery _orderbyfield=\\\\\\\\\\\\\\"menu_order ID\\\\\\\\\\\\\\" _orderByField=\\\\\\\\\\\\\\"menu_order ID\\\\\\\\\\\\\\" ids=\\\\\\\\\\\\\\"3138,3089,3079,3072,3016,3019,2988,2943\\\\\\\\\\\\\\"]\\",\\"thumb_w_gallery\\":\\"100\\",\\"thumb_h_gallery\\":\\"100\\",\\"padding_top\\":\\"5\\",\\"padding_top_unit\\":\\"%\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-2\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h2>03 <em>Strategy<\\\\/em><\\\\/h2><p>Donec convallis luctus convallis. Proin gravida porttitor sem eget interdum. Donec fermentum orci nisi. Nunc id imperdiet metus. Nulla quis facilisis mauris, non consequat quam. Quisque ut dictum magna. Suspendisse aliquam at arcu sit amet hendrerit. Nam vel risus ut felis rutrum interdum. Nam sapien arcu, mattis eu diam eleifend, dictum tincidunt ante. Proin venenatis quis purus non tempor. Ut nibh nunc, mollis commodo pulvinar in, auctor sit amet orci. Suspendisse ligula metus, viverra ac rhoncus eu, consectetur quis odio.<\\\\/p><p>Interdum et malesuada fames ac ante ipsum primis in faucibus. Aliquam eget placerat odio, vitae imperdiet urna. Mauris eu ligula eros. Curabitur semper non velit vitae consequat. Aliquam ut ipsum dui. Suspendisse scelerisque nulla non vehicula ullamcorper. Morbi cursus erat sed quam malesuada blandit.<\\\\/p><p> <\\\\/p>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"8\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"8\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h4 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Finally...<\\\\/h4>\\"}}]}],\\"styling\\":{\\"row_height\\":\\"fullheight\\",\\"background_type\\":\\"image\\",\\"background_image\\":\\"https://themify.me/demo/themes/elegant\\\\/files\\\\/2014\\\\/10\\\\/183068741.jpg\\",\\"background_repeat\\":\\"builder-parallax-scrolling\\",\\"background_color\\":\\"b8e0d2_1.00\\",\\"font_color\\":\\"ffffff_1.00\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"9\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\",\\"modules\\":[{\\"mod_name\\":\\"text\\",\\"mod_settings\\":{\\"content_text\\":\\"<h1 style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\"><em>Final<\\\\/em><\\\\/h1><p style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">[themify_button style=\\\\\\\\\\\\\\"large rect outline black\\\\\\\\\\\\\\" link=\\\\\\\\\\\\\\"https:\\\\/\\\\/themify.me\\\\/addons\\\\/counter\\\\\\\\\\\\\\"]Buy Counter Addon[\\\\/themify_button] <\\\\/p><p style=\\\\\\\\\\\\\\"text-align: center;\\\\\\\\\\\\\\">Show off your numbers in an elegant and unique way by purchasing our Counter Addon. Whether it\\\\\\\\\\\'s statistics about your company, followers, profit, or concerts performed the counting animation will make your numbers stand out. Check it below:<\\\\/p>\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top\\":\\"8\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"10\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"500%\\",\\"label_counter\\":\\"Traffic Increase\\",\\"size_counter\\":\\"large\\",\\"font_family\\":\\"Marvel\\",\\"font_size\\":\\"30\\"}}]},{\\"column_order\\":\\"1\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"60K\\",\\"label_counter\\":\\"Followers\\",\\"size_counter\\":\\"large\\",\\"font_family\\":\\"Ubuntu\\",\\"font_size\\":\\"30\\"}}]},{\\"column_order\\":\\"2\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"37\\",\\"label_counter\\":\\"Performances\\",\\"size_counter\\":\\"large\\",\\"font_family\\":\\"Rationale\\",\\"font_size\\":\\"30\\"}}]},{\\"column_order\\":\\"3\\",\\"grid_class\\":\\"col4-1\\",\\"modules\\":[{\\"mod_name\\":\\"counter\\",\\"mod_settings\\":{\\"number_counter\\":\\"$4\\",\\"label_counter\\":\\"Bill Income\\",\\"size_counter\\":\\"large\\",\\"font_family\\":\\"Sniglet\\",\\"font_size\\":\\"30\\"}}]}],\\"styling\\":{\\"background_type\\":\\"image\\",\\"padding_top_unit\\":\\"%\\",\\"padding_bottom\\":\\"5\\",\\"padding_bottom_unit\\":\\"%\\"}},{\\"row_order\\":\\"11\\",\\"cols\\":[{\\"column_order\\":\\"0\\",\\"grid_class\\":\\"col-full\\"}]}]',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'featured',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/196169018.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/192995783.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/211462315.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/214781395.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/165574220.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/2002890712.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/158788109.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/161674283.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/164404166.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/1921473141.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3182,
  'post_date' => '2014-10-14 17:05:03',
  'post_date_gmt' => '2014-10-14 17:05:03',
  'post_content' => 'Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.

<!--more-->Donec convallis luctus convallis. Proin gravida porttitor sem eget interdum. Donec fermentum orci nisi. Nunc id imperdiet metus. Nulla quis facilisis mauris, non consequat quam. Quisque ut dictum magna. Suspendisse aliquam at arcu sit amet hendrerit. Nam vel risus ut felis rutrum interdum. Nam sapien arcu, mattis eu diam eleifend, dictum tincidunt ante. Proin venenatis quis purus non tempor. Ut nibh nunc, mollis commodo pulvinar in, auctor sit amet orci. Suspendisse ligula metus, viverra ac rhoncus eu, consectetur quis odio.',
  'post_title' => 'Sepia Filter',
  'post_excerpt' => 'This portfolio post uses a sepia image filter with no hover filter',
  'post_name' => 'sepia-filter',
  'post_modified' => '2017-08-21 05:52:10',
  'post_modified_gmt' => '2017-08-21 05:52:10',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3182',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="6" size="medium" ids="3092,3085,3075,3082,3022,3026,3020,3003,3005,2996,2994,2991,2990,2967,2977,2964,2992"]',
    'imagefilter_options' => 'sepia',
    'imagefilter_options_hover' => 'none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'featured',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/2002890712.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/2084238971.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/199095755.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/202364299.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/180333233.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/183068741.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/179331083.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/172632767.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/173247353.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/169284398.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/1688089642.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/167256881.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/167123411.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/160533962.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/163372622.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/159543098.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3187,
  'post_date' => '2014-10-13 16:16:46',
  'post_date_gmt' => '2014-10-13 16:16:46',
  'post_content' => 'Curabitur nisl mauris, aliquet et lectus at, rutrum accumsan urna. Sed gravida metus ut ex convallis dictum. Ut in tincidunt nulla. In ornare, risus id rhoncus aliquam, nulla lacus maximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.

<!--more-->Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.

&nbsp;',
  'post_title' => 'Blur Hover Filter',
  'post_excerpt' => 'Blur hover image filter',
  'post_name' => 'blur-hover-filter',
  'post_modified' => '2017-08-21 05:52:13',
  'post_modified_gmt' => '2017-08-21 05:52:13',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3187',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="6" _orderbyfield="rand" size="medium" ids="3141,3138,3130,3096,3089,3086,3078,3082,3019,3020,2987,2988,3092,3090,3075,3074,3070,3065,3047,3041,3046,2994" orderby="rand"]',
    'imagefilter_options_hover' => 'blur',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/179211281.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/179331083.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/1476875542.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/202364299.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/187777541.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/2084238971.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/165574220.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/190754081.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/211462315.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/196169018.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/199648799.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/191272349.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/199095755.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/1688089642.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/165819980.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/205973320.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/1988883051.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/1974330801.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3193,
  'post_date' => '2014-10-13 17:41:21',
  'post_date_gmt' => '2014-10-13 17:41:21',
  'post_content' => 'Proin vestibulum ultrices gravida ximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.

<!--more-->Alkliquam nisl odio, congue non sagittis a, viverra et sapien.Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.

&nbsp;',
  'post_title' => 'Grayscale Hover Filter',
  'post_excerpt' => 'Grayscale hover image filter',
  'post_name' => 'grayscale-hover-filter',
  'post_modified' => '2017-08-21 05:52:12',
  'post_modified_gmt' => '2017-08-21 05:52:12',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3193',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="6" _orderbyfield="rand" size="medium" ids="3088,3065,3010,3039,2970,2963,3183,2999,3014,2996,3130,3134,3093,3085,3080,3082,3068,3063,3062,3057,3032,3011,3012"]',
    'imagefilter_options_hover' => 'grayscale',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'fashion',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/190736456.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/2050088201.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/196169018.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/175582337.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/186014612.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/171688262.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/177600644.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/169284398.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/1828952541.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/2096474412.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/2002890712.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/202364299.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/1969481661.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/1950290482.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/195029000.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/184335731.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/176083367.jpg',
      22 => 'https://themify.me/demo/themes/elegant/files/2014/10/176219729.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3197,
  'post_date' => '2014-10-13 15:54:26',
  'post_date_gmt' => '2014-10-13 15:54:26',
  'post_content' => 'Aenean leo arcu, proin vestibulum ultrices gravida ximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis.Alkliquam nisl odio, congue non sagittis a, viverra et sapien.Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus.Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.

<!--more-->Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.

&nbsp;',
  'post_title' => 'Sephia Hover Filter',
  'post_excerpt' => 'Sephia hover image filter',
  'post_name' => 'sephia-hover-filter',
  'post_modified' => '2017-08-21 05:52:15',
  'post_modified_gmt' => '2017-08-21 05:52:15',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3197',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="6" _orderbyfield="rand" size="medium" ids="2880,3265,3183,3108,3110,3090,3083,3081,3071,3066,3061,3067,3053,3050,3034,3022,3021,3018,2989,2995,2997,2985,2982,2969,2966"]',
    'imagefilter_options_hover' => 'sepia',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'photo',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/1972473293.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/1341195711.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/1742809701.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/1972473293.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/1776006532.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/205973320.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380661.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/201732071.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/198699503.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/196537895.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/195028886.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/196663985.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/192449156.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/192336203.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/1848476301.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/180333233.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/179803418.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/178954478.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/166529993.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/1690318971.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/170155304.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/165362357.jpg',
      22 => 'https://themify.me/demo/themes/elegant/files/2014/10/164961167.jpg',
      23 => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
      24 => 'https://themify.me/demo/themes/elegant/files/2014/10/160129040.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3198,
  'post_date' => '2014-10-12 20:01:47',
  'post_date_gmt' => '2014-10-12 20:01:47',
  'post_content' => 'Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Aenean leo arcu, proin vestibulum ultrices gravida ximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis.Alkliquam nisl odio, congue non sagittis a, viverra et sapien.Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales.

<!--more-->Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus.Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida. Proin vestibulum ultrices gravida Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.

&nbsp;',
  'post_title' => '3 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 3 columns with no image filter',
  'post_name' => '3-column-gallery',
  'post_modified' => '2017-08-21 05:52:17',
  'post_modified_gmt' => '2017-08-21 05:52:17',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3198',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery _orderbyfield="rand" size="large" ids="3116,3119,3110,3108,3113,3059,3058,3050,3051,3022,3021,3018,2993,2995,2985,2969"]',
    'imagefilter_options_hover' => 'none',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'featured',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/203779864.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/1576464382.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/2024380662.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/1776006532.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/1972473293.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/1562947311.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/194890865.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/194761766.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/192336203.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/192352658.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/180333233.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/179803418.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/178954478.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/168015233.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/1690318971.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/165362357.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3200,
  'post_date' => '2014-10-12 15:11:52',
  'post_date_gmt' => '2014-10-12 15:11:52',
  'post_content' => '<div id="lipsum">

Maecenas ut erat dui. Proin sit amet enim eu lorem congue commodo eu ut metus. Vestibulum tincidunt sem ut est iaculis, ut semper mi ornare. Nunc pulvinar eros nec augue faucibus viverra. Aliquam gravida velit et ante mattis posuere. Vivamus vel arcu malesuada, lobortis ex ac, iaculis massa. In rhoncus dignissim facilisis. Cras et facilisis libero. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam facilisis ante a congue semper. Aenean consequat enim eu tellus accumsan, non laoreet leo venenatis. Maecenas ut erat dui.

</div>
<!--more-->

Proin sit amet enim eu lorem congue commodo eu ut metus. Vestibulum tincidunt sem ut est iaculis, ut semper mi ornare. Sed consequat accumsan porttitor. Nulla egestas id diam sit amet sodales. Pellentesque quis venenatis orci. Mauris sit amet maximus enim, nec ullamcorper velit. Nam sed pellentesque enim. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed in odio at est egestas accumsan. Integer vel consequat massa. Donec libero nisi, cursus molestie nisl sed, viverra sagittis arcu. Suspendisse maximus tortor ut felis sodales condimentum. Suspendisse vulputate lorem eleifend lobortis suscipit.

&nbsp;

&nbsp;',
  'post_title' => '6 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 6 columns',
  'post_name' => '6-column-gallery',
  'post_modified' => '2017-08-21 05:52:22',
  'post_modified_gmt' => '2017-08-21 05:52:22',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3200',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="6" size="medium" ids="3053,3040,2978,2969,3080,3068,3054,3049,2964,3096,2992,2988,2365,2413,2963,2965,2414,2967,2970,2976"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/165362357.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/192449156.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/187290671.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/163983824.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/160846136.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/2002890712.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/1969481661.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/192995783.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/192302303.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/159543098.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/211462315.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/165819980.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/demo_paria.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/192147314.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/159672662.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/114041515.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/160533962.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/161176295.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/162619364.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3203,
  'post_date' => '2014-10-12 09:25:08',
  'post_date_gmt' => '2014-10-12 09:25:08',
  'post_content' => '<div id="lipsum">

Maecenas ut erat dui. Proin sit amet enim eu lorem congue commodo eu ut metus. Vestibulum tincidunt sem ut est iaculis, ut semper mi ornare. Nunc pulvinar eros nec augue faucibus viverra. Aliquam gravida velit et ante mattis posuere. Vivamus vel arcu malesuada, lobortis ex ac, iaculis massa. In rhoncus dignissim facilisis. Cras et facilisis libero. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nam facilisis ante a congue semper. Aenean consequat enim eu tellus accumsan, non laoreet leo venenatis.Proin sit ame Maecenas ut erat dui. Proin sit amet enim eu lorem congue commodo eu ut metus. Vestibulum tincidunt sem ut est iaculis, ut semper mi ornare.

</div>
<!--more-->

Suspendisse vulputate lorem eleifend lobortis suscipit. Nulla egestas id diam sit amet sodales. Pellentesque quis venenatis orci. Mauris sit amet maximus enim, nec ullamcorper velit. Nam sed pellentesque enim. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Sed in odio at est egestas accumsan. Integer vel consequat massa. Donec libero nisi, cursus molestie nisl sed, viverra sagittis arcu. Suspendisse maximus tortor ut felis sodales condimentum. Suspendisse vulputate lorem eleifend lobortis suscipit.Sed consequat accumsan porttitor.

&nbsp;

&nbsp;',
  'post_title' => '9 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 9 columns',
  'post_name' => '9-column-gallery-2',
  'post_modified' => '2017-08-21 05:52:27',
  'post_modified_gmt' => '2017-08-21 05:52:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3203',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="9" size="medium" ids="3033,3030,3040,3039,3043,3037,3036,3027,2967,2966,2972,2973,2979,2978,2984,2992,2997,3002,3009,3010,3016,3015,3021,3020,3025,3029,3041,3035,3038,3052,3050,3042,3073,3078,3076,3075,3074,3082,3089,3130,3183" orderby="rand"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'photo',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/1613091171.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/192357971.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/170155304.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/827412792.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/199110713.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/175582337.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/165191423.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/183178742.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/185809034.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/183592982.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/164096684.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/186014612.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/160129040.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/184876307.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/178349915.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/1988833431.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/161370467.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/1833135441.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/174301409.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/179803418.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
      22 => 'https://themify.me/demo/themes/elegant/files/2014/10/185056664.jpg',
      23 => 'https://themify.me/demo/themes/elegant/files/2014/10/182895254.jpg',
      24 => 'https://themify.me/demo/themes/elegant/files/2014/10/1988883051.jpg',
      25 => 'https://themify.me/demo/themes/elegant/files/2014/10/199095755.jpg',
      26 => 'https://themify.me/demo/themes/elegant/files/2014/10/185751812.jpg',
      27 => 'https://themify.me/demo/themes/elegant/files/2014/10/1776006531.jpg',
      28 => 'https://themify.me/demo/themes/elegant/files/2014/10/160533962.jpg',
      29 => 'https://themify.me/demo/themes/elegant/files/2014/10/1878266571.jpg',
      30 => 'https://themify.me/demo/themes/elegant/files/2014/10/192336203.jpg',
      31 => 'https://themify.me/demo/themes/elegant/files/2014/10/163983824.jpg',
      32 => 'https://themify.me/demo/themes/elegant/files/2014/10/189340448.jpg',
      33 => 'https://themify.me/demo/themes/elegant/files/2014/10/187290671.jpg',
      34 => 'https://themify.me/demo/themes/elegant/files/2014/10/187777541.jpg',
      35 => 'https://themify.me/demo/themes/elegant/files/2014/10/202364299.jpg',
      36 => 'https://themify.me/demo/themes/elegant/files/2014/10/199648799.jpg',
      37 => 'https://themify.me/demo/themes/elegant/files/2014/10/179331083.jpg',
      38 => 'https://themify.me/demo/themes/elegant/files/2014/10/1289128971.jpg',
      39 => 'https://themify.me/demo/themes/elegant/files/2014/10/172027325.jpg',
      40 => 'https://themify.me/demo/themes/elegant/files/2014/10/184756262.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3205,
  'post_date' => '2014-10-12 19:36:11',
  'post_date_gmt' => '2014-10-12 19:36:11',
  'post_content' => '<div id="lipsum">

Curabitur nisl mauris, aliquet et lectus at, rutrum accumsan urna. Sed gravida metus ut ex convallis dictum. Ut in tincidunt nulla. In ornare, risus id rhoncus aliquam, nulla lacus maximus velit, non fringilla elit elit sit amet lacus. Etiam interdum dui odio, et fermentum felis facilisis non. Pellentesque eleifend luctus massa at vestibulum. Aliquam nisl odio, congue non sagittis a, viverra et sapien. Quisque ut magna facilisis, consequat mauris non, finibus felis. Aenean leo arcu, tincidunt at nisl tempus, aliquet tristique enim. Donec ullamcorper justo non turpis feugiat fringilla. Integer lacinia diam odio, non finibus lorem convallis sit amet. Proin vestibulum ultrices gravida.

</div>
<!--more-->

Nulla pellentesque lectus ac augue malesuada condimentum. Cras pellentesque risus a condimentum sodales. Quisque nunc ipsum, posuere sagittis ex mollis, ultricies lacinia tellus. Etiam dictum enim ac ipsum tristique dapibus. Aenean venenatis justo non enim scelerisque, sed pellentesque nisl facilisis. Quisque eget venenatis justo. In vestibulum orci maximus, feugiat arcu nec, volutpat dolor. Curabitur interdum convallis lectus at aliquam. Cras quis varius lorem. Nulla mattis maximus sapien aliquam eleifend.',
  'post_title' => '4 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 4 columns',
  'post_name' => '4-column-gallery',
  'post_modified' => '2017-08-21 05:52:19',
  'post_modified_gmt' => '2017-08-21 05:52:19',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3205',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="4" size="large" ids="3039,3037,3344,3141,3096,3089,3082,3086,3072,3057,3085,3054" orderby="rand"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'fashion',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/164404166.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/202637440.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/01/53654263.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/202364299.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/186014612.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/192995783.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/1485127491.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/202812511.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/198781790.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/185751812.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/211462315.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/205462138.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 3207,
  'post_date' => '2014-10-12 12:42:23',
  'post_date_gmt' => '2014-10-12 12:42:23',
  'post_content' => '<div id="lipsum">

Aenean molestie nibh dolor, ut imperdiet felis commodo eu. Vestibulum tristique, lorem a pellentesque tempor, neque sapien congue nulla, in scelerisque leo magna sit amet mauris. Praesent accumsan condimentum ligula ut efficitur. Curabitur sagittis, eros non ullamcorper imperdiet, sem augue mattis sapien, sit amet aliquam lorem nisi in augue. Integer maximus pulvinar mauris, quis pharetra leo consectetur id. Aliquam pulvinar sed dolor a pulvinar. Aliquam erat volutpat.

</div>
<!--more-->

Nullam tincidunt in urna eu gravida. Nulla erat lacus, tincidunt sed ante eu, suscipit tempus lacus. Aenean fermentum, arcu vitae aliquam consequat, justo nisl vehicula urna, a porta mauris enim ut mauris. Nullam tortor lorem, efficitur et libero eget, accumsan malesuada nisl. Donec tristique vel enim non placerat. Nullam fringilla erat mattis imperdiet consectetur. Mauris sit amet magna rutrum metus consequat efficitur. Mauris quam nisi, ultrices pulvinar commodo ut, vestibulum a elit. Integer vel posuere elit. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Fusce convallis ex quis dui viverra, quis luctus augue commodo. Morbi convallis purus eget augue congue cursus. Pellentesque nisi urna, interdum id erat ut, aliquam tempor tellus. Integer maximus neque sem. Ut libero ex, efficitur in nibh in, gravida venenatis odio.

&nbsp;

&nbsp;',
  'post_title' => '7 Column Gallery',
  'post_excerpt' => 'Masonry gallery in 7 columns',
  'post_name' => '7-column-gallery',
  'post_modified' => '2017-08-21 05:52:24',
  'post_modified_gmt' => '2017-08-21 05:52:24',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?post_type=portfolio&#038;p=3207',
  'menu_order' => 0,
  'post_type' => 'portfolio',
  'meta_input' => 
  array (
    'gallery_shortcode' => '[gallery columns="7" size="medium" ids="3030,3027,3073,2980,2879,2366,2414,2963,2964,2962,2967,2968,2974,2975,2977,2978,2984,2982,2981,2979,2986,2990,2997,2996,2994,2992,2998,3006,3005,3011,3020,3023,3032,3028,3036,3053,3057" orderby="rand"]',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'portfolio-category' => 'designers',
  ),
  'thumb' => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
  'gallery_shortcode' => 
  array (
    'gallery_shortcode' => 
    array (
      0 => 'https://themify.me/demo/themes/elegant/files/2014/10/165191423.jpg',
      1 => 'https://themify.me/demo/themes/elegant/files/2014/10/179331083.jpg',
      2 => 'https://themify.me/demo/themes/elegant/files/2014/10/164096684.jpg',
      3 => 'https://themify.me/demo/themes/elegant/files/2014/10/185056664.jpg',
      4 => 'https://themify.me/demo/themes/elegant/files/2014/10/164299958.jpg',
      5 => 'https://themify.me/demo/themes/elegant/files/2014/10/192449156.jpg',
      6 => 'https://themify.me/demo/themes/elegant/files/2014/10/165420677.jpg',
      7 => 'https://themify.me/demo/themes/elegant/files/2014/10/162044285.jpg',
      8 => 'https://themify.me/demo/themes/elegant/files/2014/10/173247353.jpg',
      9 => 'https://themify.me/demo/themes/elegant/files/2014/10/114041515.jpg',
      10 => 'https://themify.me/demo/themes/elegant/files/2014/10/183592982.jpg',
      11 => 'https://themify.me/demo/themes/elegant/files/2014/10/170155304.jpg',
      12 => 'https://themify.me/demo/themes/elegant/files/2014/10/181161062.jpg',
      13 => 'https://themify.me/demo/themes/elegant/files/2014/10/194739149.jpg',
      14 => 'https://themify.me/demo/themes/elegant/files/2014/10/163983824.jpg',
      15 => 'https://themify.me/demo/themes/elegant/files/2014/10/169284398.jpg',
      16 => 'https://themify.me/demo/themes/elegant/files/2014/10/164404166.jpg',
      17 => 'https://themify.me/demo/themes/elegant/files/2014/10/184335731.jpg',
      18 => 'https://themify.me/demo/themes/elegant/files/2014/10/134119568.jpg',
      19 => 'https://themify.me/demo/themes/elegant/files/2014/10/demo_peter.jpg',
      20 => 'https://themify.me/demo/themes/elegant/files/2014/10/159543098.jpg',
      21 => 'https://themify.me/demo/themes/elegant/files/2014/10/158788109.jpg',
      22 => 'https://themify.me/demo/themes/elegant/files/2014/10/161674283.jpg',
      23 => 'https://themify.me/demo/themes/elegant/files/2014/10/1988833431.jpg',
      24 => 'https://themify.me/demo/themes/elegant/files/2014/10/164961167.jpg',
      25 => 'https://themify.me/demo/themes/elegant/files/2014/10/171475754.jpg',
      26 => 'https://themify.me/demo/themes/elegant/files/2014/10/159045947.jpg',
      27 => 'https://themify.me/demo/themes/elegant/files/2014/10/167123411.jpg',
      28 => 'https://themify.me/demo/themes/elegant/files/2014/10/160533962.jpg',
      29 => 'https://themify.me/demo/themes/elegant/files/2014/10/183281279.jpg',
      30 => 'https://themify.me/demo/themes/elegant/files/2014/10/183178742.jpg',
      31 => 'https://themify.me/demo/themes/elegant/files/2014/10/1688089642.jpg',
      32 => 'https://themify.me/demo/themes/elegant/files/2014/10/173750906.jpg',
      33 => 'https://themify.me/demo/themes/elegant/files/2014/10/176083367.jpg',
      34 => 'https://themify.me/demo/themes/elegant/files/2014/10/160748159.jpg',
      35 => 'https://themify.me/demo/themes/elegant/files/2014/10/163372622.jpg',
      36 => 'https://themify.me/demo/themes/elegant/files/2014/10/167678762.jpg',
    ),
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4401,
  'post_date' => '2016-03-25 21:03:27',
  'post_date_gmt' => '2016-03-25 21:03:27',
  'post_content' => '',
  'post_title' => 'Row 1',
  'post_excerpt' => '',
  'post_name' => 'row-1',
  'post_modified' => '2016-03-25 21:03:27',
  'post_modified_gmt' => '2016-03-25 21:03:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4401',
  'menu_order' => 1,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'custom',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '4401',
    '_menu_item_object' => 'custom',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
    '_menu_item_url' => '#test',
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'test',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4460,
  'post_date' => '2019-04-09 02:36:28',
  'post_date_gmt' => '2019-04-09 02:36:28',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4460',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4460',
  'menu_order' => 1,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '3238',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4402,
  'post_date' => '2016-03-25 21:03:27',
  'post_date_gmt' => '2016-03-25 21:03:27',
  'post_content' => '',
  'post_title' => 'Row 2',
  'post_excerpt' => '',
  'post_name' => 'row-2',
  'post_modified' => '2016-03-25 21:03:27',
  'post_modified_gmt' => '2016-03-25 21:03:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4402',
  'menu_order' => 2,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'custom',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '4402',
    '_menu_item_object' => 'custom',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
    '_menu_item_url' => '#test2',
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'test',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4461,
  'post_date' => '2019-04-09 02:36:28',
  'post_date_gmt' => '2019-04-09 02:36:28',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4461',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4461',
  'menu_order' => 2,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4460',
    '_menu_item_object_id' => '3231',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4403,
  'post_date' => '2016-03-25 21:03:27',
  'post_date_gmt' => '2016-03-25 21:03:27',
  'post_content' => '',
  'post_title' => 'Row 3',
  'post_excerpt' => '',
  'post_name' => 'row-3',
  'post_modified' => '2016-03-25 21:03:27',
  'post_modified_gmt' => '2016-03-25 21:03:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4403',
  'menu_order' => 3,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'custom',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '4403',
    '_menu_item_object' => 'custom',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
    '_menu_item_url' => '#test3',
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'test',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4462,
  'post_date' => '2019-04-09 02:36:28',
  'post_date_gmt' => '2019-04-09 02:36:28',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4462',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4462',
  'menu_order' => 3,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4460',
    '_menu_item_object_id' => '2406',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4404,
  'post_date' => '2016-03-25 21:03:27',
  'post_date_gmt' => '2016-03-25 21:03:27',
  'post_content' => '',
  'post_title' => 'Row 4',
  'post_excerpt' => '',
  'post_name' => 'row-4',
  'post_modified' => '2016-03-25 21:03:27',
  'post_modified_gmt' => '2016-03-25 21:03:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4404',
  'menu_order' => 4,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'custom',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '4404',
    '_menu_item_object' => 'custom',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
    '_menu_item_url' => '#test4',
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'test',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4463,
  'post_date' => '2019-04-09 02:36:28',
  'post_date_gmt' => '2019-04-09 02:36:28',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4463',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4463',
  'menu_order' => 4,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4460',
    '_menu_item_object_id' => '3234',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4464,
  'post_date' => '2019-04-09 02:36:28',
  'post_date_gmt' => '2019-04-09 02:36:28',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4464',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 3238,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4464',
  'menu_order' => 5,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4460',
    '_menu_item_object_id' => '2431',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4465,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4465',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4465',
  'menu_order' => 6,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '2376',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4466,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4466',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4466',
  'menu_order' => 7,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4465',
    '_menu_item_object_id' => '3209',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4467,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4467',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4467',
  'menu_order' => 8,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4465',
    '_menu_item_object_id' => '3176',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4468,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4468',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4468',
  'menu_order' => 9,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4465',
    '_menu_item_object_id' => '3170',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4469,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4469',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4469',
  'menu_order' => 10,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4465',
    '_menu_item_object_id' => '3278',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4470,
  'post_date' => '2019-04-09 02:54:29',
  'post_date_gmt' => '2019-04-09 02:54:29',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4470',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2376,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4470',
  'menu_order' => 11,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4465',
    '_menu_item_object_id' => '3213',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4471,
  'post_date' => '2019-04-09 02:54:48',
  'post_date_gmt' => '2019-04-09 02:54:48',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4471',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4471',
  'menu_order' => 12,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '2380',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4472,
  'post_date' => '2019-04-09 02:54:48',
  'post_date_gmt' => '2019-04-09 02:54:48',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4472',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4472',
  'menu_order' => 13,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4471',
    '_menu_item_object_id' => '3269',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4473,
  'post_date' => '2019-04-09 02:54:48',
  'post_date_gmt' => '2019-04-09 02:54:48',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4473',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4473',
  'menu_order' => 14,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4471',
    '_menu_item_object_id' => '3276',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4474,
  'post_date' => '2019-04-09 02:54:48',
  'post_date_gmt' => '2019-04-09 02:54:48',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4474',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2380,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4474',
  'menu_order' => 15,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4471',
    '_menu_item_object_id' => '3273',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4475,
  'post_date' => '2019-04-09 02:55:41',
  'post_date_gmt' => '2019-04-09 02:55:41',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4475',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4475',
  'menu_order' => 16,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '2382',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4476,
  'post_date' => '2019-04-09 02:55:41',
  'post_date_gmt' => '2019-04-09 02:55:41',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4476',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4476',
  'menu_order' => 17,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '0',
    '_menu_item_object_id' => '2353',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4477,
  'post_date' => '2019-04-09 02:55:41',
  'post_date_gmt' => '2019-04-09 02:55:41',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4477',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4477',
  'menu_order' => 18,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4476',
    '_menu_item_object_id' => '2354',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4478,
  'post_date' => '2019-04-09 02:55:41',
  'post_date_gmt' => '2019-04-09 02:55:41',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4478',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4478',
  'menu_order' => 19,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4476',
    '_menu_item_object_id' => '2355',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 4479,
  'post_date' => '2019-04-09 02:55:41',
  'post_date_gmt' => '2019-04-09 02:55:41',
  'post_content' => ' ',
  'post_title' => '',
  'post_excerpt' => '',
  'post_name' => '4479',
  'post_modified' => '2019-04-09 02:55:41',
  'post_modified_gmt' => '2019-04-09 02:55:41',
  'post_content_filtered' => '',
  'post_parent' => 2353,
  'guid' => 'https://themify.me/demo/themes/elegant/?p=4479',
  'menu_order' => 20,
  'post_type' => 'nav_menu_item',
  'meta_input' => 
  array (
    '_menu_item_type' => 'post_type',
    '_menu_item_menu_item_parent' => '4476',
    '_menu_item_object_id' => '2356',
    '_menu_item_object' => 'page',
    '_menu_item_classes' => 
    array (
      0 => '',
    ),
  ),
  'tax_input' => 
  array (
    'nav_menu' => 'main-navigation',
  ),
);
themify_process_post_import( $post );



function themify_import_get_term_id_from_slug( $slug ) {
	$menu = get_term_by( "slug", $slug, "nav_menu" );
	return is_wp_error( $menu ) ? 0 : (int) $menu->term_id;
}

	$widgets = get_option( "widget_themify-twitter" );
$widgets[1002] = array (
  'title' => 'Latest Tweets',
  'username' => 'themify',
  'show_count' => '3',
  'hide_timestamp' => NULL,
  'show_follow' => 'on',
  'follow_text' => '→ Follow me',
  'include_retweets' => 'on',
  'exclude_replies' => NULL,
);
update_option( "widget_themify-twitter", $widgets );

$widgets = get_option( "widget_themify-feature-posts" );
$widgets[1003] = array (
  'title' => 'Recent Posts',
  'category' => '0',
  'show_count' => '3',
  'show_date' => 'on',
  'show_thumb' => 'on',
  'display' => 'none',
  'hide_title' => NULL,
  'thumb_width' => '50',
  'thumb_height' => '50',
  'excerpt_length' => '55',
  'orderby' => 'date',
  'order' => 'DESC',
);
update_option( "widget_themify-feature-posts", $widgets );

$widgets = get_option( "widget_themify-recent-comments" );
$widgets[1004] = array (
  'title' => 'Recent Comments',
  'show_count' => '2',
  'show_avatar' => 'on',
  'avatar_size' => '55',
  'excerpt_length' => '60',
);
update_option( "widget_themify-recent-comments", $widgets );

$widgets = get_option( "widget_themify-social-links" );
$widgets[1005] = array (
  'title' => '',
  'show_link_name' => NULL,
  'open_new_window' => NULL,
  'icon_size' => 'icon-large',
  'orientation' => 'horizontal',
);
update_option( "widget_themify-social-links", $widgets );

$widgets = get_option( "widget_themify-social-links" );
$widgets[1006] = array (
  'title' => '',
  'show_link_name' => NULL,
  'open_new_window' => NULL,
  'icon_size' => 'icon-large',
  'orientation' => 'horizontal',
);
update_option( "widget_themify-social-links", $widgets );



$sidebars_widgets = array (
  'sidebar-main' => 
  array (
    0 => 'themify-twitter-1002',
    1 => 'themify-feature-posts-1003',
    2 => 'themify-recent-comments-1004',
  ),
  'social-widget' => 
  array (
    0 => 'themify-social-links-1005',
  ),
  'footer-social-widget' => 
  array (
    0 => 'themify-social-links-1006',
  ),
); 
update_option( "sidebars_widgets", $sidebars_widgets );

$menu_locations = array();
$menu = get_terms( "nav_menu", array( "slug" => "main-navigation" ) );
if( is_array( $menu ) && ! empty( $menu ) ) $menu_locations["main-nav"] = $menu[0]->term_id;
set_theme_mod( "nav_menu_locations", $menu_locations );


$homepage = get_posts( array( 'name' => 'blog', 'post_type' => 'page' ) );
			if( is_array( $homepage ) && ! empty( $homepage ) ) {
				update_option( 'show_on_front', 'page' );
				update_option( 'page_on_front', $homepage[0]->ID );
			}
			
	ob_start(); ?>a:74:{s:31:"setting-search_settings_exclude";s:2:"on";s:21:"setting-webfonts_list";s:11:"recommended";s:22:"setting-default_layout";s:12:"sidebar-none";s:27:"setting-default_post_layout";s:9:"list-post";s:30:"setting-default_layout_display";s:7:"content";s:25:"setting-default_more_text";s:4:"More";s:21:"setting-index_orderby";s:4:"date";s:19:"setting-index_order";s:4:"DESC";s:33:"setting-default_post_meta_comment";s:2:"no";s:31:"setting-image_post_feature_size";s:5:"blank";s:32:"setting-default_page_post_layout";s:12:"sidebar-none";s:44:"setting-default_page_post_content_text_align";s:6:"center";s:38:"setting-image_post_single_feature_size";s:5:"blank";s:27:"setting-default_page_layout";s:12:"sidebar-none";s:38:"setting-default_portfolio_index_layout";s:12:"sidebar-none";s:43:"setting-default_portfolio_index_post_layout";s:5:"grid4";s:39:"setting-default_portfolio_index_display";s:4:"none";s:41:"setting-default_portfolio_index_post_date";s:3:"yes";s:42:"setting-default_portfolio_index_post_image";s:2:"no";s:22:"themify_portfolio_slug";s:7:"project";s:34:"setting-default_team_single_layout";s:12:"sidebar-none";s:17:"themify_team_slug";s:4:"team";s:53:"setting-customizer_responsive_design_tablet_landscape";s:4:"1024";s:43:"setting-customizer_responsive_design_tablet";s:3:"768";s:43:"setting-customizer_responsive_design_mobile";s:3:"480";s:33:"setting-mobile_menu_trigger_point";s:4:"1200";s:24:"setting-gallery_lightbox";s:8:"lightbox";s:27:"setting-page_builder_expiry";s:1:"2";s:19:"setting-entries_nav";s:8:"numbered";s:27:"setting-imagefilter_applyto";s:12:"featuredonly";s:22:"setting-footer_widgets";s:17:"footerwidget-3col";s:25:"setting-img_php_base_size";s:5:"large";s:27:"setting-global_feature_size";s:5:"blank";s:22:"setting-link_icon_type";s:9:"font-icon";s:32:"setting-link_type_themify-link-0";s:10:"image-icon";s:33:"setting-link_title_themify-link-0";s:7:"Twitter";s:31:"setting-link_img_themify-link-0";s:91:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/images/twitter.png";s:32:"setting-link_type_themify-link-1";s:10:"image-icon";s:33:"setting-link_title_themify-link-1";s:8:"Facebook";s:31:"setting-link_img_themify-link-1";s:92:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/images/facebook.png";s:32:"setting-link_type_themify-link-2";s:10:"image-icon";s:33:"setting-link_title_themify-link-2";s:6:"Google";s:31:"setting-link_img_themify-link-2";s:95:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/images/google-plus.png";s:32:"setting-link_type_themify-link-3";s:10:"image-icon";s:33:"setting-link_title_themify-link-3";s:7:"YouTube";s:31:"setting-link_img_themify-link-3";s:91:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/images/youtube.png";s:32:"setting-link_type_themify-link-4";s:10:"image-icon";s:33:"setting-link_title_themify-link-4";s:9:"Pinterest";s:31:"setting-link_img_themify-link-4";s:93:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/images/pinterest.png";s:32:"setting-link_type_themify-link-6";s:9:"font-icon";s:33:"setting-link_title_themify-link-6";s:8:"Facebook";s:32:"setting-link_link_themify-link-6";s:27:"http://facebook.com/themify";s:33:"setting-link_ficon_themify-link-6";s:11:"fa-facebook";s:32:"setting-link_type_themify-link-5";s:9:"font-icon";s:33:"setting-link_title_themify-link-5";s:7:"Twitter";s:32:"setting-link_link_themify-link-5";s:26:"http://twitter.com/themify";s:33:"setting-link_ficon_themify-link-5";s:10:"fa-twitter";s:32:"setting-link_type_themify-link-7";s:9:"font-icon";s:33:"setting-link_title_themify-link-7";s:6:"Google";s:32:"setting-link_link_themify-link-7";s:73:"https://plus.google.com/u/2/b/109280316400365629341/109280316400365629341";s:33:"setting-link_ficon_themify-link-7";s:14:"fa-google-plus";s:32:"setting-link_type_themify-link-8";s:9:"font-icon";s:33:"setting-link_title_themify-link-8";s:7:"YouTube";s:32:"setting-link_link_themify-link-8";s:37:"http://www.youtube.com/user/themifyme";s:33:"setting-link_ficon_themify-link-8";s:10:"fa-youtube";s:32:"setting-link_type_themify-link-9";s:9:"font-icon";s:33:"setting-link_title_themify-link-9";s:9:"Pinterest";s:32:"setting-link_link_themify-link-9";s:26:"https://www.pinterest.com/";s:33:"setting-link_ficon_themify-link-9";s:12:"fa-pinterest";s:22:"setting-link_field_ids";s:341:"{"themify-link-0":"themify-link-0","themify-link-1":"themify-link-1","themify-link-2":"themify-link-2","themify-link-3":"themify-link-3","themify-link-4":"themify-link-4","themify-link-6":"themify-link-6","themify-link-5":"themify-link-5","themify-link-7":"themify-link-7","themify-link-8":"themify-link-8","themify-link-9":"themify-link-9"}";s:23:"setting-link_field_hash";s:2:"10";s:30:"setting-page_builder_is_active";s:6:"enable";s:46:"setting-page_builder_animation_parallax_scroll";s:6:"mobile";s:4:"skin";s:97:"https://themify.me/demo/themes/elegant/wp-content/themes/themify-elegant/themify/img/non-skin.gif";}<?php $themify_data = unserialize( ob_get_clean() );

	// fix the weird way "skin" is saved
	if( isset( $themify_data['skin'] ) ) {
		$parsed_skin = parse_url( $themify_data['skin'], PHP_URL_PATH );
		$basedir_skin = basename( dirname( $parsed_skin ) );
		$themify_data['skin'] = trailingslashit( get_template_directory_uri() ) . 'skins/' . $basedir_skin . '/style.css';
	}

	themify_set_data( $themify_data );
	
}
themify_do_demo_import();